/* ***** BEGIN LICENCE BLOCK *****
 * Version: CePL 1.1
 * 
 * The contents of this file are subject to the Celtx Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.celtx.com/CePL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Celtx Script Manager.
 * 
 * The Initial Developer of the Original Code is 4067479 Canada Inc.
 * t/a CELTX.
 * 
 * Portions created by Celtx are Copyright (C) 4067479 Canada Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENCE BLOCK ***** */

var gWindow;
var gSelectedScene;


function loaded () {
  gWindow = new Object;
  gWindow.deck      = document.getElementById("scriptnavdeck");
  gWindow.sceneTree = document.getElementById("scene-tree");
  gWindow.reportTree    = document.getElementById("report-tree");
}


function open (project, docres) {
  gWindow.ds = project.ds;
  gWindow.sceneTree.database.AddDataSource(gWindow.ds);
  // gWindow.sceneTree.ref = docres.Value;
  var rdfsvc = getRDFService();
  var scenesarc = rdfsvc.GetResource(Cx.NS_CX + "scenes");
  var scenes = gWindow.ds.GetTarget(docres, scenesarc, true);
  if (! scenes) {
    scenes = rdfsvc.GetAnonymousResource();
    gWindow.ds.Assert(docres, scenesarc, scenes, true);
    getRDFContainerUtils().MakeSeq(gWindow.ds, scenes);
  }
  scenes = scenes.QueryInterface(Components.interfaces.nsIRDFResource);
  gWindow.sceneTree.ref = scenes.Value;
  gWindow.sceneTreeView = new SceneTreeView(gWindow.sceneTree);
  gWindow.sceneTree.builderView.addObserver(gWindow.sceneTreeView);
}


function close () {
  gWindow.sceneTree.builderView.removeObserver(gWindow.sceneTreeView);
  gWindow.sceneTree.database.RemoveDataSource(gWindow.ds);
}


// The controller from the primary script window
function setDelegate (delegate) {
  gWindow.delegate = delegate;
}


function showSceneNav () {
  gWindow.deck.selectedIndex = 0;
}


function showReportNav () {
  gWindow.deck.selectedIndex = 1;
}


function selectReportIndex (index) {
  gWindow.reportTree.view.selection.select(0);
}


function getSelectedReportIndex () {
  return gWindow.reportTree.view.selection.currentIndex;
}


function getSelectedReportName () {
  var idx = getSelectedReportIndex();
  var cols = gWindow.reportTree.columns;
  return gWindow.reportTree.view.getCellText(idx, cols.getFirstColumn());
}


function getSelectedScene () {
  var treeview = gWindow.sceneTree.view;
  var idx = treeview.selection.currentIndex;
  return isSceneRow(idx) ? treeview.getResourceAtIndex(idx) : null;
}


function getSelectedShot () {
  var treeview = gWindow.sceneTree.view;
  var idx = treeview.selection.currentIndex;
  return isShotRow(idx) ? treeview.getResourceAtIndex(idx) : null;
}


function getSelectedSceneID () {
  var idx = gWindow.sceneTree.view.selection.currentIndex;
  return isSceneRow(idx) ? getSelectedID() : null;
}

function getSelectedID () {
  var idx = gWindow.sceneTree.view.selection.currentIndex;
  if (idx < 0)
    return;
  var rdfsvc = getRDFService();
  var sceneidarc = rdfsvc.GetResource(Cx.NS_CX + "sceneid");
  var shotidarc = rdfsvc.GetResource(Cx.NS_CX + "shotid");
  var res = gWindow.sceneTree.builderView.getResourceAtIndex(idx);
  var idstr = getRDFString(gWindow.ds, res, sceneidarc);
  if (idstr)
    return idstr;
  idstr = getRDFString(gWindow.ds, res, shotidarc);
  return idstr;
}


function selectScene (sceneres) {
  var view = gWindow.sceneTree.view;
  var rdfsvc = getRDFService();
  var index = view.getIndexOfResource(sceneres);
  if (index >= 0)
    view.selection.select(index);
}


function selectSceneAtIndex (index) {
  gWindow.sceneTree.view.selection.select(index);
}


function onSceneSelect (event) {
  updateTreeCommands();
  if (gWindow.delegate) {
    var idx = gWindow.sceneTree.view.selection.currentIndex;
    if (idx < 0)
      return;
    if (isSceneRow(idx))
      gWindow.delegate.onSceneSelect(event);
    else if (isShotRow(idx))
      gWindow.delegate.onShotSelect(event);
  }
}


function updateTreeCommands () {
  goUpdateCommand("cmd-treeitem-down");
  goUpdateCommand("cmd-treeitem-up");
  goUpdateCommand("cmd-treeitem-delete");
}


function isSceneRow (idx) {
  if (idx < 0)
    return false;
  var rdfsvc = getRDFService();
  var sceneidarc = rdfsvc.GetResource(Cx.NS_CX + "sceneid");
  var res = gWindow.sceneTree.builderView.getResourceAtIndex(idx);
  if (! res)
    return false;
  var sceneid = gWindow.ds.GetTarget(res, sceneidarc, true);
  return sceneid != null;
}


function isShotRow (idx) {
  if (idx < 0)
    return false;
  var rdfsvc = getRDFService();
  var shotidarc = rdfsvc.GetResource(Cx.NS_CX + "shotid");
  var res = gWindow.sceneTree.builderView.getResourceAtIndex(idx);
  if (! res)
    return false;
  var shotid = gWindow.ds.GetTarget(res, shotidarc, true);
  return shotid != null;
}


function SceneTreeView (tree) {
  this.tree = tree;
  this.dragsvc = Components.classes["@mozilla.org/widget/dragservice;1"]
    .getService(Components.interfaces.nsIDragService);
}


SceneTreeView.prototype = {
  QueryInterface: function (iid) {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsIXULTreeBuilderObserver))
      return true;
    throw Components.results.NS_NOINTERFACE;
  },


  DROP_BEFORE: Components.interfaces.nsIXULTreeBuilderObserver.DROP_BEFORE,
  DROP_ON: Components.interfaces.nsIXULTreeBuilderObserver.DROP_ON,
  DROP_AFTER: Components.interfaces.nsIXULTreeBuilderObserver.DROP_AFTER,


  sceneFlavour:  "x-celtx/x-sceneid",
  shotFlavour:      "x-celtx/x-shotid",


  canDrop: function (index, orientation) {
    if (! gWindow.delegate)
      return false;

    var dragSession = this.dragsvc.getCurrentSession();
    if (! dragSession)
      return false;

    if (dragSession.numDropItems != 1)
      return false;

    if (dragSession.isDataFlavorSupported(this.sceneFlavour)) {
      if (orientation == this.DROP_ON) {
        return false;
      }
      else {
        return isSceneRow(index);
      }
    }
    else if (dragSession.isDataFlavorSupported(this.shotFlavour)) {
      if (orientation == this.DROP_ON) {
        return isSceneRow(index);
      }
      else {
        return isShotRow(index);
      }
    }

    return false;
  },


  onCycleCell: function (row, colid) {},
  onCycleHeader: function (colid, elt) {},


  onDrop: function (row, orientation) {
    if (! gWindow.delegate) {
      dump("*** onDrop: no delegate\n");
      return;
    }

    var dragSession = this.dragsvc.getCurrentSession();
    if (! dragSession) {
      dump("*** onDrop: no dragSession\n");
      return;
    }

    if (dragSession.numDropItems != 1) {
      dump("*** onDrop: bad numDropItems or flavour not supported\n");
      return;
    }

    try {
      var trans = Components.classes["@mozilla.org/widget/transferable;1"]
        .createInstance(Components.interfaces.nsITransferable);
      var flavour = null;
      if (dragSession.isDataFlavorSupported(this.sceneFlavour)) {
        flavour = this.sceneFlavour;
      }
      else if (dragSession.isDataFlavorSupported(this.shotFlavour)) {
        flavour = this.shotFlavour;
      }
      else {
        dump("*** onDrop: no data for supported flavours\n");
        return;
      }
      trans.addDataFlavor(flavour);
      dragSession.getData(trans, 0);
      var data = {};
      var len = {};
      trans.getTransferData(flavour, data, len);
      data = data.value.QueryInterface(Components.interfaces.nsISupportsString);
      if (! data) {
        dump("*** onDrop: no data\n");
        return;
      }
      data = data.data.substring(0, len.value);

      var treeview = gWindow.sceneTree.view;
      var rdfsvc = getRDFService();
      var sceneidarc = rdfsvc.GetResource(Cx.NS_CX + "sceneid");
      var shotidarc = rdfsvc.GetResource(Cx.NS_CX + "shotid");
      var res = rdfsvc.GetResource(data);
      var srcrow = treeview.getIndexOfResource(res);
      if (flavour == this.sceneFlavour) {
        if (row == srcrow)
          return;
        var id = getRDFString(gWindow.ds, res, sceneidarc);
        var scenes = rdfsvc.GetResource(gWindow.sceneTree.getAttribute("ref"));
        scenes = new RDFSeq(gWindow.ds, scenes);
        var dstpos = scenes.indexOf(treeview.getResourceAtIndex(row));
        if (dstpos < 0)
          return;
        if (orientation == this.DROP_AFTER)
          ++dstpos;
        // Scene positions are 1-based
        gWindow.delegate.moveSceneToIndex(id, dstpos + 1);
      }
      else {
        var id = getRDFString(gWindow.ds, res, shotidarc);
        var dstseq = null;
        var dstpos = null;
        if (isShotRow(row)) {
          var seqidx = treeview.getParentIndex(row);
          dstseq = gWindow.sceneTree.view.getResourceAtIndex(seqidx);
          dstseq = new RDFSeq(gWindow.ds, dstseq);
          // If the drop is on a shot, we know the scene row is open
          dstpos = row - seqidx - 1;
          if (orientation == this.DROP_AFTER)
            ++dstpos;
          // var srcpos = dstseq.indexOf(res);
          // if (srcpos >= 0 && srcpos < dstpos)
          //   --dstpos;
        }
        else {
          dstseq = gWindow.sceneTree.view.getResourceAtIndex(row);
          dstseq = new RDFSeq(gWindow.ds, dstseq);
          dstpos = dstseq.length;
        }
        var seqid = getRDFString(gWindow.ds, dstseq.res, sceneidarc);
        // Shot positions are 1-based
        gWindow.delegate.moveShotTo(id, seqid, dstpos + 1);
      }
    }
    catch (ex) {
      dump("*** onDrop: " + ex + "\n");
    }
  },


  onPerformAction: function (action)                {},
  onPerformActionOnCell: function (action, row, colid) {},
  onPerformActionOnRow: function (action, row)      {},
  onSelectionChanged: function ()                   {},
  onToggleOpenState: function (index) {},


  dragGesture: function dragGesture (event) {
    if (event.originalTarget.localName == "treechildren")
      nsDragAndDrop.startDrag(event, this);
  },


  onDragStart: function onDragStart (aEvent, aXferData, aDragAction) {
    try {
      var selection = this.tree.view.selection;
      if (selection.count != 1) {
        dump("*** onDragStart: multi-select, aborting\n");
        return;
      }
      var index = selection.currentIndex;
      var res = this.tree.view.getResourceAtIndex(index);
      if (! res)
        return;

      // var rdfsvc = getRDFService();
      // var sceneidarc = rdfsvc.GetResource(Cx.NS_CX + "sceneid");
      // var shotidarc = rdfsvc.GetResource(Cx.NS_CX + "shotid");
      var ds = this.tree.database;
      var flavour = null;
      // var id = null;
      if (isSceneRow(index)) {
        flavour = this.sceneFlavour;
        // id = getRDFString(ds, res, sceneidarc);
      }
      else if (isShotRow(index)) {
        flavour = this.shotFlavour;
        // id = getRDFString(ds, res, shotidarc);
      }
      else {
        dump("*** row dragged that isn't a scene or a shot\n");
        return;
      }
      var data = new TransferData();
      // data.addDataForFlavour(flavour, id);
      data.addDataForFlavour(flavour, res.Value);
      aXferData.data = data;
    }
    catch (ex) {
      dump("*** onDragStart: " + ex + "\n");
    }
  }
};
