/* ***** BEGIN LICENCE BLOCK *****
 * Version: CePL 1.1
 * 
 * The contents of this file are subject to the Celtx Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.celtx.com/CePL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Celtx Script Manager.
 * 
 * The Initial Developer of the Original Code is Chad House and 4067479
 * Canada Inc. t/a CELTX.
 * 
 * Portions created by Chad House are Copyright (C) 2000-2004 Chad House,
 * parts created by Celtx are Copyright (C) 4067479 Canada Inc. All Rights
 * Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENCE BLOCK ***** */

var gWindow;

function loaded () {
  gWindow = new Object;

  gWindow.scenecount      = document.getElementById("scenecount");
  gWindow.charcount       = document.getElementById("charcount");
  gWindow.navdeck         = document.getElementById("navdeck");
  gWindow.headerdeck      = document.getElementById("headerdeck");
  gWindow.detailsdeck     = document.getElementById("detailsdeck");
  gWindow.scenetree       = document.getElementById("scenelist");
  gWindow.locationlist    = document.getElementById("locationlist");
  gWindow.slugline        = document.getElementById("slugline");
  gWindow.intextmenu      = document.getElementById("intextmenu");
  gWindow.intextpopup     = document.getElementById("intextpopup");
  gWindow.daynightmenu    = document.getElementById("daynightmenu");
  gWindow.daynightpopup   = document.getElementById("daynightpopup");
  gWindow.locationmenu    = document.getElementById("locationmenu");
  gWindow.locationpopup   = document.getElementById("locationpopup");
  gWindow.monthview       = document.getElementById("schedulemonthview");
  gWindow.weekview        = document.getElementById("scheduleweekview");
  gWindow.dayview         = document.getElementById("scheduledayview");
  gWindow.scheduledeck    = document.getElementById("scheduleviewdeck");

  window.controllers.appendController(gController);

  gScriptController.loaded();
  gBreakdownController.loaded();
  gLocationController.loaded();
  gDeptController.loaded();
  gScheduleController.loaded();

  // gWindow.navdeck.hidden = true;
}


function onEnterPrintPreview () {
  gWindow.toolbar.hidden = true;
  gWindow.inPrintPreview = true;
}


function onExitPrintPreview () {
  gWindow.toolbar.hidden = false;
  gWindow.inPrintPreview = false;
}


function getController () {
  return gController;
}


var gDummyController = {
  supportsCommand: function (cmd) { return false; },
  isCommandEnabled: function (cmd) { return false; },
  doCommand: function (cmd) {}
};


var gController = {
  QueryInterface: function (iid) {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsIDOMEventListener) ||
        iid.equals(Components.interfaces.nsIRDFObserver))
      return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  openwindows: { findreplace: null, spellcheck: null },


  get currentView () {
    var panel = gWindow.detailsdeck.selectedPanel;
    return panel.id.replace(/card$/, "");
  },


  controllerForView: function controllerForView (view) {
    switch (view) {
      case "script":
        return gScriptController;
      case "breakdown":
        return gBreakdownController;
      case "location":
        return gLocationController;
      case "department":
        return gDeptController;
      case "schedule":
        return gScheduleController;
      default:
        return gDummyController;
    }
  },


  commands: {
    "cmd-page-setup": 1,
    "cmd-print": 1,
    "cmd-print-preview": 1
  },


  supportsCommand: function (cmd) {
    if (this.commands[cmd] == 1)
      return true;
    var ctrl = this.controllerForView(this.currentView);
    if ("supportsCommand" in ctrl)
      return ctrl.supportsCommand(cmd);
    return false;
  },


  isCommandEnabled: function (cmd) {
    if (cmd == "cmd-page-setup")
      return true;
    var ctrl = this.controllerForView(this.currentView);
    if ("isCommandEnabled" in ctrl)
      return (ctrl.supportsCommand(cmd) && ctrl.isCommandEnabled(cmd));
    return false;
  },


  doCommand: function (cmd) {
    if (cmd == "cmd-page-setup") {
      PrintUtils.showPageSetup();
      return;
    }
    var ctrl = this.controllerForView(this.currentView);
    if ("supportsCommand" in ctrl &&
        ctrl.supportsCommand(cmd) &&
        ctrl.isCommandEnabled(cmd))
      ctrl.doCommand(cmd);
  },


  modificationCount: 0,


  get modified () {
    return this.modificationCount > 0;
  },


  open: function open (project, docres) {
    this.controllers = [
      gScriptController, gBreakdownController, gLocationController,
      gDeptController, gScheduleController
    ];
    this.project = project;
    this.docres = docres;

    // Load the user's department preferences now, so that it's available
    // on the next event cycle.
    var rdfsvc = getRDFService();
    var rdfsrc = currentProfileDir();
    rdfsrc.append(Cx.PREFS_FILE);
    this.deptds = rdfsvc.GetDataSourceBlocking(fileToFileURL(rdfsrc));

    // Load the schema too (this is probably already cached, but what the heck)
    this.schemads = rdfsvc.GetDataSourceBlocking(Cx.SCHEMA_URL);

    var file = this.project.fileForResource(docres);
    if (isReadableFile(file)) {
      this.bdds = rdfsvc.GetDataSourceBlocking(fileToFileURL(file));
      window.setTimeout("gController.loadBreakdown()", 0);
    }
    else {
      var progressController = {
        owner: this,
        canceled: false,
        finished: false,
        bogusTaskCompleted: false,
        get progress () {
          var base = 0;
          if (this.bogusTaskCompleted) base += 20;
          return base;
        },
        get message () {
          return gApp.getText("GeneratingReportMsg");
        },
        performNextTask: function () {
          if (! this.bogusTaskCompleted) {
            this.bogusTaskCompleted = true;
            return true;
          }
          if (! this.finished) {
            this.owner.cmdImportBreakdown();
            this.finished = true;
          }
        },
        abort: function () {
          this.canceled = true;
        }
      };
      window.openDialog(Cx.CONTENT_PATH + "progress.xul", "_blank",
                        Cx.MODAL_DIALOG_FLAGS, progressController);
      if (progressController.canceled)
        return;
      if (! progressController.finished) {
        var stack = progressController.stack;
        var ex = progressController.exception;
        var msg = gApp.getText("GenerateReportFailedMsg");
        celtxBugAlert(msg, stack, ex);
        return;
      }
      // window.setTimeout("gController.cmdImportBreakdown()", 0);
    }
  },


  // Note: Calendar is saved automatically with each operation
  save: function save () {
    var rdffile = this.project.fileForResource(this.docres);
    serializeDataSourceToFile(this.bdds, rdffile);
    var proj = gController.project;
    var scriptfile = proj.fileForResource(gController.docres, 'secondary');
    if (scriptfile)
      serializeDOMtoFile(gScriptController.frame.contentDocument, scriptfile);

    this.modificationCount = 0;
  },


  close: function () {
    gWindow.scenetree.database.RemoveDataSource(this.project.ds);
    gWindow.scenetree.database.RemoveDataSource(this.bdds);
    gWindow.locationpopup.database.RemoveDataSource(this.project.ds);
    gWindow.locationlist.database.RemoveDataSource(this.project.ds);
    // gWindow.locationscenes.database.RemoveDataSource(this.bdds);
    gScriptController.deptList.database.RemoveDataSource(this.project.ds);
    gScriptController.deptList.database.RemoveDataSource(this.deptds);

    var bdtoolbar = this.outline.document.getElementById("bdtoolbar");
    bdtoolbar.removeEventListener("command", this, false);
    gLocationController.closed();
  },


  focus: function () {
    var lastcard = gWindow.detailsdeck.selectedPanel;
    if (lastcard) {
      switch (lastcard.id) {
        case "breakdowncard":
          gBreakdownController.focus();
          break;
        case "locationcard":
          gLocationController.focus();
          break;
        case "departmentcard":
          gDeptController.focus();
          break;
        case "scriptcard":
          gScriptController.focus();
          break;
        case "schedulecard":
          gScheduleController.focus();
          break;
        default:
          break;
      }
    }
  },


  blur: function () {
    if (gWindow.inPrintPreview)
      PrintUtils.exitPrintPreview();
    var lastcard = gWindow.detailsdeck.selectedPanel;
    if (lastcard) {
      switch (lastcard.id) {
        case "breakdowncard":
          gBreakdownController.blur();
          break;
        case "locationcard":
          gLocationController.blur();
          break;
        case "departmentcard":
          gDeptController.blur();
          break;
        case "scriptcard":
          gScriptController.blur();
          break;
        case "schedulecard":
          gScheduleController.blur();
          break;
        default:
          break;
      }
    }
  },


  createScene: function createScene () {
    var rdfsvc      = getRDFService();
    var typearc     = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var titlearc    = rdfsvc.GetResource(Cx.NS_DC + "title");
    var ordarc      = rdfsvc.GetResource(Cx.NS_CX + "ordinal");
    var intextarc   = rdfsvc.GetResource(Cx.NS_CX + "intext");
    var daynightarc = rdfsvc.GetResource(Cx.NS_CX + "daynight");
    var scenetype   = rdfsvc.GetResource(Cx.NS_CX + "Scene");
    var scenesarc   = rdfsvc.GetResource(Cx.NS_CX + "scenes");

    // Localized defaults
    var intstr = gApp.getText("Location.Int");
    var daystr = gApp.getText("Location.Day");

    // Get the scene sequence
    var scenesres = this.bdds.GetTarget(this.docres, scenesarc, true);
    var sceneseq = new RDFSeq(this.bdds, scenesres);
    var ord = sceneseq.length + 1;

    // Create the new scene
    var newscene = rdfsvc.GetResource(Cx.NS_CX + generateID());
    this.bdds.Assert(newscene, typearc, scenetype, true);
    this.bdds.Assert(newscene, titlearc, rdfsvc.GetLiteral(""), true);
    this.bdds.Assert(newscene, intextarc, rdfsvc.GetLiteral(intstr), true);
    this.bdds.Assert(newscene, daynightarc, rdfsvc.GetLiteral(daystr), true);
    this.bdds.Assert(newscene, ordarc, rdfsvc.GetIntLiteral(ord), true);
    sceneseq.push(newscene);

    return newscene;
  },


  cmdCreateScene: function cmdCreateScene () {
    try {
      this.selectedScene = this.createScene();
      if (gWindow.detailsdeck.selectedPanel.id == "breakdowncard")
        gWindow.slugline.focus();
    }
    catch (ex) {
      dump("*** cmdCreateScene: " + ex + "\n");
    }
  },


  loadBreakdown: function loadBreakdown () {
    // At this point, the RDF datasource has been loaded. All that remains
    // is to construct the listboxes and populate them.
    this.bdds.AddObserver(this);
    var ILit = Components.interfaces.nsIRDFLiteral;
    var IRes = Components.interfaces.nsIRDFResource;
    var rdfsvc = getRDFService();
    var cu = getRDFContainerUtils();
    var titlearc = rdfsvc.GetResource(Cx.NS_RDFS + "label");
    var scenesarc = rdfsvc.GetResource(Cx.NS_CX + "scenes");
    var scenesres = this.bdds.GetTarget(this.docres, scenesarc, true);
    if (! (scenesres && scenesres instanceof IRes)) {
      dump("*** There's no cx:scenes arc off the breakdown document!\n");
      return;
    }
    scenesres = scenesres.QueryInterface(IRes);
    gWindow.scenetree.database.AddDataSource(this.project.ds);
    gWindow.scenetree.database.AddDataSource(this.bdds);
    gWindow.scenetree.ref = scenesres.Value;

    // Set the summary page's scene counter
    var sceneseq = new RDFSeq(this.bdds, scenesres);
    gWindow.scenecount.value = sceneseq.length;

    // Set the summary page's character counter
    var charsseen = {};
    var charcounter = 0;
    var scenes = sceneseq.toArray();
    var membersarc = rdfsvc.GetResource(Cx.NS_CX + "members");
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var chartype = rdfsvc.GetResource(Cx.NS_CX + "Cast");
    for (var i = 0; i < scenes.length; ++i) {
      var scene = scenes[i].QueryInterface(IRes);
      var members = this.bdds.GetTarget(scene, membersarc, true);
      if (! members)
        continue;
      members = new RDFSeq(this.bdds, members).toArray();
      for (var j = 0; j < members.length; ++j) {
        var member = members[j].QueryInterface(IRes);
        var type = this.project.ds.GetTarget(member, typearc, true);
        type = type.QueryInterface(IRes);
        if (type.Value != chartype.Value)
          continue;
        if (charsseen[member.Value])
          continue;
        charsseen[member.Value] = 1;
        ++charcounter;
      }
    }
    gWindow.charcount.value = charcounter;

    setTimeout("gController.opened()", 100);
  },


  opened: function opened () {
    gScriptController.opened();
    gBreakdownController.opened();
    gDeptController.opened();
    gLocationController.opened();
    gScheduleController.opened();

    this.outline.document.getElementById("bdtoolbar").selectedIndex = 0;

    // Select the first scene (if one exists)
    if (gWindow.scenetree.view && gWindow.scenetree.view.rowCount > 0)
      gWindow.scenetree.view.selection.select(0);
  },


  cmdImportBreakdown: function cmdImportBreakdown () {
    var rdfsvc = getRDFService();
    var sourcearc = rdfsvc.GetResource(Cx.NS_DC + "source");
    var source = this.project.ds.GetTarget(this.docres, sourcearc, true);
    if (! source) {
      dump("*** cmdImportBreakdown: No source on docres!\n");
      return;
    }
    source = source.QueryInterface(Components.interfaces.nsIRDFResource);
    this.bdds = getInMemoryDataSource();
    // var catarc = rdfsvc.GetResource(Cx.NS_CX + "categories");
    // var catres = rdfsvc.GetAnonymousResource();
    // this.bdds.Assert(this.docres, catarc, catres, true);
    // var catseq = new RDFSeq(this.bdds, catres);
    // for (var i = 0; i < config.categories.length; ++i)
    //   catseq.push(rdfsvc.GetResource(config.categories[i]));
    try {
      // if (config.type == "celtx") {
        // var scriptres = rdfsvc.GetResource(config.source);
        // var scriptfile = this.project.fileForResource(scriptres);
        var scriptfile = this.project.fileForResource(source);
        var scriptframe = document.getElementById("script");
        scriptframe.setAttribute("src", fileToFileURL(scriptfile));
        setTimeout("gController.delayedScriptImport()", 0);
      // }
      // else if (config.type == "moviemagic") {
      //   alert("Whoops, we don't actually support Movie Magic");
      // }
    }
    catch (ex) {
      dump("*** cmdImportBreakdown: " + ex + "\n");
    }
  },


  delayedScriptImport: function delayedScriptImport () {
    var scriptframe = document.getElementById("script");
    if (scriptframe.docShell.busyFlags) {
      setTimeout("gController.delayedScriptImport()", 100);
      return;
    }

    // Create a breakdown tree and turn it into a datasource
    try {
      var xslt = document.implementation.createDocument("", "", null);
      xslt.async = false;
      xslt.load(Cx.TRANSFORM_PATH + "breakdown-tree.xml");
      var proc = new XSLTProcessor();
      proc.importStylesheet(xslt);
      var tree = proc.transformToDocument(scriptframe.contentDocument);

      // Set up the things we need to parse scene headers
      var intextvals = [ gApp.getText("Location.Int"),
        gApp.getText("Location.Ext") ];
      try {
        var ps = getPrefService().getBranch("celtx.breakdown.");
        var list = ps.getCharPref("intexts").split(',');
        while (list.length > 0)
          intextvals.push(list.shift());
      }
      catch (ex) {}
      var intextre = "[0-9. -]*(" + intextvals.shift();
      while (intextvals.length > 0)
        intextre += "|" + intextvals.shift();
      intextre += ")[. -]*(.*)";
      intextre = new RegExp(intextre, "i");

      var daynightvals = [ gApp.getText("Location.Day"),
        gApp.getText("Location.Night") ];
      try {
        var ps = getPrefService().getBranch("celtx.breakdown.");
        var list = ps.getCharPref("daynights").split(',');
        while (list.length > 0)
          daynightvals.push(list.shift());
      }
      catch (ex) {}
      var daynightre = "(.*[^. -])[. -]*(" + daynightvals.shift();
      while (daynightvals.length > 0)
        daynightre += "|" + daynightvals.shift();
      daynightre += ")";
      daynightre = new RegExp(daynightre, "i");

      var rdfsvc = getRDFService();
      var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
      var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
      var ordarc = rdfsvc.GetResource(Cx.NS_CX + "ordinal");
      var intextarc = rdfsvc.GetResource(Cx.NS_CX + "intext");
      var daynightarc = rdfsvc.GetResource(Cx.NS_CX + "daynight");
      var locationarc = rdfsvc.GetResource(Cx.NS_CX + "location");
      var scenetype = rdfsvc.GetResource(Cx.NS_CX + "Scene");
      var scenesarc = rdfsvc.GetResource(Cx.NS_CX + "scenes");
      var scenesres = rdfsvc.GetAnonymousResource();
      var sceneseq = new RDFSeq(this.bdds, scenesres);
      this.bdds.Assert(this.docres, scenesarc, scenesres, true);
      var xpath = new XPathEvaluator();
      var xset = xpath.evaluate("/script/scene", tree, null,
                     XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
      var scene = null;
      var i = 1;
      while (scene = xset.iterateNext()) {
        var intext = "INT";
        var daynight = "DAY";
        var title = scene.getAttribute("title").toUpperCase();
        var match = intextre.exec(title);
        if (match) {
          intext = match[1];
          title = match[2];
        }
        match = daynightre.exec(title);
        if (match) {
          title = match[1];
          daynight = match[2];
        }
        var titlelit = rdfsvc.GetLiteral(title);
        var intextlit = rdfsvc.GetLiteral(intext);
        var daynightlit = rdfsvc.GetLiteral(daynight);
        var ord = rdfsvc.GetIntLiteral(i++);
        var sceneres = rdfsvc.GetResource(Cx.SCENE_PREFIX +
                                          scene.getAttribute("id"));
        // var locationres = this.lookupOrCreateLocation(title);
        this.bdds.Assert(sceneres, typearc, scenetype, true);
        this.bdds.Assert(sceneres, titlearc, titlelit, true);
        this.bdds.Assert(sceneres, ordarc, ord, true);
        this.bdds.Assert(sceneres, intextarc, intextlit, true);
        this.bdds.Assert(sceneres, daynightarc, daynightlit, true);
        // this.bdds.Assert(sceneres, locationarc, locationres, true);
        // Add it to the scene sequence
        sceneseq.push(sceneres);
        // Add individual departments
        var memberarc = rdfsvc.GetResource(Cx.NS_CX + "members");
        var memberres = rdfsvc.GetAnonymousResource();
        this.bdds.Assert(sceneres, memberarc, memberres, true);
        var memberseq = new RDFSeq(this.bdds, memberres);
        var items = scene.getElementsByTagName("item");
        var seen = {};
        for (var j = 0; j < items.length; ++j) {
          var ref = items[j].getAttribute("ref");
          if (! seen[ref]) {
            memberseq.push(rdfsvc.GetResource(ref));
            seen[ref] = 1;
          }
        }
        this.populateCharacterMap();
        var chars = scene.getElementsByTagName("char");
        for (var j = 0; j < chars.length; ++j) {
          var char = chars[j].getAttribute("name");
          var charres = this.resolveCharacterName(char);
          if (! seen[charres]) {
            memberseq.push(rdfsvc.GetResource(charres));
            seen[charres] = 1;
          }
        }
      }

      var rdffile = this.project.projectFolder;
      rdffile.append("production.rdf");
      rdffile.createUnique(0, rdffile.parent.permissions & 0644);
      serializeDataSourceToFile(this.bdds, rdffile);
      this.project.addFileToDocument(rdffile, this.docres);
    }
    catch (ex) {
      // TODO: i18n
      dump("*** delayedScriptImport: " + ex + "\n");
      celtxBugAlert("Exception: " + ex, Components.stack, ex);
    }

    var scriptfile = this.project.projectFolder;
    scriptfile.append("production.html");
    scriptfile.createUnique(0, scriptfile.parent.permissions & 0644);
    serializeDOMtoFile(scriptframe.contentDocument, scriptfile);
    this.project.addFileToDocument(scriptfile, this.docres, "secondary");

    gWindow.locationpopup.builder.rebuild();
    window.setTimeout("gController.loadBreakdown()", 0);

    return;


    // === No longer used ===

    // While we're at it, transform it into a more XML-friendly format
    // for easy manipulation.
    try {
      var xslt = document.implementation.createDocument("", "", null);
      xslt.async = false;
      xslt.load(Cx.TRANSFORM_PATH + "script-tree.xml");
      var proc = new XSLTProcessor();
      proc.importStylesheet(xslt);
      this.script = proc.transformToDocument(scriptframe.contentDocument);

      var xmlfile = this.project.projectFolder;
      xmlfile.append("production.xml");
      xmlfile.createUnique(0, xmlfile.parent.permissions & 0644);
      serializeDOMtoFile(this.script, xmlfile);
      this.project.addFileToDocument(xmlfile, this.docres, "secondary");
    }
    catch (ex) {
      // TODO: i18n
      dump("*** delayedScriptImport: " + ex + "\n");
      celtxBugAlert("Exception: " + ex, Components.stack, ex);
    }

    // We've probably added at least one new location! Rebuild the popup.
    gWindow.locationpopup.builder.rebuild();

    window.setTimeout("gController.loadBreakdown()", 0);
  },


  locationMap: {},


  lookupLocation: function lookupLocation (location) {
    if (location in this.locationMap)
      return this.locationMap[location];
    var ILit = Components.interfaces.nsIRDFLiteral;
    var IRes = Components.interfaces.nsIRDFResource;
    var rdfsvc = getRDFService();
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var locationres = rdfsvc.GetResource(Cx.NS_CX + "Location");
    var locations = this.project.ds.GetSources(typearc, locationres, true);
    while (locations.hasMoreElements()) {
      try {
        var locres = locations.getNext().QueryInterface(IRes);
        var title = this.project.ds.GetTarget(locres, titlearc, true);
        title = title.QueryInterface(ILit);
        if (title.Value.toUpperCase() == location) {
          this.locationMap[location] = locres;
          return locres;
        }
      }
      catch (ex) {
        dump("*** lookupLocation: " + ex + "\n");
      }
    }
    return null;
  },


  lookupOrCreateLocation: function lookupOrCreateLocation (location) {
    var res = this.lookupLocation(location);
    if (res)
      return res;
    var rdfsvc = getRDFService();
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var locationres = rdfsvc.GetResource(Cx.NS_CX + "Location");
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var titlelit = rdfsvc.GetLiteral(location);
    var doctypearc = rdfsvc.GetResource(Cx.NS_CX + "doctype");
    var doctyperes = rdfsvc.GetResource(Cx.NS_CX + "LocationDocument");
    var categoryarc = rdfsvc.GetResource(Cx.NS_CX + "category");
    var sourcearc = rdfsvc.GetResource(Cx.NS_DC + "source");

    // Create the location item
    res = rdfsvc.GetResource(this.project.mintURI());
    this.project.ds.Assert(res, typearc, locationres, true);
    this.project.ds.Assert(res, titlearc, titlelit, true);

    // Create the location document
    var folder = top.getSelectedDocumentFolder();
    this.project.ds.beginUpdateBatch();
    try {
      var docres = this.project.createDocument(location, doctyperes,
        folder, res);
    }
    catch (ex) {
      dump("*** lookupOrCreateLocation: " + ex + "\n");
    }
    this.project.ds.endUpdateBatch();

    this.locationMap[location] = res;
    return res;
  },


  resolveMap: {},
  resolveExplicitMap: {},


  populateCharacterMap: function populateCharacterMap () {
    this.resolveMap = {};
    var rdfsvc      = getRDFService();
    var IRes        = Components.interfaces.nsIRDFResource;
    var ILit        = Components.interfaces.nsIRDFLiteral;
    var titleProp   = rdfsvc.GetResource(Cx.NS_DC  + "title");
    var typeProp    = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var classRes    = rdfsvc.GetResource(Cx.NS_CX + "Cast");
    var ds          = this.project.ds;

    var characters = ds.GetSources(typeProp, classRes, true);
    while (characters.hasMoreElements()) {
      try {
        var ref = characters.getNext().QueryInterface(IRes);
        var title = ds.GetTarget(ref, titleProp, true).QueryInterface(ILit);
        this.resolveMap[title.Value.toUpperCase()] = ref.Value;
      }
      catch (ex) {
        dump("*** populateCharacterMap: " + ex + "\n");
      }
    }
  },


  resolveCharacterName: function resolveCharacterName (str) {
    str = str.replace(/\s*\(.*\)/, '');   // Trim parenthesized chunks
    if (str == '') return;
    str = str.toUpperCase();

    // New method: If it isn't explicit, it isn't there.
    // return this.resolveMap[str];

    // Old method (not reached)

    // Check the map
    if (this.resolveMap[str]) return this.resolveMap[str];

    var rdfsvc = getRDFService();
    var IRes = Components.interfaces.nsIRDFResource;
    var ILit = Components.interfaces.nsIRDFLiteral;

    // Not in the map, grovel through the model
    var nameProp  = rdfsvc.GetResource(Cx.NS_CX  + "scriptName");
    var titleProp = rdfsvc.GetResource(Cx.NS_DC  + "title");
    var typeProp  = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var classRes  = rdfsvc.GetResource(Cx.NS_CX + "Cast");
    var name      = rdfsvc.GetLiteral(str);

    var ds = this.project.ds;
    var res = ds.GetSource(nameProp, name, true);
    if (! res) {
      // Try to locate via title
      var matches = ds.GetSources(titleProp, name, true);
      while (matches.hasMoreElements()) {
        var match = matches.getNext().QueryInterface(IRes);
        if (ds.HasAssertion(match, typeProp, classRes, true)) {
          res = match;
          ds.Assert(res, nameProp, name, true);
          break;
        }
      }
    }

    if (! res && this.resolveExplicitMap[str]) {
      res = rdfsvc.GetResource(this.resolveExplicitMap[str]);
      ds.Assert(res, nameProp, name, true);
    }

    if (! res) {
      var casts = ds.GetSources(typeProp, classRes, true);
      while (casts.hasMoreElements()) {
        var cast = casts.getNext().QueryInterface(IRes);
        var title = ds.GetTarget(cast, titleProp, true);
        if (title) {
          title = title.QueryInterface(ILit);
          if (str == title.Value.toUpperCase()) {
            res = cast;
            ds.Assert(res, nameProp, name, true);
            break;
          }
        }
        else {
          dump("*** Empty cx:Cast element\n");
        }
      }
    }

    if (! res) {
      // Add them in ourselves
      res = rdfsvc.GetResource(Cx.DOCUMENTS_URL + generateID());
      ds.Assert(res, typeProp, classRes, true);
      ds.Assert(res, titleProp, name, true);
      ds.Assert(res, nameProp, name, true);
      gController.project.createDocument(name.Value,
        rdfsvc.GetResource(Cx.NS_CX + "CharacterDocument"),
        top.getSelectedDocumentFolder(), res);
    }

    this.resolveMap[str] = res.Value;
    return res.Value;
  },


  get selectedScene () {
    try {
      var idx = gWindow.scenetree.view.selection.currentIndex;
      if (idx < 0)
        return null;
      return gWindow.scenetree.view.getResourceAtIndex(idx);
    }
    catch (ex) {
      dump("*** selectedScene: " + ex + "\n");
      return null;
    }
  },


  set selectedScene (val) {
    try {
      var idx = gWindow.scenetree.view.getIndexOfResource(val);
      if (idx >= 0)
        gWindow.scenetree.view.selection.select(idx);
      return idx;
    }
    catch (ex) {
      dump("*** set selectedScene: " + ex + "\n");
      return -1;
    }
  },


  saveTreeSelection: function saveTreeSelection () {
    this._treeSelection = this.selectedScene;
    this.suppressSceneSelect = true;
  },


  restoreTreeSelection: function restoreTreeSelection () {
    this.selectedScene = this._treeSelection;
    this.suppressSceneSelect = false;
  },


  onSceneSelect: function onSceneSelect () {
    if (this.suppressSceneSelect) return;

    var scene = this.selectedScene;
    if (! scene)
      return;

    gBreakdownController.onSceneSelect(scene);
    gScriptController.onSceneSelect(scene);

    // If this is a first time selection, flip the "nothing selected" deck
    // if (gWindow.detailsdeck.selectedIndex == 0) {
    //   this.selectView("breakdown");
    // }
  },


  onSceneChange: function onSceneChange (sceneres) {
    for (var i = 0; i < this.controllers.length; ++i) {
      if ("onSceneChange" in this.controllers[i]) {
        try { this.controllers[i].onSceneChange(sceneres); }
        catch (ex) { dump("*** onSceneChange: " + ex + "\n"); }
      }
    }
  },


  onSceneDrag: function onSceneDrag (event) {
    if (event.originalTarget.localName == "treechildren")
      nsDragAndDrop.startDrag(event, this);
  },


  onDragStart: function (aEvent, aXferData, aDragAction) {
    var scene = this.selectedScene;
    if (! scene) {
      dump("*** No selected scene\n");
      return;
    }
    var data = new TransferData();
    data.addDataForFlavour("x-celtx/x-scene", scene.Value);
    aXferData.data = data;
  },


  selectView: function selectView (viewname) {
    var card = document.getElementById(viewname + "card");
    var lastcard = gWindow.detailsdeck.selectedPanel;
    if (lastcard) {
      switch (lastcard.id) {
        case "scriptcard":
          gScriptController.blur();
          break;
        case "breakdowncard":
          gBreakdownController.blur();
          break;
        case "locationcard":
          gLocationController.blur();
          break;
        case "departmentcard":
          gDeptController.blur();
          break;
        case "schedulecard":
          gScheduleController.blur();
          break;
        default:
          break;
      }
    }
    gWindow.detailsdeck.selectedPanel = card;
    switch (viewname) {
      case "summary":
        gWindow.navdeck.hidden = true;
        return;
      case "script":
        gWindow.navdeck.selectedPanel = document.getElementById("scenelist");
        gScriptController.focus();
        break;
      case "breakdown":
        gWindow.navdeck.selectedPanel = document.getElementById("scenelist");
        gBreakdownController.focus();
        break;
      case "location":
        gWindow.navdeck.selectedPanel = document.getElementById("locationlist");
        gLocationController.focus();
        break;
      case "department":
        gWindow.navdeck.selectedPanel = document.getElementById("itemlist");
        gDeptController.focus();
        break;
      case "schedule":
        gWindow.navdeck.selectedPanel = document.getElementById("scenelist");
        gScheduleController.focus();
        break;
      default:
        gWindow.navdeck.selectedPanel = document.getElementById("scenelist");
    }
    gWindow.headerdeck.selectedIndex = gWindow.detailsdeck.selectedIndex;
    gWindow.navdeck.hidden = false;

    top.updateProjectCommands();
  },


  // nsIDOMEventListener implementation
  handleEvent: function handleEvent (event) {
    if (event.type == "select") {
      this.selectView(event.target.selectedItem.getAttribute("value"));
    }
  },


  // nsIRDFObserver implementation
  onAssert: function onAssert (ds, src, prop, tgt) {
    ++this.modificationCount;
    // Until we can distinguish "do" from "undo", there's no point
    // listening to further events
    this.bdds.RemoveObserver(this);
  },


  onChange: function onChange (ds, src, prop, oldtgt, newtgt) {
    ++this.modificationCount;
    // Until we can distinguish "do" from "undo", there's no point
    // listening to further events
    this.bdds.RemoveObserver(this);
  },


  onMove: function onMove (ds, oldsrc, newsrc, prop, tgt) {
    ++this.modificationCount;
    // Until we can distinguish "do" from "undo", there's no point
    // listening to further events
    this.bdds.RemoveObserver(this);
  },


  onUnassert: function onUnassert (ds, src, prop, tgt) {
    ++this.modificationCount;
    // Until we can distinguish "do" from "undo", there's no point
    // listening to further events
    this.bdds.RemoveObserver(this);
  },


  onBeginUpdateBatch: function (ds) {},
  onEndUpdateBatch: function (ds) {}
};

var gScriptController = {
  loaded: function loaded () {
    this.deptList   = document.getElementById("scriptdeptlist");
    this.itemList   = document.getElementById("scriptitemlist");
    this.itemName   = document.getElementById("scriptitemname");
    this.itemDesc   = document.getElementById("scriptitemdesc");
    this.tagButton  = document.getElementById("scripttagbutton");
    this.frame      = document.getElementById("script");
  },


  opened: function opened () {
    var project = gController.project;
    var docres = gController.docres;
    var scriptfile = project.fileForResource(docres, "secondary");
    if (isReadableFile(scriptfile))
      this.frame.setAttribute("src", fileToFileURL(scriptfile));

    this.deptList.database.AddDataSource(gController.deptds);
    this.deptList.builder.rebuild();
    this.itemList.database.AddDataSource(gController.project.ds);
    this.itemList.builder.rebuild();
  },


  focus: function focus () {
    this.frame.setAttribute("type", "content-primary");
  },


  blur: function blur () {
    this.frame.setAttribute("type", "content");
  },


  get selection () {
    return this.frame.contentWindow.getSelection();
  },


  sceneForSelection: function sceneForSelection (sel) {
    var node = sel.focusNode;
    var IPara = Components.interfaces.nsIDOMHTMLParagraphElement;
    while (node && ! (node instanceof IPara))
      node = node.parentNode;
    if (! node) return null;
    var xpath = new XPathEvaluator();
    var str = "(self::p[@class='sceneheading'] | "
      + "preceding-sibling::p[@class='sceneheading'])[last()]";
    var rv = xpath.evaluate(str, node, null,
      XPathResult.FIRST_ORDERED_NODE_TYPE, null);
    if (! (rv && rv.singleNodeValue)) return null;
    var rdfsvc = getRDFService();
    return rdfsvc.GetResource(Cx.SCENE_PREFIX + rv.singleNodeValue.id);
  },


  onSceneSelect: function onSceneSelect (scene) {
    var sceneid = scene.Value.substring(Cx.SCENE_PREFIX.length);
    if (! sceneid) {
      dump("*** gScriptController.onSceneSelect: Couldn't extract scene id "
        + "from " + scene.Value + "\n");
      return;
    }

    var xpath = new XPathEvaluator();
    var str = "/html/body/p[@id='" + sceneid + "']";
    var scene = xpath.evaluate(str, this.frame.contentDocument, null,
      XPathResult.FIRST_ORDERED_NODE_TYPE, null);
    if (! scene.singleNodeValue) {
      dump("*** gScriptController.onSceneSelect: Failed to match scene "
        + sceneid + "\n");
      return;
    }
    scene = scene.singleNodeValue;
    this.frame.contentWindow.scrollTo(0, scene.offsetTop);
  },


  onDeptSelected: function onDeptSelected () {
    if (this.currentItem) {
      this.itemName.inputField.blur();
      this.itemDesc.inputField.blur();
    }

    this.currentItem = null;
    this.itemList.clearSelection();

    this.itemList.ref = this.deptList.selectedItem.id;
    this.updateTagCommand();
  },


  onItemSelected: function onItemSelected () {
    if (this.currentItem) {
      this.itemName.inputField.blur();
      this.itemDesc.inputField.blur();
    }

    var item = this.itemList.selectedItem;

    if (! item || item.id == "new-item") {
      if (item && item.id == "new-item")
        this.itemName.value = this.selection.toString();
      else
        this.itemName.value = "";
      this.itemDesc.value = "";
      this.currentItem = null;
    }
    else {
      var rdfsvc = getRDFService();
      var ILit = Components.interfaces.nsIRDFLiteral;
      var titleArc = rdfsvc.GetResource(Cx.NS_DC + "title");
      var descArc = rdfsvc.GetResource(Cx.NS_DC + "description");
      var itemRes = rdfsvc.GetResource(item.id);
      var title = gController.project.ds.GetTarget(itemRes, titleArc, true);
      var desc = gController.project.ds.GetTarget(itemRes, descArc, true);
      if (title)
        this.itemName.value = title.QueryInterface(ILit).Value;
      else
        this.itemName.value = "";
      if (desc)
        this.itemDesc.value = desc.QueryInterface(ILit).Value;
      else
        this.itemDesc.value = "";
      this.currentItem = itemRes;
    }

    this.updateTagCommand();
  },


  updateTagCommand: function updateTagCommand () {
    var disabled = false;
    if (this.itemName.value == "" || ! this.selection ||
        this.selection.rangeCount == 0 || this.selection.isCollapsed)
      disabled = true;
    this.tagButton.disabled = disabled;
  },


  nameChanged: function nameChanged () {
    var itemres = this.currentItem;
    if (! itemres)
      return;
    var rdfsvc = getRDFService();
    var ds = gController.project.ds;
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var oldtitle = ds.GetTarget(itemres, titlearc, true);
    var newtitle = rdfsvc.GetLiteral(this.itemName.value);
    if (oldtitle)
      ds.Change(itemres, titlearc, oldtitle, newtitle);
    else
      ds.Assert(itemres, titlearc, newtitle, true);
  },


  descChanged: function descChanged () {
    var itemres = this.currentItem;
    if (! itemres)
      return;
    var rdfsvc = getRDFService();
    var ds = gController.project.ds;
    var descarc = rdfsvc.GetResource(Cx.NS_DC + "description");
    var olddesc = ds.GetTarget(itemres, descarc, true);
    var newdesc = rdfsvc.GetLiteral(this.itemDesc.value);
    if (olddesc)
      ds.Change(itemres, descarc, olddesc, newdesc);
    else
      ds.Assert(itemres, descarc, newdesc, true);
  },


  tagItem: function tagItem () {
    if (! this.selection || this.selection.rangeCount != 1)
      return;
    var item = this.itemList.selectedItem;
    var dept = this.deptList.selectedItem;
    var ILit = Components.interfaces.nsIRDFLiteral;
    var rdfsvc = getRDFService();
    var ds = gController.project.ds;
    var bdds = gController.bdds;
    var deptRes = rdfsvc.GetResource(dept.id);
    var itemRes = null;
    if (item.id == "new-item") {
      itemRes = rdfsvc.GetResource(gController.project.mintURI());
      var typeArc = rdfsvc.GetResource(Cx.NS_RDF + "type");
      ds.Assert(itemRes, typeArc, deptRes, true);
      // Create the corresponding document
      var doctypeds = rdfsvc.GetDataSourceBlocking(Cx.DOCTYPES_URL);
      var catarc    = rdfsvc.GetResource(Cx.NS_CX + "category");
      var srcarc    = rdfsvc.GetResource(Cx.NS_DC + "source");
      var doctype   = doctypeds.GetSource(catarc, deptRes, true);
      if (doctype) {
        ds.beginUpdateBatch();
        try {
          var folder = top.getSelectedDocumentFolder();
          var docres = gController.project.createDocument(this.itemName.value,
            doctype, folder, itemRes);
        }
        catch (ex) {
          dump("*** cmdTagItem: " + ex + "\n");
        }
        ds.endUpdateBatch();
      }
      else {
        dump("*** cmdTagItem: " + deptRes.Value
          + " lacks a corresponding doctype\n");
      }
    }
    else {
      itemRes = rdfsvc.GetResource(item.id);
    }
    var titleArc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var descArc = rdfsvc.GetResource(Cx.NS_DC + "description");
    setRDFString(ds, itemRes, titleArc, this.itemName.value);
    setRDFString(ds, itemRes, descArc, this.itemDesc.value);

    try {
      var scene = this.sceneForSelection(this.selection);
      if (! scene)
        throw "Failed to locate a scene for the selection";
      var membersarc = rdfsvc.GetResource(Cx.NS_CX + "members");
      var membersseq = bdds.GetTarget(scene, membersarc, true);
      if (! membersseq) {
        membersseq = rdfsvc.GetAnonymousResource();
        bdds.Assert(scene, membersarc, membersseq, true);
      }
      membersseq = new RDFSeq(bdds, membersseq);
      if (membersseq.indexOf(itemRes) < 0)
        membersseq.push(itemRes);
    }
    catch (ex) {
      dump("*** tagItem: " + ex + "\n");
    }

    // Reselect the item
    var count = this.itemList.getRowCount();
    for (var i = 0; i < count; i++) {
      item = this.itemList.getItemAtIndex(i);
      if (item.id == itemRes.Value) {
        this.itemList.selectedItem = item;
        break;
      }
    }

    // Surround the item with a span tag
    try {
      var doc = this.frame.contentDocument;
      var span = doc.createElement("span");
      span.setAttribute("class", dept.getAttribute("cxelement"));
      span.setAttribute("ref", itemRes.Value);
      var range = this.selection.getRangeAt(0);
      range.surroundContents(span);
    }
    catch (ex) {
      dump("*** tagItem: Error creating span tag, " + ex + "\n");
    }
  }
};

var gBreakdownController = {
  QueryInterface: function QueryInterface (iid) {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsIRDFObserver))
      return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },


  supportsCommand: function supportsCommand (cmd) {
    switch (cmd) {
      case "cmd-print":
        return true;
      default:
        return false;
    }
  },


  isCommandEnabled: function isCommandEnabled (cmd) {
    return this.supportsCommand(cmd);
  },


  doCommand: function doCommand (cmd) {
    switch (cmd) {
      case "cmd-print":
        this.print();
        break;
    }
  },


  // The RDF Observer grunt work
  dsupdating:0,
  onAssert: function onAssert (ds, src, prop, tgt) {},
  onChange: function onChange (ds, src, prop, oldtgt, newtgt) {},
  onMove: function onMove (ds, oldsrc, newsrc, prop, tgt) {},
  onUnassert: function onUnassert (ds, src, prop, tgt) {},


  onBeginUpdateBatch: function onBeginUpdateBatch (ds) {
    ++this.dsupdating;
  },


  onEndUpdateBatch: function onEndUpdateBatch (ds) {
    if (--this.dsupdating == 0)
      setTimeout("gBreakdownController.rebuildCategoryList()", 0);
  },


  loaded: function loaded () {
    // Construct references to document elements here.
    this.scenedesc    = document.getElementById("scenenotes");
  },


  opened: function opened () {
    // Listen for updates to the category prefs ds
    gController.deptds.AddObserver(this);
    this.rebuildCategoryList();
  },


  focus: function focus () {
    // this.frame.setAttribute("type", "content-primary");
  },


  blur: function blur () {
    // this.frame.setAttribute("type", "content");
  },


  print: function print () {
    var sceneres = gController.selectedScene;
    if (! sceneres) {
      dump("*** gBreakdownController.print: No scene selected\n");
      return;
    }

    var IInt = Components.interfaces.nsIRDFInt;
    var ILit = Components.interfaces.nsIRDFLiteral;
    var IRes = Components.interfaces.nsIRDFResource;
    var rdfsvc = getRDFService();
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var labelarc = rdfsvc.GetResource(Cx.NS_RDFS + "label");
    var ordarc = rdfsvc.GetResource(Cx.NS_CX + "ordinal");
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var intextarc = rdfsvc.GetResource(Cx.NS_CX + "intext");
    var daynightarc = rdfsvc.GetResource(Cx.NS_CX + "daynight");
    var locationarc = rdfsvc.GetResource(Cx.NS_CX + "location");
    var commentsarc = rdfsvc.GetResource(Cx.NS_CX + "comments");
    var membersarc = rdfsvc.GetResource(Cx.NS_CX + "members");

    var bdds = gController.bdds;
    var ds = gController.project.ds;
    var ord = bdds.GetTarget(sceneres, ordarc, true);
    ord = ord.QueryInterface(IInt).Value;
    var title = bdds.GetTarget(sceneres, titlearc, true);
    title = title.QueryInterface(ILit).Value;
    var intext = bdds.GetTarget(sceneres, intextarc, true);
    intext = intext.QueryInterface(ILit).Value;
    var daynight = bdds.GetTarget(sceneres, daynightarc, true);
    daynight = daynight.QueryInterface(ILit).Value;
    var comments = bdds.GetTarget(sceneres, commentsarc, true);
    if (comments && comments instanceof ILit)
      comments = comments.QueryInterface(ILit).Value;
    else
      comments = "";
    var location = bdds.GetTarget(sceneres, locationarc, true);
    if (location && location instanceof IRes) {
      var loctitle = ds.GetTarget(location, titlearc, true);
      location = loctitle.QueryInterface(ILit).Value;
    }
    else
      // TODO: i18n
      location = "No location selected";

    // Turn the scene into an XML intermediate for XSLT transformation
    var xmldoc = document.implementation.createDocument("", "scene", null);
    var scenenode = xmldoc.documentElement;
    scenenode.setAttribute("ordinal", ord);
    scenenode.setAttribute("title", title);
    scenenode.setAttribute("intext", intext);
    scenenode.setAttribute("daynight", daynight);
    scenenode.setAttribute("comments", comments);
    scenenode.setAttribute("location", location);

    var members = bdds.GetTarget(sceneres, membersarc, true);
    members = members.QueryInterface(IRes);
    var memberseq = new RDFSeq(bdds, members);
    var items = memberseq.toArray();
    var categories = {};
    for (var i = 0; i < items.length; ++i) {
      var item = items[i].QueryInterface(IRes);
      var itemtitle = ds.GetTarget(item, titlearc, true);
      itemtitle = itemtitle.QueryInterface(ILit).Value;
      var type = ds.GetTarget(item, typearc, true);
      type = type.QueryInterface(IRes);
      if (! (type.Value in categories)) {
        var cattitle = gController.schemads.GetTarget(type, labelarc, true);
        cattitle = cattitle.QueryInterface(ILit).Value;
        categories[type.Value] = {
          name: cattitle,
          contents: []
        };
      }
      categories[type.Value].contents.push({ name: itemtitle });
    }

    var sortedcats = [];
    for (var catres in categories)
      sortedcats.push({ res: catres, name: categories[catres].name });

    var sortfn = function (a, b) {
      return a.name.localeCompare(b.name);
    };
    sortedcats.sort(sortfn);

    for (var i = 0; i < sortedcats.length; ++i) {
      var catnode = xmldoc.createElement("category");
      scenenode.appendChild(catnode);
      var category = categories[sortedcats[i].res];
      catnode.setAttribute("title", category.name);
      for (var j = 0; j < category.contents.length; ++j) {
        var itemnode = xmldoc.createElement("item");
        catnode.appendChild(itemnode);
        itemnode.setAttribute("title", category.contents[j].name);
      }
    }

    // Do the transformation
    var xsldoc = document.implementation.createDocument("", "", null);
    xsldoc.async = false;
    xsldoc.load(Cx.TRANSFORM_PATH + "printscene.xml");

    var proc = new XSLTProcessor();
    proc.importStylesheet(xsldoc);
    proc.setParameter(null, "production", gController.project.title);
    var result = proc.transformToDocument(xmldoc);
    var file = pathToFile("/Users/tony/Desktop/TestReport.html");
    serializeDOMtoFile(result, file);
  },


  onSceneSelect: function onSceneSelect (scene) {
    var ILit = Components.interfaces.nsIRDFLiteral;
    var IRes = Components.interfaces.nsIRDFResource;
    var rdfsvc = getRDFService();
    var intextarc = rdfsvc.GetResource(Cx.NS_CX + "intext");
    var daynightarc = rdfsvc.GetResource(Cx.NS_CX + "daynight");
    var locarc = rdfsvc.GetResource(Cx.NS_CX + "location");
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var descarc = rdfsvc.GetResource(Cx.NS_DC + "description");
    var memberarc = rdfsvc.GetResource(Cx.NS_CX + "members");

    var intext = gController.bdds.GetTarget(scene, intextarc, true);
    var daynight = gController.bdds.GetTarget(scene, daynightarc, true);
    var title = gController.bdds.GetTarget(scene, titlearc, true);
    var desc = gController.bdds.GetTarget(scene, descarc, true);
    var locres = gController.bdds.GetTarget(scene, locarc, true);

    // Select the correct INT/EXT item
    if (intext && intext instanceof ILit) {
      var found = false;
      intext = intext.QueryInterface(ILit).Value;
      for (var i = 0; i < gWindow.intextpopup.childNodes.length; ++i) {
        var item = gWindow.intextpopup.childNodes[i];
        if (item.label == intext) {
          gWindow.intextmenu.selectedItem = item;
          found = true;
          break;
        }
      }
      if (! found)
        gWindow.intextmenu.selectedIndex = 0;
    }
    else
      gWindow.intextmenu.selectedIndex = 0;

    // Fill in the slugline
    if (title && title instanceof ILit) {
      title = title.QueryInterface(ILit).Value;
      gWindow.slugline.value = title;
    }
    else
      gWindow.slugline.value = "";

    // Select the correct DAY/NIGHT item
    if (daynight && daynight instanceof ILit) {
      var found = false;
      daynight = daynight.QueryInterface(ILit).Value;
      for (var i = 0; i < gWindow.daynightpopup.childNodes.length; ++i) {
        var item = gWindow.daynightpopup.childNodes[i];
        if (item.label == daynight) {
          gWindow.daynightmenu.selectedItem = item;
          found = true;
          break;
        }
      }
      if (! found)
        gWindow.daynightmenu.selectedIndex = 0;
    }
    else
      gWindow.daynightmenu.selectedIndex = 0;

    // Fill in the description
    if (desc && desc instanceof ILit) {
      desc = desc.QueryInterface(ILit).Value;
      this.scenedesc.value = desc;
    }
    else
      this.scenedesc.value = "";

    // Select the correct location item
    if (locres && locres instanceof IRes) {
      locres = locres.QueryInterface(IRes).Value;
      var found = false;
      for (var i = 0; i < gWindow.locationpopup.childNodes.length; ++i) {
        var item = gWindow.locationpopup.childNodes[i];
        if (item.id == locres) {
          gWindow.locationmenu.selectedItem = item;
          found = true;
          break;
        }
      }
      if (! found)
        gWindow.locationmenu.selectedIndex = 0;
    }
    else
      gWindow.locationmenu.selectedIndex = 0;

    // Clear the item lists
    var deptlists = document.getElementById("deptLists")
      .getElementsByTagName("listbox");
    for (var i = 0; i < deptlists.length; ++i) {
      var rowcount = deptlists[i].getRowCount();
      while (rowcount > 0) {
        deptlists[i].removeItemAt(--rowcount);
      }
    }

    // Repopulate the item lists
    var memberres = gController.bdds.GetTarget(scene, memberarc, true);
    if (memberres && memberres instanceof IRes) {
      var memberseq = new RDFSeq(gController.bdds, memberres);
      var items = memberseq.toArray();
      for (var i = 0; i < items.length; ++i) {
        try {
          var item = items[i].QueryInterface(IRes);
          var title = gController.project.ds.GetTarget(item, titlearc, true);
          title = title.QueryInterface(ILit);
          var dept = gController.project.ds.GetTarget(item, typearc, true);
          dept = dept.QueryInterface(IRes);
          var deptname = dept.Value.replace(/.*\//, "");
          var listbox = document.getElementById(deptname + "_listbox");
          if (listbox)
            listbox.appendItem(title.Value, item.Value);
        }
        catch (ex) {
          dump("*** Error adding item " + item.Value + ": " + ex + "\n");
        }
      }
    }
  },


  onIntextChanged: function onIntextChanged () {
    var scene = gController.selectedScene;
    if (! scene) return;
    var intext = gWindow.intextmenu.selectedItem.label;
    var pred = getRDFService().GetResource(Cx.NS_CX + "intext");
    gController.saveTreeSelection();
    setRDFString(gController.bdds, scene, pred, intext);
    gController.restoreTreeSelection();
    gController.onSceneChange(scene);
  },


  onDaynightChanged: function onDaynightChanged () {
    var scene = gController.selectedScene;
    if (! scene) return;
    var daynight = gWindow.daynightmenu.selectedItem.label;
    var pred = getRDFService().GetResource(Cx.NS_CX + "daynight");
    gController.saveTreeSelection();
    setRDFString(gController.bdds, scene, pred, daynight);
    gController.restoreTreeSelection();
    gController.onSceneChange(scene);
  },


  onSluglineChanged: function onSluglineChanged () {
    var scene = gController.selectedScene;
    if (! scene) return;
    var slugline = gWindow.slugline.value;
    var pred = getRDFService().GetResource(Cx.NS_DC + "title");
    gController.saveTreeSelection();
    setRDFString(gController.bdds, scene, pred, slugline);
    gController.restoreTreeSelection();
    gController.onSceneChange(scene);
  },


  onLocationChanged: function onLocationChanged () {
    var scene = gController.selectedScene;
    if (! scene) return;
    var rdfsvc = getRDFService();
    var locarc = rdfsvc.GetResource(Cx.NS_CX + "location");
    var ds = gController.bdds;
    var oldloc = ds.GetTarget(scene, locarc, true);
    var item = gWindow.locationmenu.selectedItem;
    gController.saveTreeSelection();
    ds.beginUpdateBatch();
    if (! item.id) {
      if (oldloc) {
        ds.Unassert(scene, locarc, oldloc);
      }
    }
    else {
      var newloc = rdfsvc.GetResource(item.id);
      if (! oldloc) {
        ds.Assert(scene, locarc, newloc, true);
      }
      else if (oldloc.Value != newloc.Value) {
        ds.Change(scene, locarc, oldloc, newloc);
      }
    }
    ds.endUpdateBatch();
    gController.restoreTreeSelection();
    gController.onSceneChange(scene);
  },


  // Kludge to allow a multiline textbox with timeout
  onDescInput: function onDescInput () {
    if (this.descTimeout)
      clearTimeout(this.descTimeout);
    this.descTimeout = setTimeout("gBreakdownController.onDescChanged()", 500);
  },


  onDescChanged: function onDescChanged () {
    this.descTimeout = null;
    var scene = gController.selectedScene;
    if (! scene) return;
    var desc = this.scenedesc.value;
    var pred = getRDFService().GetResource(Cx.NS_DC + "description");
    // gController.saveTreeSelection();
    setRDFString(gController.bdds, scene, pred, desc);
    // gController.restoreTreeSelection();
    gController.onSceneChange(scene);
  },


  itemSelected: function itemSelected (category) {
    var catlist = document.getElementById(category + "_listbox");
    var delbtn = catlist.parentNode.firstChild.lastChild;
    if (catlist.selectedItem)
      delbtn.removeAttribute("disabled");
    else
      delbtn.setAttribute("disabled", "true");
  },


  addItem: function addItem (category) {
    var scene = gController.selectedScene;
    if (! scene) {
      return;
    }
    var catlist = document.getElementById(category + "_listbox");
    var args = {
      accepted: false,
      category: catlist.getAttribute("category"),
      project: gController.project,
      title: null,
      resource: null
    };
    openDialog(Cx.CONTENT_PATH + "editors/additem.xul", "_blank",
      "chrome,titlebar,modal,resizable", args);
    if (! args.accepted) return;

    gController.saveTreeSelection();

    var rdfsvc = getRDFService();
    var itemres = null;
    if (args.resource) {
      itemres = rdfsvc.GetResource(args.resource);
    }
    else {
      var ds        = gController.project.ds;
      var doctypeds = rdfsvc.GetDataSourceBlocking(Cx.DOCTYPES_URL);
      var typearc   = rdfsvc.GetResource(Cx.NS_RDF + "type");
      var titlearc  = rdfsvc.GetResource(Cx.NS_DC + "title");
      var srcarc    = rdfsvc.GetResource(Cx.NS_DC + "source");
      var catarc    = rdfsvc.GetResource(Cx.NS_CX + "category");
      var catres    = rdfsvc.GetResource(catlist.getAttribute("category"));
      itemres       = rdfsvc.GetResource(gController.project.mintURI());

      ds.Assert(itemres, typearc, catres, true);
      ds.Assert(itemres, titlearc, rdfsvc.GetLiteral(args.title), true);
      // Create the corresponding document
      var doctype   = doctypeds.GetSource(catarc, catres, true);
      if (doctype) {
        ds.beginUpdateBatch();
        try {
          var folder = top.getSelectedDocumentFolder();
          var docres = gController.project.createDocument(args.title,
            doctype, folder, itemres);
        }
        catch (ex) {
          dump("*** cmdTagItem: " + ex + "\n");
        }
        ds.endUpdateBatch();
      }
      else {
        dump("*** cmdTagItem: " + catres.Value
          + " lacks a corresponding doctype\n");
      }
    }
    var membersarc = rdfsvc.GetResource(Cx.NS_CX + "members");
    var membersseq = gController.bdds.GetTarget(scene, membersarc, true);
    if (! membersseq) {
      membersseq = rdfsvc.GetAnonymousResource();
      gController.bdds.Assert(scene, membersarc, membersseq, true);
    }
    membersseq = new RDFSeq(gController.bdds, membersseq);
    if (membersseq.indexOf(itemres) < 0) {
      membersseq.push(itemres);
      var item = catlist.appendItem(args.title, itemres.Value);
      catlist.selectedItem = item;
    }

    gController.restoreTreeSelection();
  },


  removeItem: function removeItem (category) {
    var scene = gController.selectedScene;
    if (! scene) {
      return;
    }
    var catlist = document.getElementById(category + "_listbox");
    var item = catlist.selectedItem;
    if (! item) {
      return;
    }
    var rdfsvc = getRDFService();
    var membersarc = rdfsvc.GetResource(Cx.NS_CX + "members");
    var membersseq = gController.bdds.GetTarget(scene, membersarc, true);
    if (! membersseq) {
      dump("*** removeItem: No members sequence for scene\n");
      return;
    }
    membersseq = new RDFSeq(gController.bdds, membersseq);
    var itemres = rdfsvc.GetResource(item.value);
    membersseq.remove(itemres);
    catlist.removeItemAt(catlist.getIndexOfItem(item));
  },


  createCategoryBox: function createCategoryBox(category, title) {
    var catname = category.Value.replace(/.*\//, '');
    // The vbox
    var vbox = document.createElementNS(Cx.NS_XUL, "vbox");
    vbox.setAttribute("flex", "1");
    // The hbox
    var hbox = document.createElementNS(Cx.NS_XUL, "hbox");
    hbox.setAttribute("align", "center");
    vbox.appendChild(hbox);
    // The label
    var label = document.createElementNS(Cx.NS_XUL, "label");
    label.setAttribute("value", title);
    hbox.appendChild(label);
    // The spacer
    var spacer = document.createElementNS(Cx.NS_XUL, "spacer");
    spacer.setAttribute("flex", "1");
    hbox.appendChild(spacer);
    // The Add button
    var addbtn = document.createElementNS(Cx.NS_XUL, "toolbarbutton");
    addbtn.setAttribute("class", "miniaddbtn");
    addbtn.setAttribute("oncommand", "gBreakdownController.addItem('"
      + catname + "')");
    hbox.appendChild(addbtn);
    // The Remove button
    var delbtn = document.createElementNS(Cx.NS_XUL, "toolbarbutton");
    delbtn.setAttribute("class", "minidelbtn");
    delbtn.setAttribute("oncommand",
      "gBreakdownController.removeItem('" + catname + "')");
    delbtn.setAttribute("disabled", "true");
    hbox.appendChild(delbtn);
    // The listbox
    var listbox = document.createElementNS(Cx.NS_XUL, "listbox");
    listbox.setAttribute("id", catname + "_listbox");
    listbox.setAttribute("category", category.Value);
    listbox.setAttribute("rows", "8");
    listbox.setAttribute("onselect", "gBreakdownController.itemSelected('"
      + catname + "')");
    vbox.appendChild(listbox);

    return vbox;
  },


  rebuildCategoryList: function rebuildCategoryList () {
    dump("--- rebuildCategoryList\n");
    var ILit = Components.interfaces.nsIRDFLiteral;
    var IRes = Components.interfaces.nsIRDFResource;
    var rdfsvc = getRDFService();
    var titlearc = rdfsvc.GetResource(Cx.NS_RDFS + "label");
    var catres = rdfsvc.GetResource(Cx.NS_CX + "Prefs/Categories");
    var schemads = gController.schemads;
    var catseq = new RDFSeq(gController.deptds, catres);
    var categories = catseq.toArray();
    var rows = document.getElementById("deptLists")
      .getElementsByTagName("rows")[0];
    while (rows.hasChildNodes())
      rows.removeChild(rows.lastChild);

    var sortedcats = [];
    for (var i = 0; i < categories.length; ++i) {
      if (! (categories[i] instanceof IRes)) {
        dump("*** Category is not an instance of nsIRDFResource\n");
        continue;
      }
      categories[i] = categories[i].QueryInterface(IRes);
      var cattitle = schemads.GetTarget(categories[i], titlearc, true);
      if (! (cattitle && cattitle instanceof ILit)) {
        dump("*** Category title is null or not an nsIRDFLiteral\n");
        continue;
      }
      cattitle = cattitle.QueryInterface(ILit).Value;
      sortedcats.push({ res: categories[i], title: cattitle });
    }
    var sortfn = function (a, b) {
      return a.title.localeCompare(b.title);
    };
    sortedcats.sort(sortfn);

    var row = null;
    var col = 0;
    for (var i = 0; i < sortedcats.length; ++i) {
      var box = this.createCategoryBox(sortedcats[i].res, sortedcats[i].title);
      if (! row) {
        row = document.createElementNS(Cx.NS_XUL, "row");
        rows.appendChild(row);
      }
      row.appendChild(box);
      if (++col == 3) {
        row = null;
        col = 0;
      }
    }
    if (row) {
      while (col-- > 0) {
        row.appendChild(document.createElementNS(Cx.NS_XUL, "spacer"));
      }
      rows.appendChild(row);
    }

    if (gController.selectedScene)
      this.onSceneSelect(gController.selectedScene);
  },


  editCategories: function editCategories () {
    top.openPreferences("paneCategories");
  }
};


var gLocationController = {
  QueryInterface: function (iid) {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsIDOMEventListener))
      return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },


  nameNodeMap: {},


  loaded: function loaded () {
    this.frame = document.getElementById("locationframe");
  },


  opened: function opened () {
    if (this.frame.docShell.busyFlags) {
      setTimeout("gLocationController.opened()", 100);
      return;
    }

    // Populate the list of locations.
    var project = gController.project;
    gWindow.locationpopup.database.AddDataSource(project.ds);
    gWindow.locationpopup.builder.rebuild();
    gWindow.locationlist.database.AddDataSource(project.ds);
    gWindow.locationlist.builder.rebuild();

    // Add the datasource for the location-scene list
    // gWindow.locationscenes.database.AddDataSource(gController.bdds);

    // Get rid of the scouting fields
    var scoutnode = this.frame.contentDocument.getElementById("expand-1");
    scoutnode.parentNode.parentNode.removeChild(scoutnode.parentNode);

    // Build a map of attributes to nodes (fields)
    var nodelist = [ this.frame.contentDocument.documentElement ];
    while (nodelist.length > 0) {
      var node = nodelist.shift();
      if (node.hasChildNodes()) {
        for (var i = 0; i < node.childNodes.length; ++i) {
          if (node.childNodes[i] instanceof Components.interfaces.nsIDOMElement)
            nodelist.push(node.childNodes[i]);
        }
      }
      if (node.hasAttribute("name")) {
        var name = node.getAttribute("name");
        if (name in this.nameNodeMap) {
          // Check if it's an array (i.e., radio buttons)
          var existing = this.nameNodeMap[name];
          if (existing.__proto__ == Array.prototype) {
            existing.push(node);
          }
          else {
            this.nameNodeMap[name] = [ existing, node ];
          }
        }
        else
          this.nameNodeMap[node.getAttribute("name")] = node;
      }
    }

    /*
    if (gWindow.locationlist.getRowCount() == 0)
      this.addLocation();
    */

    // gWindow.locationlist.selectedIndex = 0;
    if (gWindow.locationlist.getRowCount() > 0)
      gWindow.locationlist.selectedIndex = 0;

    this.frame.contentDocument.addEventListener("change", this, true);
    this.frame.contentDocument.addEventListener("input", this, true);
  },


  closed: function closed () {
    this.frame.contentDocument.removeEventListener("change", this, true);
    this.frame.contentDocument.removeEventListener("input", this, true);
  },


  focus: function focus () {
    this.frame.setAttribute("type", "content-primary");
  },


  blur: function blur () {
    this.frame.setAttribute("type", "content");
  },


  deleteLocation: function deleteLocation () {
    var idx = gWindow.locationlist.selectedIndex;
    var item = gWindow.locationlist.selectedItem;
    if (! item) return;
    var ds = gController.project.ds;
    var bdds = gController.bdds;
    var rdfsvc = getRDFService();
    var IRes = Components.interfaces.nsIRDFResource;
    var locarc = rdfsvc.GetResource(Cx.NS_CX + "location");
    var srcarc = rdfsvc.GetResource(Cx.NS_DC + "source");
    var res = rdfsvc.GetResource(item.id);
    var srcdoc = ds.GetSource(srcarc, res, true);

    try {
      // Check for scenes that point to this Location
      var scenes = bdds.GetSources(locarc, res, true);
      while (scenes.hasMoreElements()) {
        var scene = scenes.getNext().QueryInterface(IRes);
        bdds.Unassert(scene, locarc, res);
      }
      if (srcdoc) {
        srcdoc = srcdoc.QueryInterface(Components.interfaces.nsIRDFResource);
        top.removeDocument(srcdoc);
      }
    }
    catch (ex) {
      dump("*** deleteLocation: " + ex + "\n");
    }

    gWindow.locationlist.builder.rebuild();

    try {
      var numrows = gWindow.locationlist.getRowCount();
      if (numrows == 0) {
        this.addLocation();
        gWindow.locationlist.selectedIndex = 0;
      }
      else if (idx >= numrows) {
        gWindow.locationlist.selectedIndex = idx - 1;
      }
      else {
        gWindow.locationlist.selectedIndex = idx;
      }
    }
    catch (ex) {
      dump("*** deleteLocation: " + ex + "\n");
    }

    // gWindow.locationpopup.builder.rebuild();
  },


  arcForName: function arcForName (name) {
    switch (name) {
      case "title":
      case "description":
        return Cx.NS_DC + name;
      case "media":
        return Cx.NS_CX + "media";
      default:
        return gController.project.res.Value + "/NS/location-" + name;
    }
  },


  addLocation: function addLocation () {
    var ps = getPromptService();
    // TODO: i18n
    var title = { value: "" };
    var checkstate = { value: false };
    if (! ps.prompt(window, "Add Location", "Enter a new for this new location",
      title, null, checkstate))
      return;

    var ds = gController.project.ds;
    var rdfsvc = getRDFService();
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var locationres = rdfsvc.GetResource(Cx.NS_CX + "Location");
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var titlelit = rdfsvc.GetLiteral(title.value);
    var doctypearc = rdfsvc.GetResource(Cx.NS_CX + "doctype");
    var doctyperes = rdfsvc.GetResource(Cx.NS_CX + "LocationDocument");
    var categoryarc = rdfsvc.GetResource(Cx.NS_CX + "category");
    var sourcearc = rdfsvc.GetResource(Cx.NS_DC + "source");

    // Create the location item
    res = rdfsvc.GetResource(gController.project.mintURI());
    ds.Assert(res, typearc, locationres, true);
    ds.Assert(res, titlearc, titlelit, true);

    // Create the location document
    var folder = top.getSelectedDocumentFolder();
    ds.beginUpdateBatch();
    try {
      // TODO: i18n
      var docres = gController.project.createDocument(titlelit.Value,
        doctyperes, folder, res);
    }
    catch (ex) {
      dump("*** addLocation: " + ex + "\n");
    }
    ds.endUpdateBatch();
  },


  clearFields: function clearFields () {
    dump("--- clearFields\n");
    // gWindow.locationtitle.value = "";
    var IText = Components.interfaces.nsIDOMHTMLTextAreaElement;
    var IInput = Components.interfaces.nsIDOMHTMLInputElement;
    for (var name in this.nameNodeMap) {
      var node = this.nameNodeMap[name];
      if (node instanceof IText ||
         (node instanceof IInput && node.getAttribute("type") == "text")) {
        node.value = "";
      }
      else if (name == "media") {
        node.clear();
      }
      else if (node.__proto__ == Array.prototype) {
        node[0].checked = true;
        for (var i = 1; i < node.length; ++i) {
          node[i].checked = false;
        }
      }
    }
  },


  handleEvent: function handleEvent (event) {
    var loc = gWindow.locationlist.selectedItem;
    if (! loc) return;
    var elem = event.target;
    if (! elem.getAttribute("name")) return;
    var rdfsvc = getRDFService();
    var ds = gController.project.ds;
    var res = rdfsvc.GetResource(loc.id);
    var IText = Components.interfaces.nsIDOMHTMLTextAreaElement;
    var IInput = Components.interfaces.nsIDOMHTMLInputElement;
    for (var name in this.nameNodeMap) {
      var node = this.nameNodeMap[name];
      if (node != elem) continue;

      var arc = rdfsvc.GetResource(this.arcForName(name));

      if (node instanceof IText ||
         (node instanceof IInput && node.getAttribute("type") == "text")) {
        dump("--- setting " + name + " to " + node.value + "\n");
        setRDFString(ds, res, arc, node.value);
      }
      else if (name == "media") {
        dump("--- ignoring media change\n");
      }
      else if (node.__proto__ == Array.prototype) {
        dump("--- ignoring radio button change\n");
      }
      else {
        dump("*** Can't handle change for " + name + "\n");
      }
      break;
    }
  },


  onTitleChanged: function onTitleChanged () {
    var loc = gWindow.locationlist.selectedItem;
    if (! loc) return;
    var rdfsvc = getRDFService();
    var ds = gController.project.ds;
    var res = rdfsvc.GetResource(loc.id);
    var arc = rdfsvc.GetResource(Cx.NS_DC + "title");
    setRDFString(ds, res, arc, gWindow.locationtitle.value);
  },


  onLocationSelect: function onLocationSelect () {
    dump("--- onLocationSelect\n");
    if (gWindow.locationlist.selectedCount == 0) {
      if (this.lastSelection &&
          this.lastSelection.parentNode == gWindow.locationlist) {
        dump("--- Reverting selection\n");
        gWindow.locationlist.toggleItemSelection(this.lastSelection);
        // gWindow.locationlist.selectedItem = this.lastSelection;
        return;
      }
      document.getElementById("removelocation").disabled = true;
      return;
    }
    var item = gWindow.locationlist.selectedItem;
    if (item == this.lastSelection) return;
    this.clearFields();
    this.lastSelection = item;
    document.getElementById("removelocation").disabled = false;

    var IText = Components.interfaces.nsIDOMHTMLTextAreaElement;
    var IInput = Components.interfaces.nsIDOMHTMLInputElement;
    var ILit = Components.interfaces.nsIRDFLiteral;
    var IRes = Components.interfaces.nsIRDFResource;

    var rdfsvc = getRDFService();
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var filearc = rdfsvc.GetResource(Cx.NS_CX + "localFile");
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");

    var ds = gController.project.ds;
    var res = rdfsvc.GetResource(item.id);
    var loctitle = ds.GetTarget(res, titlearc, true);
    if (loctitle && loctitle instanceof ILit) {
      loctitle = loctitle.QueryInterface(ILit);
      // gWindow.locationtitle.value = loctitle.Value;
    }
    // gWindow.locationscenes.setAttribute("ref", item.id);
    /*
    var srcarc = rdfsvc.GetResource(Cx.NS_DC + "source");
    var docres = ds.GetSource(srcarc, res, true);
    if (docres && docres instanceof Components.interfaces.nsIRDFResource)
      res = docres.QueryInterface(Components.interfaces.nsIRDFResource);
    */
    for (var name in this.nameNodeMap) {
      var node = this.nameNodeMap[name];
      var arc = rdfsvc.GetResource(this.arcForName(name));
      var value = ds.GetTarget(res, arc, true);
      if (! value) {
        // dump("*** onLocationSelect: No value for " + name + "\n");
        continue;
      }
      if (value instanceof Components.interfaces.nsIRDFLiteral)
        value = value.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
      if (node instanceof IText ||
         (node instanceof IInput && node.getAttribute("type") == "text")) {
        node.value = value;
      }
      else if (name == "media") {
        var arc = rdfsvc.GetResource(this.arcForName(name));
        var seqres = ds.GetTarget(res, arc, true);
        if (seqres) {
          seqres = new RDFSeq(ds, seqres);
          var images = seqres.toArray();
          for (var i = 0; i < images.length; ++i) {
            var img = images[i].QueryInterface(IRes);
            var type = ds.GetTarget(img, typearc, true);
            if (! (type && type instanceof IRes))
              continue;
            type = type.QueryInterface(IRes);
            if (type.Value != Cx.NS_CX + "Image")
              continue;
            var title = ds.GetTarget(img, titlearc, true);
            if (title)
              title = title.QueryInterface(ILit).Value;
            else
              title = "";
            var file = ds.GetTarget(img, filearc, true);
            if (file) {
              file = file.QueryInterface(ILit);
              file = gController.project.fileURLOf(file.Value);
            }
            else
              file = "";
            var rec = {
              uri: img.Value,
              type: 'image',
              title: title,
              src: file
            };
            node.add(rec);
          }
        }
      }
      else if (node.__proto__ == Array.prototype) {
        for (var i = 0; i < node.length; ++i) {
          if (node[i].value == value)
            node[i].checked = true;
          else
            node[i].checked = false;
        }
      }
      else {
        dump("*** Don't know what to do with the " + name + " field.\n");
      }
    }
  }
};


var gScheduleController = {
  QueryInterface: function(aIID) {
    if (aIID.equals(Components.interfaces.nsISupports) ||
        aIID.equals(Components.interfaces.calICalendarViewController) ||
        aIID.equals(Components.interfaces.calIObserver)) {
      return this;
    }
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },


  supportsCommand: function supportsCommand (cmd) {
    switch (cmd) {
      case "cmd-print":
        return true;
      default:
        return false;
    }
  },


  isCommandEnabled: function isCommandEnabled (cmd) {
    switch (cmd) {
      case "cmd-print":
        return true;
      default:
        return false;
    }
  },


  doCommand: function doCommand (cmd) {
    switch (cmd) {
      case "cmd-print":
        this.print();
        break;
    }
  },


  scenemap: {},


  onOperationComplete: function (cal, status, action, itemid, detail) {
    if (status == Components.results.NS_OK)
      dump("--- onOperationComplete: Succeeded\n");
    else
      dump("*** onOperationComplete: " + detail + "\n");
  },


  onSceneChange: function (sceneres) {
    var mapentry = this.scenemap[sceneres.Value];
    if (! mapentry) {
      dump("--- No map entry for " + sceneid + "\n");
      return;
    }
    for (var i = 0; i < mapentry.length; ++i) {
      var item = mapentry[i].clone();
      
      // item.title = this.getSceneHeader(sceneres.Value);
      // dump("--- Changing item title from " + mapentry[i].title + " to "
      //   + item.title + "\n");
      var tmpevent = this.createEventForScene(sceneres.Value);
      item.title = tmpevent.title;
      if (tmpevent.hasProperty("DESCRIPTION"))
        item.setProperty("DESCRIPTION", tmpevent.getProperty("DESCRIPTION"));
      if (tmpevent.hasProperty("LOCATION"))
        item.setProperty("LOCATION", tmpevent.getProperty("LOCATION"));
      doTransaction("modify", item, this.calendar, mapentry[i], this);
    }
  },


  // calIObserver methods (for synchronizing the rdf data source)
  onStartBatch: function () {},
  onEndBatch: function () {},
  onLoad: function () {
    this.loading = false;
    // Update the datasource
    var IInt = Components.interfaces.nsIRDFInt;
    var ILit = Components.interfaces.nsIRDFLiteral;
    var IRes = Components.interfaces.nsIRDFResource;
    var rdfsvc = getRDFService();
    var scenesarc = rdfsvc.GetResource(Cx.NS_CX + "scenes");
    var scheduledarc = rdfsvc.GetResource(Cx.NS_CX + "scheduled");
    var isscheduledarc = rdfsvc.GetResource(Cx.NS_CX + "isscheduled");
    var truelit = rdfsvc.GetLiteral("true");
    var bdds = gController.bdds;
    var scenesres = bdds.GetTarget(gController.docres, scenesarc, true);
    var scenes = new RDFSeq(bdds, scenesres).toArray();
    bdds.beginUpdateBatch();
    for (var i = 0; i < scenes.length; ++i) {
      var scene = scenes[i].QueryInterface(IRes);
      // Correct previous attempts at tracking info in RDF
      var isscheduled = bdds.GetTarget(scene, isscheduledarc, true);
      if (isscheduled)
        bdds.Unassert(scene, isscheduledarc, isscheduled);
      var scheduled = bdds.GetTarget(scene, scheduledarc, true);
      if (this.scenemap[scene.Value]) {
        if (scheduled && scheduled instanceof IInt)
          bdds.Change(scene, scheduledarc, scheduled, truelit);
        else if (! scheduled)
          bdds.Assert(scene, scheduledarc, truelit, true);
      }
      else {
        if (scheduled)
          bdds.Unassert(scene, scheduledarc, scheduled);
      }
    }
    bdds.endUpdateBatch();
  },
  onModifyItem: function (item, oldItem) {
    var calIEvent = Components.interfaces.calIEvent;
    if (! (item instanceof calIEvent && item.hasProperty("X-CELTX-SCENE")))
      return;
    item = item.QueryInterface(calIEvent);
    var sceneid = item.getProperty("X-CELTX-SCENE");
    var mapentry = this.scenemap[sceneid];
    if (! mapentry) {
      dump("*** onModifyItem: No map entry for " + sceneid + "\n");
      return;
    }
    for (var i = 0; i < mapentry.length; ++i) {
      if (mapentry[i].id == oldItem.id) {
        mapentry[i] = item;
        break;
      }
    }
  },
  onError: function  (aErrNo, aMessage) {
    dump("*** gScheduleController.onError: " + aErrNo + ", " + aMessage + "\n");
    var frame = Components.stack.caller;
    while (frame) {
      dump("    " + frame + "\n");
      frame = frame.caller;
    }
  },


  onAddItem: function (item) {
    var calIEvent = Components.interfaces.calIEvent;
    if (! (item instanceof calIEvent && item.hasProperty("X-CELTX-SCENE")))
      return;
    item = item.QueryInterface(calIEvent);

    var sceneid = item.getProperty("X-CELTX-SCENE");
    if (! this.scenemap[sceneid])
      this.scenemap[sceneid] = new Array();
    var mapentry = this.scenemap[sceneid];
    mapentry.push(item);

    if (this.loading)
      return;

    var ds = gController.bdds;
    var nsIRDFInt = Components.interfaces.nsIRDFInt;
    var rdfsvc = getRDFService();
    var scheduledarc = rdfsvc.GetResource(Cx.NS_CX + "scheduled");
    var scene = rdfsvc.GetResource(sceneid);
    var truelit = rdfsvc.GetLiteral("true");

    var scheduled = ds.GetTarget(scene, scheduledarc, true);
    if (! scheduled) {
      gController.saveTreeSelection();
      ds.beginUpdateBatch();
      ds.Assert(scene, scheduledarc, truelit, true);
      ds.endUpdateBatch();
      gController.restoreTreeSelection();
    }
  },


  onDeleteItem: function (item) {
    var calIEvent = Components.interfaces.calIEvent;
    if (! (item instanceof calIEvent && item.hasProperty("X-CELTX-SCENE")))
      return;
    item = item.QueryInterface(calIEvent);

    var sceneid = item.getProperty("X-CELTX-SCENE");
    var mapentry = this.scenemap[sceneid];
    if (mapentry) {
      for (var i = 0; i < mapentry.length; ++i) {
        if (mapentry[i].id == item.id) {
          mapentry.splice(i, 1);
          break;
        }
      }
    }

    if (this.loading)
      return;

    var ds = gController.bdds;
    var nsIRDFInt = Components.interfaces.nsIRDFInt;
    var rdfsvc = getRDFService();
    var scheduledarc = rdfsvc.GetResource(Cx.NS_CX + "scheduled");
    var scene = rdfsvc.GetResource(item.getProperty("X-CELTX-SCENE"));

    var scheduled = ds.GetTarget(scene, scheduledarc, true);
    if (scheduled && (! mapentry || mapentry.length == 0)) {
      gController.saveTreeSelection();
      ds.beginUpdateBatch();
      ds.Unassert(scene, scheduledarc, scheduled);
      ds.endUpdateBatch();
      gController.restoreTreeSelection();
    }
  },


  // Modified version of nsDragAndDrop.startDrag with additional |dragrect|
  // field for specifying the drag rect in the |action| parameter
  startDrag: function (aEvent) {
    const kDSIID = Components.interfaces.nsIDragService;
    var dragAction = {
      action: kDSIID.DRAGDROP_ACTION_COPY |
              kDSIID.DRAGDROP_ACTION_MOVE |
              kDSIID.DRAGDROP_ACTION_LINK,
      dragrect: null
    };
    var transferData = { data: null };
    try {
      this.onDragStart(aEvent, transferData, dragAction);
    }
    catch (ex) { return; }
    if (! transferData.data) return;
    transferData = transferData.data;

    var transArray = createSupportsArray();
    var count = 0;
    do {
      var trans = nsTransferable.set(transferData._XferID == "TransferData"
        ? transferData : transferData.dataList[count++]);
      transArray.AppendElement(trans.QueryInterface(
        Components.interfaces.nsISupports));
    }
    while (transferData.XferID == "TransferDataSet" &&
           count < transferData.dataList.length);
    try {
      nsDragAndDrop.mDragService.invokeDragSession(aEvent.target, transArray,
        dragAction.dragrect, dragAction.action);
    }
    catch (ex) {}
    aEvent.stopPropagation();
  },


  onDragStart: function (event, xferData, action) {
    var target = event.currentTarget;

    if (target.id == "movingitem") {
      xferData.data = new TransferData();
      xferData.data.addDataForFlavour("x-celtx/x-banner", "moving");
    }
    else if (target.id == "dayoffitem") {
      xferData.data = new TransferData();
      xferData.data.addDataForFlavour("x-celtx/x-banner", "dayoff");
    }
    else {
      dump("*** invalid drag from " + event.currentTarget.localName + "\n");
      return;
    }

    if (target != event.target) {
      var region = Components.classes["@mozilla.org/gfx/region;1"]
        .createInstance(Components.interfaces.nsIScriptableRegion);
      region.init();
      var bo = target.boxObject;
      region.setToRect(bo.x, bo.y, bo.width, bo.height);
      action.dragrect = region;
    }
  },


  loaded: function loaded () {
  },


  opened: function opened () {
    try {
      var calfile = gController.project.projectFolder;
      var ILit = Components.interfaces.nsIRDFLiteral;
      var rdfsvc = getRDFService();
      var ds = gController.project.ds;
      var calarc = rdfsvc.GetResource(Cx.NS_CX + "calendar");
      var calsrc = ds.GetTarget(gController.docres, calarc, true);
      var validFile = false;
      if (calsrc) {
        // This might be a full file uri, an early 099 mistake. Correct it
        // if it exists. This will not be necessary in the release.
        try {
          calfile.append(calsrc.QueryInterface(ILit).Value);
          if (isReadableFile(calfile))
            validFile = true;
        }
        catch (ex) {}
      }
      if (! validFile) {
        calfile = gController.project.projectFolder;
        calfile.append("production.ics");
        calfile.createUnique(0, 0644 & calfile.parent.permissions);
        if (calsrc) {
          var newcalsrc = rdfsvc.GetLiteral(calfile.leafName);
          ds.Change(gController.docres, calarc, calsrc, newcalsrc);
          calsrc = newcalsrc;
        }
        else {
          calsrc = rdfsvc.GetLiteral(calfile.leafName);
          ds.Assert(gController.docres, calarc, calsrc, true);
        }
      }
      var ios = getIOService();
      var calmgr = Components.classes["@mozilla.org/calendar/manager;1"]
        .getService(Components.interfaces.calICalendarManager);
      calsrc = ios.newURI(fileToFileURL(calfile), null, null);
      this.loading = true;
      this.calendar = calmgr.createCalendar("ics", calsrc);
      this.calendar.addObserver(this);

      var startMinute = 8 * 60; // 8:00 am
      try {
        var pref = getPrefService().getBranch("celtx.calendar.");
        var startDate = new Date();
        var startSecs = pref.getIntPref("daystart");
        startDate.setHours(Math.floor(startSecs / 3600),
          Math.floor((startSecs % 3600) / 60), startSecs % 60);
        startMinute = startDate.getHours() * 60 + startDate.getMinutes();
      }
      catch (ex) {
        dump("*** gScheduleController.loaded: " + ex + "\n");
      }

      gWindow.monthview.displayCalendar = this.calendar;
      gWindow.monthview.timezone = calendarDefaultTimezone();
      gWindow.monthview.controller = gScheduleController;
      gWindow.monthview.goToDay(now());
      gWindow.weekview.displayCalendar = this.calendar;
      gWindow.weekview.timezone = calendarDefaultTimezone();
      gWindow.weekview.controller = gScheduleController;
      gWindow.weekview.goToDay(now());
      gWindow.dayview.displayCalendar = this.calendar;
      gWindow.dayview.timezone = calendarDefaultTimezone();
      gWindow.dayview.controller = gScheduleController;
      gWindow.dayview.goToDay(now());
      var innerWeek = document.getAnonymousElementByAttribute(gWindow.weekview,
        "anonid", "view-element");
      var innerDay = document.getAnonymousElementByAttribute(gWindow.dayview,
        "anonid", "view-element");
      innerWeek.scrollToMinute(startMinute);
      innerDay.scrollToMinute(startMinute);
    }
    catch (ex) {
      dump("*** gScheduleController.loaded: " + ex + "\n");
    }
  },


  focus: function focus () {
    // this.frame.setAttribute("type", "content-primary");
  },


  blur: function blur () {
    // this.frame.setAttribute("type", "content");
  },


  print: function print () {
    calPrint();
  },


  createEventForScene: function createEventForScene (sceneuri) {
    var rdfsvc = getRDFService();
    var IRes = Components.interfaces.nsIRDFResource;
    var ILit = Components.interfaces.nsIRDFLiteral;
    var IInt = Components.interfaces.nsIRDFInt;
    var sceneres = rdfsvc.GetResource(sceneuri);
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var descarc = rdfsvc.GetResource(Cx.NS_DC + "description");
    var intextarc = rdfsvc.GetResource(Cx.NS_CX + "intext");
    var daynightarc = rdfsvc.GetResource(Cx.NS_CX + "daynight");
    var ordarc = rdfsvc.GetResource(Cx.NS_CX + "ordinal");
    var locationarc = rdfsvc.GetResource(Cx.NS_CX + "location");

    var ds = gController.project.ds;
    var bdds = gController.bdds;

    var title = bdds.GetTarget(sceneres, titlearc, true);
    title = title.QueryInterface(ILit).Value;
    var intext = bdds.GetTarget(sceneres, intextarc, true);
    intext = intext.QueryInterface(ILit).Value;
    var daynight = bdds.GetTarget(sceneres, daynightarc, true);
    daynight = daynight.QueryInterface(ILit).Value;
    var ord = bdds.GetTarget(sceneres, ordarc, true);
    ord = ord.QueryInterface(IInt).Value;
    var desc = bdds.GetTarget(sceneres, descarc, true);
    if (desc)
      desc = desc.QueryInterface(ILit).Value;
    else
      desc = "";
    var loctitle = "";
    var location = bdds.GetTarget(sceneres, locationarc, true);
    if (location) {
      location = location.QueryInterface(IRes);
      var tmploctitle = ds.GetTarget(location, titlearc, true);
      loctitle = tmploctitle.QueryInterface(ILit).Value;
      dump("--- createEventForScene: loctitle == " + loctitle + "\n");
    }
    else
      dump("--- No cx:location for " + sceneres.Value + "\n");

    var event = Components.classes["@mozilla.org/calendar/event;1"]
      .createInstance(Components.interfaces.calIEvent);
    event.title = ord + " " + intext + ". " + title + " - " + daynight;
    event.setProperty("DESCRIPTION", desc);
    event.setProperty("LOCATION", loctitle);
    event.setProperty("X-CELTX-SCENE", sceneuri);
    return event;
  },


  getSceneHeader: function (aSceneURI) {
    try {
      var rdfsvc = getRDFService();
      var IRes = Components.interfaces.nsIRDFResource;
      var ILit = Components.interfaces.nsIRDFLiteral;
      var IInt = Components.interfaces.nsIRDFInt;
      var sceneres = rdfsvc.GetResource(aSceneURI);
      var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
      var intextarc = rdfsvc.GetResource(Cx.NS_CX + "intext");
      var daynightarc = rdfsvc.GetResource(Cx.NS_CX + "daynight");
      var ordarc = rdfsvc.GetResource(Cx.NS_CX + "ordinal");

      var title = gController.bdds.GetTarget(sceneres, titlearc, true);
      title = title.QueryInterface(ILit);
      var intext = gController.bdds.GetTarget(sceneres, intextarc, true);
      intext = intext.QueryInterface(ILit);
      var daynight = gController.bdds.GetTarget(sceneres, daynightarc, true);
      daynight = daynight.QueryInterface(ILit);
      var ord = gController.bdds.GetTarget(sceneres, ordarc, true);
      ord = ord.QueryInterface(IInt);
      return ord.Value + " " + intext.Value + ". " + title.Value
        + " - " + daynight.Value;
    }
    catch (ex) {
      dump("*** getSceneHeader: " + ex + "\n");
      return "Some Scene";
    }
  },


  createNewEvent: function (aCalendar, aStartTime, aEndTime) {
    // XXX If we're adding an item from the view, let's make sure that
    // XXX the calendar in question is visible!
    // XXX unify these
    if (!aCalendar) {
      /*
      if ("ltnSelectedCalendar" in window) {
          aCalendar = ltnSelectedCalendar();
      } else {
          aCalendar = getSelectedCalendarOrNull();
      }
      */
      dump("*** createNewEvent: aCalendar not specified\n");
      return;
    }

    // if we're given both times, skip the dialog
    if (aStartTime && aEndTime && !aStartTime.isDate && !aEndTime.isDate) {
      var event = createEvent();
      event.startDate = aStartTime;
      event.endDate = aEndTime;
      var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"]
                          .getService(Components.interfaces.nsIStringBundleService);
      var props = sbs.createBundle("chrome://calendar/locale/calendar.properties");
      event.title = props.GetStringFromName("newEvent");
      setDefaultAlarmValues(event);
      doTransaction('add', event, aCalendar, null, null);
    } else if (aStartTime && aStartTime.isDate) {
      var event = createEvent();
      event.startDate = aStartTime;
      setDefaultAlarmValues(event);
      doTransaction('add', event, aCalendar, null, null);
    } else {
      // default pop up the dialog
      var date;
      if (aStartTime) {
        date = aStartTime;
      } else {
        date = gWindow.monthview.selectedDay.clone();
        date.isDate = false;
      }
      try {
        createEventWithDialog(aCalendar, date, null);
      }
      catch (ex) {
        dump("*** createEventWithDialog: " + ex + "\n");
      }
    }
  },

  modifyOccurrence: function (aOccurrence, aNewStartTime,
                              aNewEndTime, aNewTitle) {
    // if modifying this item directly (e.g. just dragged to new time),
    // then do so; otherwise pop up the dialog
    if ((aNewStartTime && aNewEndTime) || aNewTitle) {
      var instance = aOccurrence.clone();

      if (aNewTitle) {
        instance.title = aNewTitle;
      }

      if (aNewStartTime) { // we know we have aEndTime too then
        instance.startDate = aNewStartTime;
        instance.endDate = aNewEndTime;
      }
      doTransaction('modify', instance, instance.calendar, aOccurrence, null);
    } else {
      var args = {
        calendarEvent: aOccurrence,
        calendar: gWindow.monthview.displayCalendar,
        mode: "modify",
        onOk: function (item, calendar, originalItem) {
          doTransaction('modify', item, item.calendar, originalItem, null);
        }
      };
      if (aOccurrence.hasProperty("X-CELTX-SCENE"))
        dump("--- Opening a scene item\n");
      else
        dump("--- Opening a non-scene item\n");
      openDialog(Cx.CONTENT_PATH + "editors/eventdialog.xul", "_blank",
        "chrome,titlebar,modal,resizable", args);
    }
  },

  deleteOccurrence: function (aOccurrence) {
    var itemToDelete = getOccurrenceOrParent(aOccurrence);
    if (!itemToDelete) {
      return;
    }
    if (!itemToDelete.parentItem.hasSameIds(itemToDelete)) {
      var event = itemToDelete.parentItem.clone();
      event.recurrenceInfo.removeOccurrenceAt(itemToDelete.recurrenceId);
      doTransaction('modify', event, event.calendar,
        itemToDelete.parentItem, null);
    }
    else {
      doTransaction('delete', itemToDelete, itemToDelete.calendar, null, null);
    }
  },


  switchView: function switchView (view) {
    var selectedDay = gWindow.scheduledeck.selectedPanel.selectedDay;
    switch (view) {
      case "month":
        gWindow.scheduledeck.selectedPanel = gWindow.monthview;
        gWindow.monthview.goToDay(selectedDay);
        break;
      case "week":
        gWindow.scheduledeck.selectedPanel = gWindow.weekview;
        gWindow.weekview.goToDay(selectedDay);
        break;
      case "day":
        gWindow.scheduledeck.selectedPanel = gWindow.dayview;
        gWindow.dayview.goToDay(selectedDay);
        break;
    }
  }
};

function setOutlineView (outline) {
  try {
    gController.outline = outline;
    var bdtoolbar = outline.document.getElementById("bdtoolbar");
    bdtoolbar.addEventListener("select", gController, false);
  }
  catch (ex) {
    dump("*** setOutlineView: " + ex + "\n");
  }
}

function getCalStringBundle()
{
    var strBundleService = 
        Components.classes["@mozilla.org/intl/stringbundle;1"]
                  .getService(Components.interfaces.nsIStringBundleService);
    return strBundleService.createBundle("chrome://calendar/locale/calendar.properties");
}


// For calPrint()
function currentView () {
  return gWindow.scheduledeck.selectedPanel;
}


// For calPrint()
function getCompositeCalendar () {
  return gScheduleController.calendar;
}


function showError(aMsg)
{
    /*
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                  .getService(Components.interfaces.nsIPromptService);
    promptService.alert(null,
                        getCalStringBundle().GetStringFromName('errorTitle'),
                        aMsg);
    */
  var msg = gApp.getText("CalendarImportFailedMsg");
  celtxBugAlert(msg, Components.stack.caller, aMsg); 
}


var gDeptController = {
  loaded: function loaded () {
    this.itemlist = document.getElementById("itemlist");
    this.frame    = document.getElementById("deptframe");
  },


  opened: function opened () {
    this.itemlist.database.AddDataSource(gController.deptds);
    this.itemlist.database.AddDataSource(gController.project.ds);
    // this.itemlist.builder.rebuild();
  },


  focus: function focus () {
    this.frame.setAttribute("type", "content-primary");
  },


  blur: function blur () {
    this.frame.setAttribute("type", "content");
  },


  onItemSelected: function onItemSelected () {
    // Save the current frame
    if (this.frame.getAttribute("src") != "about:blank") {
      try {
        var ctrl = this.frame.contentWindow.getController();
        ctrl.save();
      }
      catch (ex) {
        dump("*** onItemSelected: " + ex + "\n");
      }
    }

    // Clear the frame so loading the same form won't go ignored
    this.frame.setAttribute("src", "about:blank");
    var sel = this.itemlist.view.selection;
    if (sel.count == 0)
      return;

    var ds = gController.project.ds;
    var rdfsvc = getRDFService();
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var srcarc = rdfsvc.GetResource(Cx.NS_DC + "source");
    // var itemres = rdfsvc.GetResource(this.itemlist.selectedItem.id);
    var itemres = this.itemlist.view.getResourceAtIndex(sel.currentIndex);
    var typeres = ds.GetTarget(itemres, typearc, true);
    if (! typeres) {
      dump("*** onItemSelected: No type for " + itemres.Value + "\n");
      return;
    }
    typeres = typeres.QueryInterface(Components.interfaces.nsIRDFResource);
    var docres = ds.GetSource(srcarc, itemres, true);
    if (! docres) {
      dump("*** onItemSelected: No source doc for " + itemres.Value + "\n");
      return;
    }
    docres = docres.QueryInterface(Components.interfaces.nsIRDFResource);
    this.pendingDocument = docres;
    var formname = null;
    switch (typeres.Value) {
      case Cx.NS_CX + "Actor":
        formname = "actor";
        break;
      case Cx.NS_CX + "Cast":
        formname = "character";
        break;
      case Cx.NS_CX + "Location":
        formname = "location";
        break;
      case Cx.NS_CX + "Props":
        formname = "prop";
        break;
      case Cx.NS_CX + "Wardrobe":
        formname = "wardrobe";
        break;
      default:
        formname = "generic";
    }
    this.frame.setAttribute("src", Cx.CONTENT_PATH + "editors/"
      + formname + ".xul");

    setTimeout("gDeptController.checkLoaded()", 100);
  },


  checkLoaded: function checkLoaded () {
    if (this.frame.docShell.busyFlags) {
      setTimeout("gDeptController.checkLoaded()", 100);
      return;
    }
    var ctrl = this.frame.contentWindow.getController();
    ctrl.open(gController.project, this.pendingDocument);
  }
};
