var gWindow = {};


function loaded () {
  gWindow.scenetree     = document.getElementById("scenelist");
  gWindow.monthview     = document.getElementById("schedulemonthview");
  gWindow.weekview      = document.getElementById("scheduleweekview");
  gWindow.dayview       = document.getElementById("scheduledayview");
  gWindow.scheduledeck  = document.getElementById("scheduleviewdeck");

  window.controllers.appendController(gScheduleController);

  gWindow.cycleObserver = new TreeHeaderCycleObserver(gWindow.scenetree);
}


function getController () {
  return gScheduleController;
}


var gScheduleController = {
  QueryInterface: function(aIID) {
    if (aIID.equals(Components.interfaces.nsISupports) ||
        aIID.equals(Components.interfaces.nsIDOMEventListener) ||
        aIID.equals(Components.interfaces.nsIObserver) ||
        aIID.equals(Components.interfaces.calICalendarViewController) ||
        aIID.equals(Components.interfaces.calIObserver)) {
      return this;
    }
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },


  scenemap: {},


  supportsCommand: function supportsCommand (cmd) {
    switch (cmd) {
      case "cmd-page-setup":
      case "cmd-print":
        return true;
      default:
        return false;
    }
  },


  isCommandEnabled: function isCommandEnabled (cmd) {
    switch (cmd) {
      case "cmd-page-setup":
      case "cmd-print":
        return true;
      default:
        return false;
    }
  },


  doCommand: function doCommand (cmd) {
    switch (cmd) {
      case "cmd-page-setup":
        PrintUtils.showPageSetup();
        break;
      case "cmd-print":
        this.print();
        break;
    }
  },


  open: function open (project, docres) {
    this.project = project;
    this.docres = docres;

    var IRes = Components.interfaces.nsIRDFResource;
    var rdfsvc = getRDFService();
    var srcarc = rdfsvc.GetResource(Cx.NS_DC + "source");
    this.source = this.project.ds.GetTarget(this.docres, srcarc, true);
    if (! this.source) {
      dump("*** gScheduleController.open: Not linked to a source script\n");
      return;
    }
    this.source = this.source.QueryInterface(IRes);
    this.ds = getInMemoryDataSource();
    gWindow.scenetree.database.AddDataSource(this.project.ds);
    gWindow.scenetree.database.AddDataSource(this.ds);
    gWindow.scenetree.ref = this.source.Value;

    this.outline.setDelegate(window);

    var ps = getPrefService();
    ps.addObserver("celtx.calendar.workdaysonly", this, false);
    ps = ps.getBranch("celtx.calendar.");
    var workdaysOnly = ps.getBoolPref("workdaysonly");
    gWindow.monthview.workdaysOnly = workdaysOnly;
    gWindow.weekview.workdaysOnly = workdaysOnly;
    gWindow.dayview.workdaysOnly = workdaysOnly;

    try {
      var forceLoad = false;
      var calfile = this.project.fileForResource(this.docres);
      if (! isReadableFile(calfile)) {
        calfile = this.project.projectFolder;
        calfile.append("schedule.ics");
        calfile.createUnique(0, 0644 & calfile.parent.permissions);
        this.project.addFileToDocument(calfile, this.docres);
        forceLoad = true;
      }
      this.loading = true;
      var ios = getIOService();
      var calmgr = Components.classes["@mozilla.org/calendar/manager;1"]
        .getService(Components.interfaces.calICalendarManager);
      calsrc = ios.newURI(fileToFileURL(calfile), null, null);
      this.initCalendarManager();
      this.calendar = calmgr.createCalendar("ics", calsrc);
      this.calendar.addObserver(this);
      if (forceLoad)
        this.onLoad();

      var startMinute = 8 * 60; // 8:00 am
      try {
        var pref = getPrefService().getBranch("celtx.calendar.");
        var startDate = new Date();
        var startSecs = pref.getIntPref("daystart");
        startDate.setHours(Math.floor(startSecs / 3600),
          Math.floor((startSecs % 3600) / 60), startSecs % 60);
        startMinute = startDate.getHours() * 60 + startDate.getMinutes();
      }
      catch (ex) {
        dump("*** gScheduleController.open: " + ex + "\n");
      }

      gWindow.minimonth = this.outline.document.getElementById("minimonth");
      if (gWindow.minimonth) {
        gWindow.minimonth.value = new Date();
        gWindow.minimonth.addEventListener("change", this, false);
      }

      gWindow.monthview.displayCalendar = this.calendar;
      gWindow.monthview.timezone = calendarDefaultTimezone();
      gWindow.monthview.controller = gScheduleController;
      gWindow.monthview.goToDay(now());
      gWindow.weekview.displayCalendar = this.calendar;
      gWindow.weekview.timezone = calendarDefaultTimezone();
      gWindow.weekview.controller = this;
      gWindow.weekview.goToDay(now());
      gWindow.dayview.displayCalendar = this.calendar;
      gWindow.dayview.timezone = calendarDefaultTimezone();
      gWindow.dayview.controller = this;
      gWindow.dayview.goToDay(now());
      var innerWeek = document.getAnonymousElementByAttribute(gWindow.weekview,
        "anonid", "view-element");
      var innerDay = document.getAnonymousElementByAttribute(gWindow.dayview,
        "anonid", "view-element");
      innerWeek.scrollToMinute(startMinute);
      innerDay.scrollToMinute(startMinute);

      gWindow.scheduledeck.addEventListener("dayselect", this, false);

      var outlineDoc = this.outline.document;
      gWindow.agendaTree = outlineDoc.getElementById("agenda-tree");
      gWindow.todoList = outlineDoc.getElementById("calendar-todo-list");
      setAgendaTreeView();
      initializeTodoList();
    }
    catch (ex) {
      dump("*** gScheduleController.open: " + ex + "\n");
    }
  },


  initCalendarManager: function initCalendarManager () {
    var mgr = Components.classes["@mozilla.org/calendar/manager;1"]
      .getService(Components.interfaces.calICalendarManager);

    if (mgr.getCalendars({}).length > 0)
      return;

    var homeURL = getIOService().newURI("moz-profile-calendar://", null, null);
    var homeCalendar = mgr.createCalendar("storage", homeURL);
    mgr.registerCalendar(homeCalendar);
    homeCalendar.name = "Home"; // Don't bother localizing, it's bogus anyway
  },


  close: function close () {
    gWindow.scenetree.database.RemoveDataSource(this.project.ds);
    gWindow.scenetree.database.RemoveDataSource(this.ds);
    var ps = getPrefService();
    ps.removeObserver("celtx.calendar.workdaysonly", this);
  },


  save: function save () {
    this._modified = false;
  },


  get modified () {
    return this._modified;
  },


  get selectedScene () {
    try {
      var idx = gWindow.scenetree.view.selection.currentIndex;
      if (idx < 0)
        return null;
      return gWindow.scenetree.view.getResourceAtIndex(idx);
    }
    catch (ex) {
      dump("*** selectedScene: " + ex + "\n");
      return null;
    }
  },


  set selectedScene (val) {
    try {
      var idx = gWindow.scenetree.view.getIndexOfResource(val);
      if (idx >= 0)
        gWindow.scenetree.view.selection.select(idx);
      return idx;
    }
    catch (ex) {
      dump("*** set selectedScene: " + ex + "\n");
      return -1;
    }
  },


  handleEvent: function handleEvent (event) {
    if (event.type == "change") {
      this.minimonthPick(event.target);
    }
    else if (event.type == "dayselect") {
      this.observeViewDaySelect(event);
    }
  },


  minimonthPick: function minimonthPick (minimonth) {
    var jsDate = minimonth.value;
    var cdt = new CalDateTime();
    cdt.year = jsDate.getFullYear();
    cdt.month = jsDate.getMonth();
    cdt.day = jsDate.getDate();
    cdt.isDate = true;
    cdt.timezone = currentView().timezone;
    currentView().goToDay(cdt);
  },


  observeViewDaySelect: function observeViewDaySelect (event) {
    var date = event.detail;
    var jsDate = new Date(date.year, date.month, date.day);

    // for the month and multiweek view find the main month,
    // which is the month with the most visible days in the view;
    // note, that the main date is the first day of the main month
    var jsMainDate;
    if (!event.originalTarget.supportsDisjointDates) {
      var mainDate = null;
      var maxVisibleDays = 0;
      var startDay = currentView().startDay;
      var endDay = currentView().endDay;
      var firstMonth = startDay.startOfMonth;
      var lastMonth = endDay.startOfMonth;
      for (var month = firstMonth.clone(); month.compare(lastMonth) <= 0;
           month.month += 1, month.normalize()) {
        var visibleDays = 0;
        if (month.compare(firstMonth) == 0) {
            visibleDays = startDay.endOfMonth.day - startDay.day + 1;
        } else if (month.compare(lastMonth) == 0) {
            visibleDays = endDay.day;
        } else {
            visibleDays = month.endOfMonth.day;
        }
        if (visibleDays > maxVisibleDays) {
            mainDate = month.clone();
            maxVisibleDays = visibleDays;
        }
      }
      jsMainDate = new Date(mainDate.year, mainDate.month, mainDate.day);
    }

    if (gWindow.minimonth)
      gWindow.minimonth.selectDate(jsDate, jsMainDate);
    currentView().focus();
  },


  observe: function observe (subject, topic, data) {
    var ps = getPrefService().getBranch("celtx.calendar.");
    var workdaysOnly = ps.getBoolPref("workdaysonly");
    gWindow.monthview.workdaysOnly = workdaysOnly;
    gWindow.weekview.workdaysOnly = workdaysOnly;
    gWindow.dayview.workdaysOnly = workdaysOnly;
    // Force refresh of current view
    var curview = gWindow.scheduledeck.selectedPanel;
    curview.goToDay(curview.selectedDay);
  },


  isValidScene: function isValidScene (scene) {
    var rdfsvc = getRDFService();
    var projds = this.project.ds;
    var scenesarc = rdfsvc.GetResource(Cx.NS_CX + "scenes");
    var scenesres = projds.GetTarget(this.source, scenesarc, true);
    if (! scenesres)
      return false;
    scenesres = scenesres.QueryInterface(Components.interfaces.nsIRDFResource);
    var cu = getRDFContainerUtils();
    var idx = cu.indexOf(projds, scenesres, scene);
    return cu.indexOf(projds, scenesres, scene) >= 0;
  },


  onSceneDblClick: function onSceneDblClick (event) {
    if (event.originalTarget.localName != "treechildren")
      return;
    var scene = this.selectedScene;
    if (! scene)
      return;
    var rdfsvc = getRDFService();
    // var IInt = Components.interfaces.nsIRDFInt;
    var ds = this.project.ds;
    var ordarc = rdfsvc.GetResource(Cx.NS_CX + "ordinal");
    var intextarc = rdfsvc.GetResource(Cx.NS_CX + "intext");
    var settingarc = rdfsvc.GetResource(Cx.NS_CX + "setting");
    var daynightarc = rdfsvc.GetResource(Cx.NS_CX + "daynight");
    var descarc = rdfsvc.GetResource(Cx.NS_DC + "description");
    var locationarc = rdfsvc.GetResource(Cx.NS_CX + "location");

    // var ord = ds.GetTarget(scene, ordarc, true);
    // ord = ord ? ord.QueryInterface(IInt).Value : "";
    var ord = getRDFString(ds, scene, ordarc);
    var intext = getRDFString(ds, scene, intextarc);
    var setting = getRDFString(ds, scene, settingarc);
    var daynight = getRDFString(ds, scene, daynightarc);
    var desc = getRDFString(ds, scene, descarc);
    var location = getRDFString(ds, scene, locationarc);

    var scheduled = false;
    var completed = false;
    var events = this.scenemap[scene.Value];
    if (events) {
      scheduled = true;
      completed = true;
      for (var i = 0; i < events.length; ++i) {
        if (! events[i].hasProperty("X-CELTX-COMPLETED")) {
          completed = false;
          break;
        }
      }
    }

    var args = {
      accepted: false,
      ord: ord,
      intext: intext,
      setting: setting,
      daynight: daynight,
      desc: desc,
      location: location,
      scheduled: scheduled,
      completed: completed
    };
    window.openDialog(Cx.CONTENT_PATH + "editors/editscene.xul", "_blank",
      "chrome,titlebar,modal", args);
    if (! args.accepted)
      return;
    setRDFString(ds, scene, descarc, args.desc);
    if (! args.location)
      args.location = "";
    setRDFStringAllowEmpty(ds, scene, locationarc, args.location);
    if (events) {
      for (var i = 0; i < events.length; ++i)
        this.updateSceneEvent(events[i]);
    }
  },


  onOperationComplete: function (cal, status, action, itemid, detail) {
    if (status != Components.results.NS_OK)
      dump("*** onOperationComplete: " + detail + "\n");
  },


  onSceneDrag: function onSceneDrag (event) {
    if (event.originalTarget.localName == "treechildren")
      nsDragAndDrop.startDrag(event, this);
  },


  // calIObserver methods (for synchronizing the rdf data source)
  onStartBatch: function () {},
  onEndBatch: function () {},
  onLoad: function () {
    this.loading = false;
    // Update the datasource
    // var IInt = Components.interfaces.nsIRDFInt;
    var ILit = Components.interfaces.nsIRDFLiteral;
    var IRes = Components.interfaces.nsIRDFResource;
    var rdfsvc = getRDFService();
    var scenesarc = rdfsvc.GetResource(Cx.NS_CX + "scenes");
    var scheduledarc = rdfsvc.GetResource(Cx.NS_CX + "scheduled");
    var completedarc = rdfsvc.GetResource(Cx.NS_CX + "completed");
    var ds = this.ds;
    var projds = this.project.ds;
    var scenesres = projds.GetTarget(this.source, scenesarc, true);
    var scenes = new RDFSeq(projds, scenesres).toArray();
    ds.beginUpdateBatch();
    for (var i = 0; i < scenes.length; ++i) {
      var scene = scenes[i].QueryInterface(IRes);
      var entry = this.scenemap[scene.Value];
      if (entry) {
        setRDFString(ds, scene, scheduledarc, "true");
        var shouldBeComplete = (entry.length > 0);
        for (var j = 0; j < entry.length; ++j) {
          if (! entry[j].hasProperty("X-CELTX-COMPLETED")) {
            shouldBeComplete = false;
            break;
          }
        }
        if (shouldBeComplete)
          setRDFString(ds, scene, completedarc, "true");
        else
          setRDFString(ds, scene, completedarc, "false");
      }
      else {
        setRDFString(ds, scene, scheduledarc, "false");
        setRDFString(ds, scene, completedarc, "false");
      }
    }
    ds.endUpdateBatch();
  },
  onModifyItem: function (item, oldItem) {
    if (! this.loading)
      this._modified = true;

    var calIEvent = Components.interfaces.calIEvent;
    if (! (item instanceof calIEvent && item.hasProperty("X-CELTX-SCENE")))
      return;
    item = item.QueryInterface(calIEvent);
    var sceneid = iCalStrToStr(item.getProperty("X-CELTX-SCENE"));
    var mapentry = this.scenemap[sceneid];
    if (! mapentry) {
      dump("*** onModifyItem: No map entry for " + sceneid + "\n");
      return;
    }
    var shouldBeComplete = true;
    for (var i = 0; i < mapentry.length; ++i) {
      if (mapentry[i].id == oldItem.id) {
        mapentry[i] = item;
      }
      if (! mapentry[i].hasProperty("X-CELTX-COMPLETED"))
        shouldBeComplete = false;
    }
    var ds = this.ds;
    var rdfsvc = getRDFService();
    var completedarc = rdfsvc.GetResource(Cx.NS_CX + "completed");
    var sceneres = rdfsvc.GetResource(sceneid);
    if (shouldBeComplete)
      setRDFString(ds, sceneres, completedarc, "true");
    else
      setRDFString(ds, sceneres, completedarc, "false");
  },
  onError: function  (aErrNo, aMessage) {
    dump("*** gScheduleController.onError: " + aErrNo + ", " + aMessage + "\n");
    var frame = Components.stack.caller;
    while (frame) {
      dump("    " + frame + "\n");
      frame = frame.caller;
    }
  },


  onAddItem: function (item) {
    if (! this.loading)
      this._modified = true;

    var calIEvent = Components.interfaces.calIEvent;
    if (! (item instanceof calIEvent && item.hasProperty("X-CELTX-SCENE")))
      return;
    item = item.QueryInterface(calIEvent);

    var sceneid = iCalStrToStr(item.getProperty("X-CELTX-SCENE"));
    if (! this.scenemap[sceneid]) {
      this.scenemap[sceneid] = new Array();
    }
    var mapentry = this.scenemap[sceneid];
    mapentry.push(item);

    if (this.loading)
      return;

    var ds = this.ds;
    var rdfsvc = getRDFService();
    var scheduledarc = rdfsvc.GetResource(Cx.NS_CX + "scheduled");
    var completedarc = rdfsvc.GetResource(Cx.NS_CX + "completed");
    var scene = rdfsvc.GetResource(sceneid);

    var selection = this.selectedScene;
    ds.beginUpdateBatch();
    setRDFString(ds, scene, scheduledarc, "true");
    if (! item.hasProperty("X-CELTX-COMPLETED"))
      setRDFString(ds, scene, completedarc, "false");
    ds.endUpdateBatch();
    this.selectedScene = selection;
  },


  onDeleteItem: function (item) {
    if (! this.loading)
      this._modified = true;

    var calIEvent = Components.interfaces.calIEvent;
    if (! (item instanceof calIEvent && item.hasProperty("X-CELTX-SCENE")))
      return;
    item = item.QueryInterface(calIEvent);

    var sceneid = iCalStrToStr(item.getProperty("X-CELTX-SCENE"));
    var mapentry = this.scenemap[sceneid];
    if (mapentry) {
      for (var i = 0; i < mapentry.length; ++i) {
        if (mapentry[i].id == item.id) {
          mapentry.splice(i, 1);
          break;
        }
      }
    }

    if (this.loading)
      return;

    var ds = this.ds;
    var ILit = Components.interfaces.nsIRDFLiteral;
    var rdfsvc = getRDFService();
    var scheduledarc = rdfsvc.GetResource(Cx.NS_CX + "scheduled");
    var completedarc = rdfsvc.GetResource(Cx.NS_CX + "completed");
    var scene = rdfsvc.GetResource(
      iCalStrToStr(item.getProperty("X-CELTX-SCENE")));

    var scheduled = ds.GetTarget(scene, scheduledarc, true);
    if (scheduled)
      scheduled = scheduled.QueryInterface(ILit);
    var completed = ds.GetTarget(scene, completedarc, true);
    if (completed)
      completed = completed.QueryInterface(ILit);
    if (! mapentry || mapentry.length == 0) {
      var selection = this.selectedScene;
      ds.beginUpdateBatch();
      setRDFString(ds, scene, scheduledarc, "false");
      ds.endUpdateBatch();
      this.selectedScene = selection;
    }
    // Update completed flag. Flag is true only when at least one shoot is
    // scheduled and all scheduled shoots are complete.
    if (mapentry.length > 0) {
      var shouldBeComplete = true;
      for (var i = 0; i < mapentry.length; ++i) {
        if (! mapentry[i].hasProperty("X-CELTX-COMPLETED")) {
          shouldBeComplete = false;
          break;
        }
      }
      if (shouldBeComplete)
        setRDFString(ds, scene, completedarc, "true");
      else
        setRDFString(ds, scene, completedarc, "false");
    }
    else {
      setRDFString(ds, scene, completedarc, "false");
    }
  },


  // Modified version of nsDragAndDrop.startDrag with additional |dragrect|
  // field for specifying the drag rect in the |action| parameter
  startDrag: function (aEvent) {
    const kDSIID = Components.interfaces.nsIDragService;
    var dragAction = {
      action: kDSIID.DRAGDROP_ACTION_COPY |
              kDSIID.DRAGDROP_ACTION_MOVE |
              kDSIID.DRAGDROP_ACTION_LINK,
      dragrect: null
    };
    var transferData = { data: null };
    try {
      this.onDragStart(aEvent, transferData, dragAction);
    }
    catch (ex) { return; }
    if (! transferData.data) return;
    transferData = transferData.data;

    var transArray = createSupportsArray();
    var count = 0;
    do {
      var trans = nsTransferable.set(transferData._XferID == "TransferData"
        ? transferData : transferData.dataList[count++]);
      transArray.AppendElement(trans.QueryInterface(
        Components.interfaces.nsISupports));
    }
    while (transferData.XferID == "TransferDataSet" &&
           count < transferData.dataList.length);
    try {
      nsDragAndDrop.mDragService.invokeDragSession(aEvent.target, transArray,
        dragAction.dragrect, dragAction.action);
    }
    catch (ex) {}
    aEvent.stopPropagation();
  },


  onDragStart: function (event, xferData, action) {
    if (event.originalTarget.localName == "treechildren") {
      var scene = this.selectedScene;
      if (! scene) {
        dump("*** No selected scene\n");
        return;
      }
      var data = new TransferData();
      data.addDataForFlavour("x-celtx/x-scene", scene.Value);
      xferData.data = data;
      return;
    }

    var target = event.currentTarget;
    if (target.id == "movingitem") {
      xferData.data = new TransferData();
      xferData.data.addDataForFlavour("x-celtx/x-banner", "moving");
    }
    else if (target.id == "dayoffitem") {
      xferData.data = new TransferData();
      xferData.data.addDataForFlavour("x-celtx/x-banner", "dayoff");
    }
    else {
      dump("*** invalid drag from " + event.currentTarget.localName + "\n");
      return;
    }

    if (target != event.target) {
      var region = Components.classes["@mozilla.org/gfx/region;1"]
        .createInstance(Components.interfaces.nsIScriptableRegion);
      region.init();
      var bo = target.boxObject;
      region.setToRect(bo.x, bo.y, bo.width, bo.height);
      action.dragrect = region;
    }
  },


  opened: function opened () {
  },


  focus: function focus () {
    if (this.loading) return;
    for (var sceneuri in this.scenemap) {
      var entries = this.scenemap[sceneuri];
      if (! entries) continue;
      for (var i = 0; i < entries.length; ++i) {
        var entry = entries[i].QueryInterface(Components.interfaces.calIEvent);
        this.updateSceneEvent(entry);
      }
    }
  },


  blur: function blur () {},


  print: function print () {
    calPrint();
  },


  createEventForScene: function createEventForScene (sceneuri) {
    var rdfsvc = getRDFService();
    var IRes = Components.interfaces.nsIRDFResource;
    var ILit = Components.interfaces.nsIRDFLiteral;
    var sceneres = rdfsvc.GetResource(sceneuri);
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var descarc = rdfsvc.GetResource(Cx.NS_DC + "description");
    var intextarc = rdfsvc.GetResource(Cx.NS_CX + "intext");
    var settingarc = rdfsvc.GetResource(Cx.NS_CX + "setting");
    var daynightarc = rdfsvc.GetResource(Cx.NS_CX + "daynight");
    var ordarc = rdfsvc.GetResource(Cx.NS_CX + "ordinal");
    var locationarc = rdfsvc.GetResource(Cx.NS_CX + "location");

    var ds = this.project.ds;

    // var ord = ds.GetTarget(sceneres, ordarc, true);
    // ord = ord.QueryInterface(IInt).Value;
    var ord = getRDFString(ds, sceneres, ordarc);
    var title = getRDFString(ds, sceneres, titlearc);
    var desc = getRDFString(ds, sceneres, descarc);
    var location = getRDFString(ds, sceneres, locationarc);

    var event = Components.classes["@mozilla.org/calendar/event;1"]
      .createInstance(Components.interfaces.calIEvent);
    event.title = ord + ". " + title;
    event.setProperty("DESCRIPTION", desc);
    event.setProperty("LOCATION", location);
    event.setProperty("X-CELTX-SCENE", strToICalStr(sceneuri));
    return event;
  },


  updateSceneEvent: function updateSceneEvent (event) {
    // Create a template event so we can just compare the events
    var sceneuri = iCalStrToStr(event.getProperty("X-CELTX-SCENE"));
    var tmpl = this.createEventForScene(sceneuri);
    if (event.title == tmpl.title &&
        event.getProperty("DESCRIPTION") == tmpl.getProperty("DESCRIPTION") &&
        event.getProperty("LOCATION") == tmpl.getProperty("LOCATION"))
      return;
    // Clone the original event to preserve user-specified fields
    var newevent = event.clone();
    newevent.title = tmpl.title;
    newevent.setProperty("DESCRIPTION", tmpl.getProperty("DESCRIPTION"));
    newevent.setProperty("LOCATION", tmpl.getProperty("LOCATION"));
    doTransaction('modify', newevent, newevent.calendar, event, null);
  },


  createNewEvent: function (aCalendar, aStartTime, aEndTime) {
    if (!aCalendar) {
      aCalendar = gWindow.scheduledeck.selectedPanel.displayCalendar;
      // dump("*** createNewEvent: aCalendar not specified\n");
      // return;
    }

    // if we're given both times, skip the dialog
    if (aStartTime && aEndTime && !aStartTime.isDate && !aEndTime.isDate) {
      var event = createEvent();
      event.startDate = aStartTime;
      event.endDate = aEndTime;
      var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"]
        .getService(Components.interfaces.nsIStringBundleService);
      // var props = sbs.createBundle(
      //   "chrome://calendar/locale/calendar.properties");
      // event.title = props.GetStringFromName("newEvent");
      event.title = gApp.getText("NewEvent");
      setDefaultAlarmValues(event);
      doTransaction('add', event, aCalendar, null, null);
    } else if (aStartTime && aStartTime.isDate) {
      var event = createEvent();
      event.startDate = aStartTime;
      setDefaultAlarmValues(event);
      doTransaction('add', event, aCalendar, null, null);
    } else {
      // default pop up the dialog
      var date;
      if (aStartTime) {
        date = aStartTime;
      } else {
        date = gWindow.monthview.selectedDay.clone();
        date.isDate = true;
      }
      try {
        this.createEventWithDialog(aCalendar, date, null);
      }
      catch (ex) {
        dump("*** createEventWithDialog: " + ex + "\n");
      }
    }
  },


  createEventWithDialog: function (aCalendar, aStartDate, aEndDate) {
    const kDefaultTimezone = calendarDefaultTimezone();
    var curview = gWindow.scheduledeck.selectedPanel;
    if (! aCalendar)
      aCalendar = curview.displayCalendar;

    event = createEvent();

    if (! aStartDate) {
      aStartDate = curview.selectedDay.clone();
      aStartDate.isDate = true;
    }
    if (aStartDate.isDate) {
      if (! aStartDate.isMutable) {
        aStartDate = aStartDate.clone();
      }
      aStartDate.isDate = false;
      aStartDate.hour = now().hour;
      aStartDate.minute = 0;
      aStartDate.second = 0;
      aStartDate.normalize();
    }
    event.startDate = aStartDate.clone();

    if (! aEndDate) {
      aEndDate = aStartDate.clone();
      var pref = getPrefService().getBranch("celtx.calendar.");
      aEndDate.minute += pref.getIntPref("scene.defaultlength");
      aEndDate.normalize();
    }
    event.endDate = aEndDate.clone();

    event.calendar = aCalendar;

    setDefaultAlarmValues(event);

    var args = {
      calendarEvent: event,
      calendar: aCalendar,
      controller: this,
      mode: "new",
      onOk: function (item, calendar, originalItem) {
        doTransaction('add', item, calendar, null, null);
      }
    };
    openDialog(Cx.CONTENT_PATH + "editors/eventdialog.xul", "_blank",
      "chrome,titlebar,modal,resizable", args);
  },



  modifyOccurrence: function (aOccurrence, aNewStartTime,
                              aNewEndTime, aNewTitle) {
    // if modifying this item directly (e.g. just dragged to new time),
    // then do so; otherwise pop up the dialog
    if ((aNewStartTime && aNewEndTime) || aNewTitle) {
      var instance = aOccurrence.clone();

      if (aNewTitle) {
        instance.title = aNewTitle;
      }

      if (aNewStartTime) { // we know we have aEndTime too then
        instance.startDate = aNewStartTime;
        instance.endDate = aNewEndTime;
      }
      doTransaction('modify', instance, instance.calendar, aOccurrence, null);
    } else {
      var args = {
        calendarEvent: aOccurrence,
        calendar: gWindow.monthview.displayCalendar,
        controller: this,
        mode: "modify",
        onOk: function (item, calendar, originalItem) {
          doTransaction('modify', item, item.calendar, originalItem, null);
        }
      };
      openDialog(Cx.CONTENT_PATH + "editors/eventdialog.xul", "_blank",
        "chrome,titlebar,modal,resizable", args);
    }
  },

  deleteOccurrence: function (aOccurrence) {
    var itemToDelete = getOccurrenceOrParent(aOccurrence);
    if (!itemToDelete) {
      return;
    }
    if (!itemToDelete.parentItem.hasSameIds(itemToDelete)) {
      var event = itemToDelete.parentItem.clone();
      event.recurrenceInfo.removeOccurrenceAt(itemToDelete.recurrenceId);
      doTransaction('modify', event, event.calendar,
        itemToDelete.parentItem, null);
    }
    else {
      doTransaction('delete', itemToDelete, itemToDelete.calendar, null, null);
    }
  },


  createTodoWithDialog: function (aCalendar) {
    var kDefaultTimezone = calendarDefaultTimezone();

    var todo = Components.classes["@mozilla.org/calendar/todo;1"]
      .createInstance(Components.interfaces.calITodo);
    todo.calendar = aCalendar;
    dump("--- set todo.calendar to " + todo.calendar + "\n");
    setDefaultAlarmValues(todo);

    var args = {
      calendarTodo: todo,
      calendar: aCalendar,
      controller: this,
      mode: "new",
      onOk: function (item, calendar, originalItem) {
        dump("--- createTodo.onOk: " + item + ", " + calendar + ", "
          + originalItem + "\n");
        doTransaction('add', item, calendar, null, null);
      }
    };
    openDialog(Cx.CONTENT_PATH + "editors/tododialog.xul", "_blank",
      "chrome,titlebar,modal,resizable", args);
  },


  modifyTodoWithDialog: function modifyTodoWithDialog (todo) {
    var args = {
      calendarTodo: todo,
      calendar: gScheduleController.calendar,
      controller: this,
      mode: "modify",
      onOk: function (item, calendar, originalItem) {
        doTransaction('modify', item, item.calendar, originalItem, null);
      }
    };
    openDialog(Cx.CONTENT_PATH + "editors/tododialog.xul", "_blank",
      "chrome,titlebar,modal,resizable", args);
  },


  switchView: function switchView (view) {
    var selectedDay = gWindow.scheduledeck.selectedPanel.selectedDay;
    switch (view) {
      case "month":
        gWindow.scheduledeck.selectedPanel = gWindow.monthview;
        gWindow.monthview.goToDay(selectedDay);
        break;
      case "week":
        gWindow.scheduledeck.selectedPanel = gWindow.weekview;
        gWindow.weekview.goToDay(selectedDay);
        break;
      case "day":
        gWindow.scheduledeck.selectedPanel = gWindow.dayview;
        gWindow.dayview.goToDay(selectedDay);
        break;
    }
  }
};


function setOutlineView (outline) {
  gScheduleController.outline = outline;
}


function getCalStringBundle() {
    var strBundleService = 
        Components.classes["@mozilla.org/intl/stringbundle;1"]
                  .getService(Components.interfaces.nsIStringBundleService);
    return strBundleService.createBundle("chrome://calendar/locale/calendar.properties");
}


// For calPrint()
function currentView () {
  return gWindow.scheduledeck.selectedPanel;
}


// For calPrint()
function getCompositeCalendar () {
  return gScheduleController.calendar;
}


function showError(aMsg) {
  var msg = gApp.getText("CalendarImportFailedMsg");
  celtxBugAlert(msg, Components.stack.caller, aMsg); 
}


function setAgendaTreeView()
{
    agendaTreeView.setCalendar(gScheduleController.calendar);
    /*
    document.getElementById("agenda-tree").view = agendaTreeView;
    */
    gWindow.agendaTree.view = agendaTreeView;
}


function today()
{
    var d = Components.classes['@mozilla.org/calendar/datetime;1'].createInstance(Components.interfaces.calIDateTime);
    d.jsDate = new Date();
    return d.getInTimezone(calendarDefaultTimezone());
}

function nextMonth(dt)
{
    var d = new Date(dt);
    d.setDate(1); // make sure we avoid "June 31" when we bump the month

    var mo = d.getMonth();
    if (mo == 11) {
        d.setMonth(0);
        d.setYear(d.getYear() + 1);
    } else {
        d.setMonth(mo + 1);
    }

    return d;
}


function Synthetic(title, open)
{
    this.title = title;
    this.open = open;
    this.events = [];
}

var agendaTreeView = {
    events: [],
    todayCount: 0,
    tomorrowCount: 0,
    soonCount: 0,
    prevRowCount: 0
};

agendaTreeView.init =
function initAgendaTree()
{
    this.today = new Synthetic(ltnGetString("lightning", "agendaToday"), true);
    this.tomorrow = new Synthetic(ltnGetString("lightning", "agendaTomorrow"), false);
    this.soon = new Synthetic(ltnGetString("lightning", "agendaSoon"), false);
    this.periods = [this.today, this.tomorrow, this.soon];
}

agendaTreeView.addEvents =
function addEvents(master)
{
    this.events.push(master);
    if (master.open)
        this.events = this.events.concat(master.events);
};

agendaTreeView.rebuildEventsArray =
function rebuildEventsArray()
{
    this.events = [];
    this.addEvents(this.today);
    this.addEvents(this.tomorrow);
    this.addEvents(this.soon);
};

agendaTreeView.forceTreeRebuild =
function forceTreeRebuild()
{
    if (this.tree) {
        this.tree.view = this;
    }
};

agendaTreeView.rebuildAgendaView =
function rebuildAgendaView(invalidate)
{
    this.rebuildEventsArray();
    this.forceTreeRebuild();
};

agendaTreeView.__defineGetter__("rowCount",
function get_rowCount()
{
    return this.events.length;
});

agendaTreeView.isContainer =
function isContainer(row)
{
    return (this.events[row] instanceof Synthetic);
};

agendaTreeView.isContainerOpen =
function isContainerOpen(row)
{
    var open = this.events[row].open;
    return open;
};

agendaTreeView.isContainerEmpty =
function isContainerEmpty(row)
{
    if (this.events[row].events.length == 0)
        return true;
    return false;
};

agendaTreeView.setTree =
function setTree(tree)
{
    this.tree = tree;
};

agendaTreeView.getCellText =
function getCellText(row, column)
{
    // title column
    var event = this.events[row];
    if (column.id == "col-agenda-item") {
        if (event instanceof Synthetic)
            return event.title;
        return event.title;
    }
    // date/time column
    var dateFormatter = Components.classes["@mozilla.org/calendar/datetime-formatter;1"]
                                  .getService(Components.interfaces.calIDateTimeFormatter);
    if (event instanceof Synthetic) {
        if (event == this.today) {
            return dateFormatter.formatDate(this.today.start);
        }
        else if (event == this.tomorrow) {
            return dateFormatter.formatDate(this.tomorrow.start);
        }
        return "";
    }
    var start = event.startDate || event.dueDate || event.entryDate;
    start = start.getInTimezone(calendarDefaultTimezone());
    if (start.compare(this.tomorrow.end) == -1) {
        // time only for events on today and tomorrow
        return  dateFormatter.formatTime(start);
    }
    else {
        return dateFormatter.formatDateTime(start);
    }
};

agendaTreeView.getLevel =
function getLevel(row)
{
    if (this.isContainer(row))
        return 0;
    return 1;
};

agendaTreeView.isSorted =
function isSorted() { return false; };

agendaTreeView.isEditable =
function isEditable(row, column) { return false; };

agendaTreeView.isSeparator =
function isSeparator(row) { return false; };

agendaTreeView.getImageSrc =
function getImageSrc(row, column) { return null; };

agendaTreeView.getCellProperties =
function getCellProperties(row, column) { return null; };

agendaTreeView.getRowProperties =
function getRowProperties(row) { return null; };

agendaTreeView.getColumnProperties =
function getColumnProperties(column) { return null; };

agendaTreeView.cycleHeader =
function cycleHeader(header)
{
    this.refreshCalendarQuery(); // temporary hackishness
    this.rebuildAgendaView();
    this.forceTreeRebuild();
};

agendaTreeView.getParentIndex =
function getParentIndex(row)
{
    if (this.isContainer(row))
        return -1;
    var i = row - 1;
    do {
        if (this.events[i] instanceof Synthetic)
            return i;
        i--;
    } while (i != -1);
    throw "no parent for row " + row + "?";
};

agendaTreeView.toggleOpenState =
function toggleOpenState(row)
{
    if (!this.isContainer(row))
        throw "toggling open state on non-container row " + row + "?";
    var header = this.events[row];
    if (!("open") in header)
        throw "no open state found on container row " + row + "?";
    header.open = !header.open;
    this.rebuildAgendaView(); // reconstruct the visible row set
    this.forceTreeRebuild();
};

agendaTreeView.hasNextSibling =
function hasNextSibling(row, afterIndex)
{
};

agendaTreeView.findPeriodForItem =
function findPeriodForItem(item)
{
    var start = item.startDate || item.entryDate || item.dueDate;
    if (!start) 
        return null;
    if (start.compare(this.today.end) == -1)
        return this.today;
        
    if (start.compare(this.tomorrow.end) == -1)
        return this.tomorrow;
    
    if (start.compare(this.soon.end) == -1)
        return this.soon;
    

    return null;
};

agendaTreeView.addItem =
function addItem(item)
{
    var when = this.findPeriodForItem(item);
    if (!when)
        return;
    when.events.push(item);
    this.calendarUpdateComplete();
};

agendaTreeView.onDoubleClick = function agendaDoubleClick(event) {
  // We only care about left-clicks
  if (event.button != 0) 
      return;

  // Find the row clicked on, and the corresponding event
  /*
  var tree = document.getElementById("agenda-tree");
  */
  var tree = gWindow.agendaTree;
  var row = tree.treeBoxObject.getRowAt(event.clientX, event.clientY);
  var calendar = gScheduleController.calendar;
  var calEvent = this.events[row];

  if (!calEvent) { // Clicked in empty space, just create a new event
    gScheduleController.createEventWithDialog(calendar, today(), today());
    return;
  }
  if (!this.isContainer(row)) { // Clicked on a task/event, edit it
    var eventToEdit = getOccurrenceOrParent(calEvent);
    gScheduleController.modifyEventWithDialog(eventToEdit);
  }
  else { // Clicked on a container, create an event that day
    if (calEvent == this.today) {
      gScheduleController.createEventWithDialog(calendar, today(), today());
    }
    else {
      var tom = today().clone();
      var offset = (calEvent == this.tomorrow) ? 1 : 2;
      tom.day += offset;
      tom.normalize()
      gScheduleController.createEventWithDialog(calendar, tom, tom);
    }
  }
}

agendaTreeView.deleteItem =
function deleteItem(item)
{
    var when = this.findPeriodForItem(item);
    if (!when) {
        return;
    }
    
    when.events = when.events.filter(function (e) {
                                         if (e.id != item.id)
                                             return true;
                                         if (e.recurrenceId && item.recurrenceId &&
                                             e.recurrenceId.compare(item.recurrenceId) != 0)
                                             return true;
                                         return false;
                                     });
    this.rebuildAgendaView(true);
};

agendaTreeView.calendarUpdateComplete =
function calendarUpdateComplete()
{
    [this.today, this.tomorrow, this.soon].forEach(function(when) {
        function compare(a, b) {
            // The assumption is that tasks having a dueDate should be added
            // to the agenda based on _that_, rather than entryDate, but tasks
            // with an entryDate but no dueDate shouldn't be left out.
            var ad = a.startDate || a.dueDate || a.entryDate;
            var bd = b.startDate || b.dueDate || b.entryDate;
            return ad.compare(bd);
        }
        when.events.sort(compare);
    });
    this.rebuildAgendaView(true);
};

agendaTreeView.calendarOpListener =
{
    agendaTreeView: agendaTreeView
};

agendaTreeView.calendarOpListener.onOperationComplete =
function listener_onOperationComplete(calendar, status, optype, id,
                                      detail)
{
    this.agendaTreeView.calendarUpdateComplete();  
};

agendaTreeView.calendarOpListener.onGetResult =
function listener_onGetResult(calendar, status, itemtype, detail, count, items)
{
    if (!Components.isSuccessCode(status))
        return;

    items.forEach(this.agendaTreeView.addItem, this.agendaTreeView);
};

agendaTreeView.refreshCalendarQuery =
function refreshCalendarQuery()
{
    var filter = this.calendar.ITEM_FILTER_COMPLETED_ALL |
                 this.calendar.ITEM_FILTER_CLASS_OCCURRENCES;
    if (!this.filterType)
        this.filterType = 'all';
    switch (this.filterType) {
        case 'all': 
            filter |= this.calendar.ITEM_FILTER_TYPE_EVENT |
                      this.calendar.ITEM_FILTER_TYPE_TODO;
            break;
        case 'events':
            filter |= this.calendar.ITEM_FILTER_TYPE_EVENT;
            break;
        case 'tasks':
            filter |= this.calendar.ITEM_FILTER_TYPE_TODO;
            break;
    }

    this.periods.forEach(function (p) { p.events = []; });
    this.calendar.getItems(filter, 0, this.today.start, this.soon.end,
                           this.calendarOpListener);
};

agendaTreeView.updateFilter =
function updateAgendaFilter(menulist) {
    this.filterType = menulist.selectedItem.value;
    this.refreshCalendarQuery();
    return;
};

agendaTreeView.refreshPeriodDates =
function refreshPeriodDates()
{
    var now = new Date();
    var d = new CalDateTime();
    d.jsDate = now;
    d = d.getInTimezone(calendarDefaultTimezone());

    // Today: now until midnight of tonight
    this.today.start = d.clone();
    d.hour = d.minute = d.second = 0;
    d.day++;
    d.normalize();
    this.today.end = d.clone();

    // Tomorrow: midnight of next day to +24 hrs
    this.tomorrow.start = d.clone();
    d.day++;
    d.normalize();
    this.tomorrow.end = d.clone();

    // Soon: end of tomorrow to 6 six days later (remainder of the week period)
    this.soon.start = d.clone();
    d.day += 6;
    d.normalize();
    this.soon.end = d.clone();

    this.refreshCalendarQuery();
};

agendaTreeView.calendarObserver = {
    agendaTreeView: agendaTreeView
};

agendaTreeView.calendarObserver.QueryInterface = function agenda_QI(aIID) {
    if (!aIID.equals(Components.interfaces.calIObserver) &&
        !aIID.equals(Components.interfaces.calICompositeObserver) &&
        !aIID.equals(Components.interfaces.nsISupports)) {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
};

// calIObserver:
agendaTreeView.calendarObserver.onStartBatch = function agenda_onBatchStart() {
    this.mBatchCount++;
};
agendaTreeView.calendarObserver.onEndBatch = function() {
    this.mBatchCount--;
    if (this.mBatchCount == 0) {
        // Rebuild everything
        this.agendaTreeView.refreshCalendarQuery();
    }
};
agendaTreeView.calendarObserver.onLoad = function() {};

agendaTreeView.calendarObserver.onAddItem =
function observer_onAddItem(item)
{
    if (this.mBatchCount) {
        return;
    }
    var occs = item.getOccurrencesBetween(this.agendaTreeView.today.start,
                                          this.agendaTreeView.soon.end, {});
    occs.forEach(this.agendaTreeView.addItem, this.agendaTreeView);
    this.agendaTreeView.rebuildAgendaView();
};

agendaTreeView.calendarObserver.onDeleteItem =
function observer_onDeleteItem(item, rebuildFlag)
{
    if (this.mBatchCount) {
        return;
    }
    var occs = item.getOccurrencesBetween(this.agendaTreeView.today.start,
                                          this.agendaTreeView.soon.end, {});
    occs.forEach(this.agendaTreeView.deleteItem, this.agendaTreeView);
    if (rebuildFlag != "no-rebuild")
        this.agendaTreeView.rebuildAgendaView();
};

agendaTreeView.calendarObserver.onModifyItem =
function observer_onModifyItem(newItem, oldItem)
{
    if (this.mBatchCount) {
        return;
    }
    this.onDeleteItem(oldItem, "no-rebuild");
    this.onAddItem(newItem);
};

agendaTreeView.calendarObserver.onError = function(errno, msg) {};

agendaTreeView.calendarObserver.onCalendarAdded = 
function agenda_calAdd(aCalendar) {
    this.agendaTreeView.refreshCalendarQuery();
};

agendaTreeView.calendarObserver.onCalendarRemoved = 
function agenda_calRemove(aCalendar) {
    this.agendaTreeView.refreshCalendarQuery();
};

agendaTreeView.calendarObserver.onDefaultCalendarChanged = function(aCalendar) {
};

agendaTreeView.setCalendar =
function setCalendar(calendar)
{
    if (this.calendar)
        this.calendar.removeObserver(this.calendarObserver);
    this.calendar = calendar;
    calendar.addObserver(this.calendarObserver);

    this.init();

    // Update everything
    this.refreshPeriodDates();
};

function setAgendaTreeView()
{
    agendaTreeView.setCalendar(getCompositeCalendar());
    /*
    document.getElementById("agenda-tree").view = agendaTreeView;
    */
    gWindow.agendaTree.view = agendaTreeView;
}

/*
window.addEventListener("load", setAgendaTreeView, false);
*/

// todo-list.js overrides

function eventToTodo(event)
{
  try {
    return event.originalTarget.selectedItem.todo;
  } catch (e) {
    return null;
  }
}

function editTodoItem(event) {
  var todo = eventToTodo(event);
  if (todo)
    gScheduleController.modifyTodoWithDialog(todo);
}

function newTodoItem(event) {
  gScheduleController.createTodoWithDialog(gScheduleController.calendar);
}

function deleteTodoItem(event) {
  var todo = eventToTodo(event);
  if (todo)
    todo.calendar.deleteItem(todo, null);
}

function initializeTodoList() {
  var todoList = gWindow.todoList;
  todoList.calendar = getCompositeCalendar();
  todoList.addEventListener("todo-item-open", editTodoItem, false);
  todoList.addEventListener("todo-item-delete", deleteTodoItem, false);
  todoList.addEventListener("todo-empty-dblclick", newTodoItem, false);
}
