/* ***** BEGIN LICENCE BLOCK *****
 * Version: CePL 1.1
 * 
 * The contents of this file are subject to the Celtx Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.celtx.com/CePL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Celtx Script Manager.
 * 
 * The Initial Developer of the Original Code is Chad House and 4067479
 * Canada Inc. t/a CELTX.
 * 
 * Portions created by Chad House are Copyright (C) 2000-2004 Chad House,
 * parts created by Celtx are Copyright (C) 4067479 Canada Inc. All Rights
 * Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENCE BLOCK ***** */


var gDeptOrder = [
  Cx.NS_CX + "Location",
  Cx.NS_CX + "Cast",
  Cx.NS_CX + "Actor",
  Cx.NS_CX + "Extras",
  Cx.NS_CX + "Hair",
  Cx.NS_CX + "Makeup",
  Cx.NS_CX + "Wardrobe",
  Cx.NS_CX + "Props",
  Cx.NS_CX + "Set",
  Cx.NS_CX + "SetDressing",
  Cx.NS_CX + "Painting",
  Cx.NS_CX + "Construction",
  Cx.NS_CX + "Greenery",
  Cx.NS_CX + "Camera",
  Cx.NS_CX + "Lights",
  Cx.NS_CX + "Electrics",
  Cx.NS_CX + "Stunts",
  Cx.NS_CX + "Vehicles",
  Cx.NS_CX + "Weapons",
  Cx.NS_CX + "SpecialEquipment",
  Cx.NS_CX + "AdditionalLabour",
  Cx.NS_CX + "Security",
  Cx.NS_CX + "AnimalHandler",
  Cx.NS_CX + "Animals",
  Cx.NS_CX + "Livestock",
  Cx.NS_CX + "Storyboard",
  Cx.NS_CX + "Music",
  Cx.NS_CX + "Sound",
  Cx.NS_CX + "SoundFX",
  Cx.NS_CX + "MechFX",
  Cx.NS_CX + "OpticalFX",
  Cx.NS_CX + "SpecialFX",
  Cx.NS_CX + "CGI",
  Cx.NS_CX + "ProductionNotes",
  Cx.NS_CX + "Misc"
];


var gReportController = {
  kScenesURL: "http://celtx.com/selection/scenes",
  kDeptsURL: "http://celtx.com/selection/departments",
  kItemsURL: "http://celtx.com/selection/items",

  selections: null,
  bdds: null,
  sceneAllSelected: false,
  selectEventsSuppressed: false,
  _loaded: false,
  _changedScenes: [],


  QueryInterface: function (iid) {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsIRDFObserver))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },


  get mediachecked () {
    return this.mediacheckbox.checked;
  },


  loaded: function loaded () {
    this.reporttitle    = document.getElementById("reporttitle");
    this.browser        = document.getElementById("reportframe");
    this.columnbox      = document.getElementById("columnbox");
    this.columndeck     = document.getElementById("columndeck");
    this.scenelist      = document.getElementById("scene-listbox");
    this.deptlist       = document.getElementById("dept-listbox");
    this.itemlist       = document.getElementById("item-listbox");
    this.charlist       = document.getElementById("charlist");
    this.elemlist       = document.getElementById("elemlist");
    this.mediacheckbox  = document.getElementById("media-checkbox");
  },


  commands: {
    "cmd-report-changed": 1,
    "cmd-report-scene-changed": 1,
    "cmd-report-dept-changed": 1,
    "cmd-report-item-changed": 1,
    "cmd-report-speaker-changed": 1,
    "cmd-report-element-changed": 1,
    "cmd-report-toggle-media": 1,
    "cmd-page-setup": 1,
    "cmd-print": 1,
    "cmd-print-preview": 1
  },


  supportsCommand: function supportsCommand (cmd) {
    return this.commands[cmd] == 1;
  },


  isCommandEnabled: function isCommandEnabled (cmd) {
    switch (cmd) {
      case "cmd-page-setup":
      case "cmd-print":
      case "cmd-print-preview":
        return true;
      default:
        return ! this.inPrintPreview;
    }
  },


  doCommand: function doCommand (cmd) {
    if (! this._loaded)
      return;

    switch (cmd ) {
      case "cmd-page-setup":
        PrintUtils.showPageSetup();
        break;
      case "cmd-print":
        this.print();
        break;
      case "cmd-print-preview":
        PrintUtils.printPreview(report_onEnterPrintPreview,
          report_onExitPrintPreview);
        break;
      case "cmd-report-scene-changed":
        this.cmdSceneSelectionChanged();
        break;
      case "cmd-report-dept-changed":
        this.cmdDeptSelectionChanged();
        break;
      case "cmd-report-item-changed":
        this.cmdItemSelectionChanged();
        break;
      case "cmd-report-element-changed":
      case "cmd-report-speaker-changed":
        if (! this.selectEventsSuppressed)
          this.cmdCreateScriptReport();
        break;
      case "cmd-report-toggle-media":
        this.cmdCreateBreakdown();
        break;
      case "cmd-report-changed":
        this.cmdChangeReport();
        break;
    }
  },


  updateCommands: function updateCommands () {
    for (var cmd in this.commands)
      goUpdateCommand(cmd);
  },


  /*
  getDataFile: function getDataFile (type) {
    var m = new RDFModel(this.project.localDS);
    
    var fileRes = m.target(RES(this.docres.Value), PROP('cx:' + type));
    if (! fileRes) {
      dump("*** getDataFile: Couldn't locate " + type + " file\n");
      return null;
    }

    return fileURLToFile(fileRes.value);
  },
  */


  open: function open (project, docres) {
    this.project = project;
    this.docres = docres;
    this.outlineView = gController.outlineView;
    if (! this.outlineView)
      dump("*** gReportController.open: no outline view\n");

    this.reportFile = tempFile('html');
    this.project.ds.AddObserver(this);
  },


  onScriptLoad: function onScriptLoad () {
  },


  generateData: function generateData () {
    var progressController = {
      owner: this,
      canceled: false,
      initialized: false,
      postprocessed: false,
      sceneCount: gScriptController.editor.sceneCount,
      curScene: 0,
      get progress () {
        var base = 0;
        if (this.initialized) base += 1;
        if (this.sceneCount > 0)
          base += Math.floor(98 * this.curScene / this.sceneCount);
        else
          base += 98;
        if (this.postprocessed)
          base += 1;
        return base;
      },
      get message () {
        return gApp.getText("GeneratingReportMsg");
      },
      get finished () {
        return this.postprocessed;
      },
      performNextTask: function () {
        try {
          if (! this.initialized) {
            this.owner.beginScriptTree();
            this.owner.beginBreakdownDS();
            this.initialized = true;
            return true;
          }
          if (this.curScene < this.sceneCount) {
            this.owner.updateScriptTree(this.curScene);
            this.owner.updateBreakdownDS(this.curScene);
            this.curScene += 1;
            return true;
          }
          if (! this.postprocessed) {
            this.owner.endScriptTree();
            this.owner.endBreakdownDS();
            this.postprocessed = true;
            return true;
          }
        }
        catch (ex) {
          this.stack = Components.stack;
          this.exception = ex;
          return false;
        }
      },
      abort: function () {
        // this.scriptTree = null;
        // this.breakdownDS = null;
        this.canceled = true;
      }
    };
    window.openDialog(Cx.CONTENT_PATH + "progress.xul", "_blank",
                      Cx.MODAL_DIALOG_FLAGS, progressController);
    if (progressController.canceled)
      return;
    if (! progressController.finished) {
      var stack = progressController.stack;
      var ex = progressController.exception;
      var msg = gApp.getText("GenerateReportFailedMsg");
      celtxBugAlert(msg, stack, ex);
      return;
    }

    this.generateDataCallback();
  },


  generateDataCallback: function generateDataCallback () {
    this.outlineView.selectReportIndex(0);

    // Initialize the datasources
    var rdfsvc = getRDFService();
    var utils = getRDFContainerUtils();
    var ILit = Components.interfaces.nsIRDFLiteral;
    var IRes = Components.interfaces.nsIRDFResource;

    this.scenelist.database.AddDataSource(this.project.ds);
    var scenesarc = rdfsvc.GetResource(Cx.NS_CX + "scenes");
    var scenes = this.project.ds.GetTarget(this.docres, scenesarc, true);
    if (scenes) {
      scenes = scenes.QueryInterface(IRes);
    }
    else {
      scenes = rdfsvc.GetAnonymousResource();
      this.project.ds.Assert(this.docres, scenesarc, scenes, true);
    }
    // Make sure it's decorated as an RDF sequence
    new RDFSeq(this.project.ds, scenes);
    this.scenelist.setAttribute("ref", scenes.Value);

    var allDS = this.createBogusAllDS();

    // The magical "ALL" scene
    var titleArc      = rdfsvc.GetResource(Cx.NS_DC + "title");
    var sortordArc    = rdfsvc.GetResource(Cx.NS_CX + "sortord");
    var allItemArc    = rdfsvc.GetResource(Cx.NS_CX + "allItem");
    var allScene = rdfsvc.GetResource("urn:celtx:scene:all");
    allDS.Assert(scenes, allItemArc, allScene, true);
    allDS.Assert(allScene, sortordArc, rdfsvc.GetLiteral("0000"), true);
    allDS.Assert(allScene, titleArc, rdfsvc.GetLiteral("ALL"), true);

    // Selections (intermediate datasources)
    this.selections = getInMemoryDataSource();
    var sceneSeq = rdfsvc.GetResource(this.kScenesURL);
    this.scenes = utils.MakeSeq(this.selections, sceneSeq);
    // bd_win.depts is not a sequence; it uses cx:member instead
    this.depts = rdfsvc.GetResource(this.kDeptsURL);
    var itemSeq = rdfsvc.GetResource(this.kItemsURL);
    this.items = utils.MakeSeq(this.selections, itemSeq);

    // Scenes
    this.scenelist.database.allowNegativeAssertions = false;
    this.scenelist.database.AddDataSource(allDS);
    this.scenelist.builder.rebuild();

    // Departments
    this.deptlist.database.allowNegativeAssertions = false;
    // deptlist.ref = gProjWin.project.markupURL;
    this.deptlist.database.AddDataSource(allDS);
    this.deptlist.database.AddDataSource(this.project.ds);
    this.deptlist.builder.rebuild();

    // Items
    this.itemlist.database.allowNegativeAssertions = false;
    this.itemlist.database.AddDataSource(allDS);
    // this.itemlist.database.AddDataSource(this.sceneds);
    this.itemlist.database.AddDataSource(this.project.ds);
    this.itemlist.database.AddDataSource(this.bdds);
    this.itemlist.database.AddDataSource(this.selections);
    this.itemlist.builder.rebuild();

    // Speakers
    this.charlist.database.allowNegativeAssertions = false;
    this.charlist.database.AddDataSource(allDS);
    this.chards = this.createCharacterDS(this.scripttree);
    this.charlist.database.AddDataSource(this.chards);
    this.charlist.builder.rebuild();

    // Elements
    // this.elemlist.database.allowNegativeAssertions = false;
    // Prune the element list (remove Act and unused)
    var mode = this.scripttree.documentElement.getAttribute("scriptmode");
    var ps = getPrefService().getBranch("celtx.scripteditor." + mode + ".");
    var formats = ps.getCharPref("formats").split(/\s*,\s*/);
    var elemview = this.elemlist.view;
    var elemcontent = elemview.QueryInterface(
      Components.interfaces.nsITreeContentView);
    for (var i = 0; i < elemview.rowCount; ++i) {
      var elem = elemcontent.getItemAtIndex(i);
      if (elem.id == Cx.NS_CX + "Formats/Act") {
        dump("--- Removing element " + elem.id + "\n");
        elem.hidden = true;
        --i;
      }
      else {
        var found = false;
        for (var j = 0; j < formats.length; ++j) {
          if (elem.id == Cx.NS_CX + "Formats/" + formats[j]) {
            found = true;
            break;
          }
        }
        if (! found) {
          dump("--- Removing element " + elem.id + "\n");
          elem.hidden = true;
          --i;
        }
      }
    }

    this._loaded = true;

    this.selectEventsSuppressed = true;
    this.deptlist.view.selection.select(0);
    this.charlist.view.selection.select(0);
    this.elemlist.view.selection.select(0);
    this.itemlist.view.selection.select(0);
    this.columnbox.collapsed = false;
    this.columndeck.selectedIndex = 1;
    this.columndeck.flex = 1;
    this.mediacheckbox.collapsed = true;
    this.selectEventsSuppressed = false;
    this.scenelist.view.selection.select(0);
  },


  save: function save () {
    // Implementation-dependent mechanism: If closing is triggered by a
    // window close event, this will get called as part of saveAllFrames, so
    // that when this document gets restored the next time the project is
    // opened, it doesn't bork itself for lack of data. If the tab is closed
    // instead by clicking the tab close button, nothing gets saved.
    this.saved = true;
  },


  close: function close () {
    gScriptController.sceneTracker.removeObserver(this);
    this.project.ds.RemoveObserver(this);

    this.reportFile.remove(false);

    if (this.saved)
      return;
  },


  focus: function focus () {
    dump("--- gReportController.focus\n");
    this.browser.setAttribute("type", "content-primary");
    this.outlineView.showReportNav();
    if (! this._loaded) {
      this.generateData();
      gScriptController.sceneTracker.addObserver(this);
      return;
    }

    if (this._dirty) {
      this.refresh();
    }
    else if (this._changedScenes.length > 0) {
      this.updateChangedScenes();
    }
    else
      return;

    this.cmdChangeReport();
  },


  onAssert: function (ds, src, prop, tgt) {},
  onChange: function (ds, src, prop, oldtgt, newtgt) {},
  onMove: function (ds, oldsrc, newsrc, prop, tgt) {},
  onUnassert: function (ds, src, prop, tgt) {},


  onBeginUpdateBatch: function onBeginUpdateBatch (ds) {
    dump("--- onBeginUpdateBatch\n");
    this.selectEventsSuppressed = true;
    this.cacheSelections();
  },


  onEndUpdateBatch: function onEndUpdateBatch (ds) {
    dump("--- onEndUpdateBatch\n");
    this.restoreSelections();
    this.selectEventsSuppressed = false;
  },


  cacheSelections: function cacheSelections () {
    // This is a quick and dirty implementation. We cache the rows
    // that are selected by row index rather than value. Insertions or
    // removals will result in an incorrect restore.
    var trees = [ this.scenelist, this.deptlist, this.itemlist,
      this.charlist, this.elemlist ];
    this._cachedSelections = {};
    for (var i = 0; i < trees.length; ++i) {
      var ranges = [];
      var sel = trees[i].view.selection;
      var count = sel.getRangeCount();
      for (var j = 0; j < count; ++j) {
        var min = { value: 0 };
        var max = { value: 0 };
        sel.getRangeAt(j, min, max);
        ranges.push([min.value, max.value]);
      }
      this._cachedSelections[trees[i].id] = ranges;
    }
  },


  restoreSelections: function restoreSelections () {
    var trees = [ this.scenelist, this.deptlist, this.itemlist,
      this.charlist, this.elemlist ];
    for (var i = 0; i < trees.length; ++i) {
      var ranges = this._cachedSelections[trees[i].id];
      var sel = trees[i].view.selection;
      sel.clearSelection();
      try {
        for (var j = 0; j < ranges.length; ++j) {
          sel.rangedSelect(ranges[j][0], ranges[j][1], true);
        }
      }
      catch (ex) {
        dump("*** restoreSelections: " + ex + "\n");
        sel.select(0);
      }
    }
  },


  // Observer method
  sceneListChanged: function sceneListChanged () {
    this._dirty = true;
  },


  sceneChanged: function sceneChanged (sceneres) {
    this.sceneContentChanged(sceneres);
  },


  // Observer method
  sceneContentChanged: function sceneContentChanged (sceneres) {
    var rdfsvc = getRDFService();
    var sceneidarc = rdfsvc.GetResource(Cx.NS_CX + "sceneid");
    var sceneid = getRDFString(this.project.ds, sceneres, sceneidarc);
    if (! sceneid) {
      dump("*** sceneContentChanged: no cx:sceneid on sceneres\n");
      return;
    }
    for (var i = 0; i < this._changedScenes.length; ++i) {
      if (this._changedScenes[i] == sceneid)
        return;
    }
    this._changedScenes.push(sceneid);
  },


  refresh: function refresh () {
    this._dirty = false;
    this._changedScenes = [];
  },


  updateChangedScenes: function updateChangedScenes () {
    while (this._changedScenes.length > 0) {
      var sceneid = this._changedScenes.shift();
      var idx = gScriptController.editor.scenePosition(sceneid);
      this.updateScriptTree(idx - 1);
      this.updateBreakdownDS(idx - 1);
    }
  },


  blur: function blur () {
    if (this.inPrintPreview)
      PrintUtils.exitPrintPreview();
    this.browser.setAttribute("type", "content");
  },


  print: function print () {
    try {
      var webBrowserPrint = PrintUtils.getWebBrowserPrint();
      var printSettings = PrintUtils.getPrintSettings();
      printSettings.headerStrLeft   = "&D";
      printSettings.headerStrCenter = "&T";
      printSettings.headerStrRight  = "&PT";
      printSettings.footerStrLeft   = "";
      printSettings.footerStrCenter = "";
      printSettings.footerStrRight  = "";
      webBrowserPrint.print(printSettings, null);
    }
    catch (ex) {
      dump("*** scriptreport.print: " + ex + "\n");
    }
  },


  createBogusAllDS: function createBogusAllDS () {
    // The datasource providing "ALL" for scenes and departments
    var svc = getRDFService();
    var ds = getInMemoryDataSource();

    var typeArc       = svc.GetResource(Cx.NS_RDF + "type");
    var titleArc      = svc.GetResource(Cx.NS_DC + "title");
    var labelArc      = svc.GetResource(Cx.NS_RDFS + "label");
    var ordArc        = svc.GetResource(Cx.NS_CX + "ordinal");
    var sceneType     = svc.GetResource(Cx.NS_CX + "Scene");
    var memberArc     = svc.GetResource(Cx.NS_CX + "member");
    var breakdownType = svc.GetResource(Cx.NS_CX + "Breakdown");
    var castType      = svc.GetResource(Cx.NS_CX + "Cast");
    var formatType    = svc.GetResource(Cx.NS_CX + "Format");

    // The magical "ALL" scene
    var allScene = svc.GetResource("urn:celtx:scene:all");
    ds.Assert(allScene, typeArc, sceneType, true);
    ds.Assert(allScene, ordArc, svc.GetIntLiteral(0), true);
    ds.Assert(allScene, titleArc, svc.GetLiteral("ALL"), true);
    

    // The magical "ALL" department
    var allDept = svc.GetResource("urn:celtx:department:all");
    var markupURL = svc.GetResource(this.deptlist.ref);
    ds.Assert(allDept, typeArc, breakdownType, true);
    // This isn't how the rest are done, it's just a convenient work-around
    // since markupURL is a sequence and sequences don't merge well
    ds.Assert(markupURL, memberArc, allDept, true);
    ds.Assert(allDept, labelArc, svc.GetLiteral("0ALL"), true);

    // The magical "ALL" character
    var allChar = svc.GetResource("urn:celtx:speaker:all");
    ds.Assert(allChar, typeArc, castType, true);
    ds.Assert(allChar, titleArc, svc.GetLiteral("0ALL"), true);

    // The magical "ALL" item
    var allItem = svc.GetResource("urn:celtx:item:all");
    var scenes = svc.GetResource(this.kScenesURL);
    ds.Assert(scenes, memberArc, allItem, true);

    return ds;
  },


  createSceneDS: function createSceneDS (doc) {
    var rdfsvc = getRDFService();
    var ds = getInMemoryDataSource();
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var scenetype = rdfsvc.GetResource(Cx.NS_CX + "Scene");
    var ordarc = rdfsvc.GetResource(Cx.NS_CX + "ordinal");

    var xpath = new XPathEvaluator();
    var xset = xpath.evaluate("/script/scene", doc, null,
                   XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
    var scene = null;
    var i = 1;
    while (scene = xset.iterateNext()) {
      var sceneres = rdfsvc.GetResource(Cx.SCENE_PREFIX +
                                        scene.getAttribute("id"));
      var title = rdfsvc.GetLiteral(scene.getAttribute("title"));
      var ord = rdfsvc.GetIntLiteral(i++);
      ds.Assert(sceneres, typearc, scenetype, true);
      ds.Assert(sceneres, titlearc, title, true);
      ds.Assert(sceneres, ordarc, ord, true);
    }
    return ds;
  },


  createCharacterDS: function createCharacterDS (doc) {
    var rdfsvc = getRDFService();
    var ds = getInMemoryDataSource();
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var casttype = rdfsvc.GetResource(Cx.NS_CX + "Cast");
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");

    var xpath = new XPathEvaluator();
    var xset = xpath.evaluate("/script/scene/speech", doc, null, 0, null);
    var chars = {};
    var e = null;
    while (e = xset.iterateNext()) {
      var name = e.getAttribute("speaker");
      name = name.replace(/\s*\(.*\)\s*$/, "");
      if (name == "" || name in chars) continue;
      chars[name] = 1;
    }

    for (var char in chars) {
      var res = rdfsvc.GetAnonymousResource();
      ds.Assert(res, typearc, casttype, true);
      ds.Assert(res, titlearc, rdfsvc.GetLiteral(char), true);
    }
    return ds;
  },


  cmdSceneSelectionChanged: function cmdSceneSelectionChanged () {
    if (this.selectEventsSuppressed)
      return;
    var svc = getRDFService();
    // Clear the initial selection list
    while (this.scenes.GetCount() > 0)
      this.scenes.RemoveElementAt(this.scenes.GetCount(), true);
    // Repopulate it, checking for the "ALL" special case
    if (isTreeRowSelected(this.scenelist, 0)) {
      this.sceneAllSelected = true;
      // If ALL is selected, make it the only selection
      // scenelist.view.selection.selectEventsSupressed = true;
      this.selectEventsSuppressed = true;
      this.scenelist.view.selection.select(0);
      this.selectEventsSuppressed = false;
      // scenelist.view.selection.selectEventsSupressed = false;
      var count = this.scenelist.view.rowCount;
      for (var i = 1; i < count; ++i) {
        var res = this.scenelist.view.getResourceAtIndex(i);
        this.scenes.AppendElement(res);
      }
    }
    else {
      this.sceneAllSelected = false;
      dump("--- counting selected scenes\n");
      var seliter = new TreeSelectionIterator(this.scenelist);
      while (seliter.hasMore()) {
        var res = this.scenelist.view.getResourceAtIndex(seliter.getNext());
        dump("--- adding scene " + res.Value + "\n");
        this.scenes.AppendElement(res);
      }
    }
    // this.itemlist.builder.rebuild();

    switch (this.outlineView.getSelectedReportIndex()) {
      case 0: // Dialog
      case 1: // Elements
      case 2: // Script with Notes
        this.cmdCreateScriptReport();
        break;
      case 3: // Scene Breakdown
      case 4: // Scene Breakdown with Description
        this.cmdDeptSelectionChanged();
        break;
    }
  },


  cmdDeptSelectionChanged: function cmdDeptSelectionChanged () {
    if (this.selectEventsSuppressed)
      return;
    var svc = getRDFService();
    var member = svc.GetResource(Cx.NS_CX + "member");
    // Check membership assertions, checking for the "ALL" special case
    if (isTreeRowSelected(this.deptlist, 0)) {
      // If ALL is selected, make it the only selection
      this.deptlist.view.selection.selectEventsSupressed = true;
      this.deptlist.view.selection.select(0);
      this.deptlist.view.selection.selectEventsSupressed = false;
      var count = this.deptlist.view.rowCount;
      for (var i = 1; i < count; ++i) {
        var res = this.deptlist.view.getResourceAtIndex(i);
        if (! this.selections.HasAssertion(this.depts, member, res, true))
          this.selections.Assert(this.depts, member, res, true);
      }
    }
    else {
      var count = this.deptlist.view.rowCount;
      for (var i = 1; i < count; i++) {
        var res = this.deptlist.view.getResourceAtIndex(i);
        if (isTreeRowSelected(this.deptlist, i)) {
          if (! this.selections.HasAssertion(this.depts, member, res, true))
            this.selections.Assert(this.depts, member, res, true);
        }
        else {
          if (this.selections.HasAssertion(this.depts, member, res, true))
            this.selections.Unassert(this.depts, member, res);
        }
      }
    }
    // this.itemlist.builder.rebuild();
    this.cmdItemSelectionChanged();
  },


  cmdItemSelectionChanged: function cmdItemSelectionChanged () {
    if (this.selectEventsSuppressed)
      return;
    var svc = getRDFService();
    // Clear the initial selection list
    while (this.items.GetCount() > 0)
      this.items.RemoveElementAt(this.items.GetCount(), true);
    // Repopulate it, counting no selection as "ALL"
    if (this.itemlist.view.selection.count == 0) {
      var count = this.itemlist.view.rowCount;
      for (var i = 0; i < count; ++i) {
        var res = this.itemlist.view.getResourceAtIndex(i);
        this.items.AppendElement(res);
      }
    }
    else {
      var seliter = new TreeSelectionIterator(this.itemlist);
      while (seliter.hasMore()) {
        var res = this.itemlist.view.getResourceAtIndex(seliter.getNext());
        this.items.AppendElement(res);
      }
    }
    try {
      this.cmdCreateBreakdown();
    }
    catch (ex) {
      dump("*** cmdCreateBreakdown: " + ex + "\n");
    }
  },


  cmdCreateScriptReport: function cmdCreateScriptReport () {
    this.reporttitle.value = this.outlineView.getSelectedReportName();
    var xslt = document.implementation.createDocument("", "", null);
    xslt.async = false;
    xslt.load(Cx.TRANSFORM_PATH + "script-report.xml");

    var proc = new XSLTProcessor();
    proc.importStylesheet(xslt);

    // Determine the transformation parameters
    var svc = getRDFService();
    var IRes = Components.interfaces.nsIRDFResource;

    var reportIndex = this.outlineView.getSelectedReportIndex();
    // Scene restrictions
    // Does not apply for Script with Notes
    if (reportIndex != 2 && ! this.sceneAllSelected) {
      var ds = this.project.ds;
      var sceneidarc = svc.GetResource(Cx.NS_CX + "sceneid");
      proc.setParameter(null, "limit-scenes", 1);
      var prefixLength = Cx.SCENE_PREFIX.length;
      var scenes = this.scenes.GetElements();
      var scenelist = "|"
        while (scenes.hasMoreElements()) {
          var scene = scenes.getNext().QueryInterface(IRes);
          var sceneid = getRDFString(ds, scene, sceneidarc);
          if (! sceneid) {
            dump("*** skipping scene res without scene id\n");
            continue;
          }
          scenelist += sceneid + "|";
        }
      proc.setParameter(null, "scenes", scenelist);
    }

    // Element/character restrictions
    var titleArc = svc.GetResource(Cx.NS_DC + "title");
    // Dialog report
    if (reportIndex == 0) {
      proc.setParameter(null, "show-dialog", 1);
      proc.setParameter(null, "show-char", 1);
      proc.setParameter(null, "show-paren", 1);
      if (this.charlist.view.selection.count > 0 &&
          ! isTreeRowSelected(this.charlist, 0)) {
        proc.setParameter(null, "limit-chars", 1);
        var charlist = "|";
        var charview = this.charlist.view;
        var sel = charview.selection;
        for (var i = 0; i < charview.rowCount; ++i) {
          if (sel.isSelected(i)) {
            var res = charview.getResourceAtIndex(i);
            var title = this.charlist.database.GetTarget(res, titleArc, true);
            title = title.QueryInterface(Components.interfaces.nsIRDFLiteral);
            charlist += title.Value + "|";
          }
        }
        proc.setParameter(null, "chars", charlist);
      }
    }
    else {
      var elems = {
        StageDir:       "action",
        ProductionNote: "action",
        Action:         "action",
        Character:      "char",
        Parenthetical:  "paren",
        Dialog:         "dialog",
        Transition:     "trans",
        Shot:           "shot",
        Music:          "music",
        Sound:          "sound",
        Voice:          "voice",
        Text:           "text"
      };
      // Script with Notes
      if (reportIndex != 2 &&
          this.elemlist.view.selection.count > 0 &&
          ! isTreeRowSelected(this.elemlist, 0)) {
        var elemview = this.elemlist.view;
        var sel = elemview.selection;
        var elemcontent = elemview.QueryInterface(
          Components.interfaces.nsITreeContentView);
        var elemrows = elemview.rowCount;
        var prefix = Cx.NS_CX + "Formats/";
        for (var i = 1; i < elemrows; ++i) {
          var elem = elemcontent.getItemAtIndex(i);
          if (sel.isSelected(i)) {
            var res = elem.id.substring(prefix.length);
            if (elems[res])
              proc.setParameter(null, "show-" + elems[res], 1);
          }
        }
      }
      else {
        for (var res in elems) {
          proc.setParameter(null, "show-" + elems[res], 1);
        }
      }
      // Script with Notes
      if (reportIndex == 2)
        proc.setParameter(null, "show-notes", 1);
    }

    var report = proc.transformToDocument(this.scripttree);
    // Prune empty scene headings
    const IHTMLPElem = Components.interfaces.nsIDOMHTMLParagraphElement;
    const IHTMLDivElem = Components.interfaces.nsIDOMHTMLDivElement;
    var para = report.body.firstChild;
    while (para) {
      var nextpara = para.nextSibling;
      while (nextpara && ! (nextpara instanceof IHTMLPElem)) {
        nextpara = nextpara.nextSibling;
      }
      if (para.className == "sceneheading") {
        if (! nextpara || nextpara.className == "sceneheading") {
          // Gather the scene number divs before it too
          var div1 = para.previousSibling;
          var div2 = div1.previousSibling;
          if (div1 instanceof IHTMLDivElem)
            div1.parentNode.removeChild(div1);
          if (div2 instanceof IHTMLDivElem)
            div2.parentNode.removeChild(div2);
          para.parentNode.removeChild(para);
        }
      }
      para = nextpara;
    }
    // var reportFile = tempFile('html');
    serializeDOMtoFile(report, this.reportFile);
    this.browser.webNavigation.loadURI(fileToFileURL(this.reportFile),
      Components.interfaces.nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE,
      null, null, null);
  },


  cmdCreateBreakdown: function cmdCreateBreakdown () {
    this.reporttitle.value = this.outlineView.getSelectedReportName();
    try {
      this.createBreakdownImpl();
    }
    catch (ex) {
      celtxBugAlert(gApp.getText("UnknownErrorMsg"), Components.stack, ex);
    }
  },


  createBreakdownImpl: function createBreakdownImpl () {
    if (this.scenes.GetCount() == 0)
      return;

    const IRes = Components.interfaces.nsIRDFResource;
    const ILit = Components.interfaces.nsIRDFLiteral;

    var svc = getRDFService();
    var cu = getRDFContainerUtils();
    var ds = getCompositeDataSource();
    ds.AddDataSource(this.bdds);
    ds.AddDataSource(this.project.ds);
    // ds.AddDataSource(this.sceneds);
    ds = ds.QueryInterface(Components.interfaces.nsIRDFDataSource);
    var schema = svc.GetDataSourceBlocking(Cx.CONTENT_PATH + "schema.rdf");
    var impl = document.implementation;

    var titleArc = svc.GetResource(Cx.NS_DC + "title");
    var descArc = svc.GetResource(Cx.NS_DC + "description");
    var labelArc = svc.GetResource(Cx.NS_RDFS + "label");
    var typeArc = svc.GetResource(Cx.NS_RDF + "type");
    var ordinalArc = svc.GetResource(Cx.NS_CX + "ordinal");
    var memberArc = svc.GetResource(Cx.NS_CX + "member");
    var membersArc = svc.GetResource(Cx.NS_CX + "members");
    var mediaArc = svc.GetResource(Cx.NS_CX + "media");
    var imgRsrc = svc.GetResource(Cx.NS_CX + "Image");

    // Create the result tree
    var tree = { order: [], scenes: {}, depts: {} };
    // Cache the scene order and details
    var scenes = this.scenes.GetElements();
    while (scenes.hasMoreElements()) {
      var scene = scenes.getNext().QueryInterface(IRes);
      // var sceneOrdinal = ds.GetTarget(scene, ordinalArc, true);
      // sceneOrdinal = sceneOrdinal.QueryInterface(ILit).Value;
      // var sceneTitle = ds.GetTarget(scene, titleArc, true);
      // sceneTitle = sceneTitle.QueryInterface(ILit).Value;
      // var sceneDesc = ds.GetTarget(scene, descArc, true);
      // if (sceneDesc)
      //   sceneDesc = sceneDesc.QueryInterface(ILit).Value;
      // else
      //   sceneDesc = "";
      var sceneOrdinal = getRDFString(ds, scene, ordinalArc);
      var sceneTitle = getRDFString(ds, scene, titleArc);
      var sceneDesc = getRDFString(ds, scene, descArc);
      var members = ds.GetTarget(scene, membersArc, true);
      // Don't include scenes without any breakdown items
      if (members) {
        members = members.QueryInterface(IRes);
        if (cu.IsSeq(ds, members) && ! cu.IsEmpty(ds, members)) {
          tree.scenes[scene.Value] = {
            ordinal:      sceneOrdinal,
            title:        sceneTitle,
            description:  sceneDesc,
            members:      members,
            depts:        {}
          };
          tree.order.push(scene.Value);
        }
      }
    }

    // Cache the department titles
    var depts = this.selections.GetTargets(this.depts, memberArc, true);
    while (depts.hasMoreElements()) {
      var dept = depts.getNext().QueryInterface(IRes);
      var deptTitle = schema.GetTarget(dept, labelArc, true);
      tree.depts[dept.Value] = deptTitle.QueryInterface(ILit).Value;
    }

    // Place the items in the tree
    var usemedia = this.mediachecked;
    var count = this.itemlist.view.rowCount;
    var allSelected = this.itemlist.view.selection.isSelected(0);
    for (var i = 1; i < count; ++i) {
      if (! (allSelected || this.itemlist.view.selection.isSelected(i)))
        continue;
      // Build up the item's description
      var item = this.itemlist.view.getResourceAtIndex(i);
      var type = ds.GetTarget(item, typeArc, true);
      type = type.QueryInterface(IRes).Value;
      var itemTitle = ds.GetTarget(item, titleArc, true);
      if (itemTitle)
        itemTitle = itemTitle.QueryInterface(ILit).Value;
      else
        itemTitle = "";
      if (type == Cx.NS_CX + "Cast")
        itemTitle = itemTitle.toUpperCase();
      var itemDesc = ds.GetTarget(item, descArc, true);
      if (itemDesc)
        itemDesc = itemDesc.QueryInterface(ILit).Value;
      else
        itemDesc = "";
      var itemEntry = {
        title:        itemTitle,
        description:  itemDesc,
        media:        []
      }

      // Build up the item's media list (if necessary)
      if (usemedia) {
        // var media = projds.GetTargets(item, mediaArc, true);
        var media = [];
        var mediaseq = ds.GetTarget(item, mediaArc, true);
        if (mediaseq) {
          try {
            mediaseq = new RDFSeq(ds, mediaseq);
            media = mediaseq.toArray();
          }
          catch (ex) {
            dump("*** createBreakdownImpl: " + ex + "\n");
          }
        }
        // while (media.hasMoreElements()) {
        while (media.length) {
          // var image = media.getNext().QueryInterface(IRes);
          var image = media.shift().QueryInterface(IRes);
          if (ds.HasAssertion(image, typeArc, imgRsrc, true)) {
            var path = this.project.localFileFor(image);
            if (! path) continue;
            var imgName = ds.GetTarget(image, titleArc, true);
            var imgDesc = ds.GetTarget(image, descArc, true);
            if (imgName)
              imgName = imgName.QueryInterface(ILit).Value;
            else
              imgName = path.leafName;
            if (imgDesc)
              imgDesc = imgDesc.QueryInterface(ILit).Value;
            else
              imgDesc = "";
            itemEntry.media.push({
              title: imgName,
              description: imgDesc,
              src: fileToFileURL(path)
            });
          }
        }
      }

      // Include the item in all applicable scenes
      var scene;
      for (scene in tree.scenes) {
        var node = tree.scenes[scene];
        if (! node.members )
          continue;
        if (cu.indexOf(ds, node.members, item) < 0)
          continue;
        if (! (type in node.depts))
          node.depts[type] = [];
        node.depts[type].push(itemEntry);
      }
    }

    // Create the result DOM from the result tree
    var doc = impl.createDocument("", "breakdown", null);
    var root = doc.documentElement;
    var title = this.scripttree.documentElement.getAttribute("title");
    root.setAttribute("title", title);
    root.setAttribute("date", (new Date()).toLocaleDateString());
    for (var i = 0; i < tree.order.length; ++i) {
      var scene = tree.scenes[tree.order[i]];
      var sceneNode = doc.createElement("scene");
      sceneNode.setAttribute("title", scene.title);
      sceneNode.setAttribute("ordinal", scene.ordinal);
      sceneNode.setAttribute("description", scene.description);
      root.appendChild(sceneNode);
      for (var deptindex = 0; deptindex < gDeptOrder.length; ++deptindex) {
        var dept = gDeptOrder[deptindex];
        if (! (dept in scene.depts))
          continue;
        var deptNode = doc.createElement("department");
        deptNode.setAttribute("title", tree.depts[dept]);
        sceneNode.appendChild(deptNode);
        var items = scene.depts[dept];
        for (var j = 0; j < items.length; ++j) {
          var item = items[j];
          var itemNode = doc.createElement("item");
          itemNode.setAttribute("title", item.title);
          itemNode.setAttribute("description", item.description);
          deptNode.appendChild(itemNode);
          for (var k = 0; k < item.media.length; ++k) {
            var media = item.media[k];
            var mediaNode = doc.createElement("image");
            mediaNode.setAttribute("title", media.title);
            mediaNode.setAttribute("description", media.description);
            mediaNode.setAttribute("src", media.src);
            itemNode.appendChild(mediaNode);
          }
        }
      }
    }

    // Perform an XSL transformation on the result DOM
    var xslt = document.implementation.createDocument('', '', null);
    xslt.async = false;
    xslt.load(Cx.TRANSFORM_PATH + "breakdown-report.xml");

    var proc = new XSLTProcessor();
    proc.importStylesheet(xslt);
    // Breakdown with Description
    if (this.outlineView.getSelectedReportIndex() == 4)
      proc.setParameter(null, "show-description", 1);

    if (this.mediachecked)
      proc.setParameter(null, "show-media", 1);

    var report = proc.transformToDocument(doc);
    // var reportFile = tempFile('html');
    serializeDOMtoFile(report, this.reportFile);
    this.browser.webNavigation.loadURI(fileToFileURL(this.reportFile),
      Components.interfaces.nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE,
      null, null, null);
  },


  cmdChangeReport: function cmdChangeReport () {
    switch (this.outlineView.getSelectedReportIndex()) {
      case 0: // Dialog
        this.columnbox.collapsed = false;
        this.columndeck.selectedIndex = 1;
        this.columndeck.flex = 1;
        this.mediacheckbox.collapsed = true;
        this.cmdCreateScriptReport();
        break;
      case 1: // Elements
        this.columnbox.collapsed = false;
        this.columndeck.selectedIndex = 2;
        this.columndeck.flex = 1;
        this.mediacheckbox.collapsed = true;
        this.cmdCreateScriptReport();
        break;
      case 2: // Script with Notes
        this.columnbox.collapsed = true;
        this.mediacheckbox.collapsed = true;
        this.cmdCreateScriptReport();
        break;
      case 3: // Scene Breakdown
      case 4: // Scene Breakdown with Description
        this.columnbox.collapsed = false;
        this.columndeck.selectedIndex = 0;
        this.columndeck.flex = 2;
        this.mediacheckbox.collapsed = false;
        // this.cmdCreateBreakdown();
        this.cmdSceneSelectionChanged();
        break;
    }
  },


  beginScriptTree: function beginScriptTree () {
    var xslt = document.implementation.createDocument("", "", null);
    xslt.async = false;
    xslt.load(Cx.TRANSFORM_PATH + "script-tree.xml");
    this.scriptproc = new XSLTProcessor();
    this.scriptproc.importStylesheet(xslt);
    this.scripttree = document.implementation.createDocument("", "script",
      null);
  },


  updateFragmentInTree: function updateFragmentInTree (frag, idx) {
    // var existing = this.scripttree.getElementById(
    //   frag.firstChild.getAttribute("id"));
    var sceneid = frag.firstChild.getAttribute("id");
    var existing = null;
    var scenes = this.scripttree.documentElement.childNodes;
    for (var i = 0; i < scenes.length; ++i) {
      if (scenes[i].getAttribute("id") == sceneid) {
        existing = scenes[i];
        break;
      }
    }
    if (existing)
      existing.parentNode.removeChild(existing);

    if (scenes.length > idx) {
      this.scripttree.documentElement.insertBefore(frag.firstChild,
        scenes[idx]);
    }
    else {
      this.scripttree.documentElement.appendChild(frag.firstChild);
    }
  },


  updateScriptTree: function updateScriptTree (idx) {
    var IPara = Components.interfaces.nsIDOMHTMLParagraphElement;
    var scene = gScriptController.editor.sceneAt(idx + 1);
    // Check for an Act preceding a scene, since this scene-by-scene parsing
    // was implemented before Acts became so vital to Stageplay
    /*
    var act = previousElement(scene);
    if (act && act instanceof IPara && act.className == "act") {
      var actfrag = this.scriptproc.transformToFragment(act, this.scripttree);
      this.updateFragmentInTree(actfrag, idx);
    }
    */
    var frag = this.scriptproc.transformToFragment(scene, this.scripttree);
    this.updateFragmentInTree(frag, idx);
  },


  endScriptTree: function endScriptTree () {
    // this.scriptproc = null;
    var m = new RDFModel(this.project.ds);
    var docres = RES(this.docres.Value);
    var titleprop = PROP("dc:title");
    var title = m.target(docres, titleprop);
    if (title)
      title = this.project.title + " - " + title.value;
    else
      title = this.project.title;
    this.scripttree.documentElement.setAttribute("title", title);
    this.scripttree.documentElement.setAttribute("scriptmode",
      gScriptController.mode);
    this.scripttree.documentElement.setAttribute("papersize",
      gScriptController.scriptConfig.size);
  },


  beginBreakdownDS: function beginBreakdownDS () {
    var xslt = document.implementation.createDocument("", "", null);
    xslt.async = false;
    xslt.load(Cx.TRANSFORM_PATH + "breakdown-tree.xml");
    this.bdproc = new XSLTProcessor();
    this.bdproc.importStylesheet(xslt);
    // this.bdtree = document.implementation.createDocument("", "script", null);

    this.bdds = getInMemoryDataSource();
    this.populateCharacterMap();
  },


  updateBreakdownDS: function updateBreakdownDS (idx) {
    var scene = gScriptController.editor.sceneAt(idx + 1);
    // var frag = this.bdproc.transformToFragment(scene, this.bdtree);
    // while (frag.hasChildNodes())
    //   this.bdtree.documentElement.appendChild(frag.firstChild);
    var doc = this.bdproc.transformToDocument(scene);

    var sceneid = doc.documentElement.getAttribute("id");
    var sceneres = gScriptController.sceneTracker.sceneForSceneID(sceneid);
    if (! sceneres) {
      dump("*** updateBreakdownDS: No sceneres for scene " + sceneid + "\n");
      return;
    }

    // Get the scene's cx:member sequence, creating if necessary
    var rdfsvc = getRDFService();
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var casttype = rdfsvc.GetResource(Cx.NS_CX + "cast");
    var membersarc = rdfsvc.GetResource(Cx.NS_CX + "members");
    var members = this.bdds.GetTarget(sceneres, membersarc, true);
    if (! members) {
      members = rdfsvc.GetAnonymousResource();
      this.bdds.Assert(sceneres, membersarc, members, true);
    }
    members = new RDFSeq(this.bdds, members);

    // Empty the sequence
    while (members.length > 0)
      members.remove(members.length - 1);

    // Parse the list of items, representing explicit markup
    var seen = {};
    var items = doc.documentElement.getElementsByTagName("item");
    for (var i = 0; i < items.length; ++i) {
      var item = items[i];
      var ref = item.getAttribute("ref");
      if (! seen[ref]) {
        members.push(rdfsvc.GetResource(ref));
        seen[ref] = 1;
      }
      // Update the character map if item.className == "cast" ?
      /*
      if (item.className == "cast") {
        var charname = item.getAttribute("name").toUpperCase();
        charname = charname.replace(/\s*\(.*\)/, "");
        this.resolveExplicitMap[charname] = ref;
      }
      */
    }

    // Parse the list of chars, representing character paragraphs
    var chars = scene.getElementsByTagName("char");
    for (var i = 0; i < chars.length; ++i) {
      var charuri = this.resolveCharacterName(chars[i].getAttribute("name"));
      if (! charuri) {
        // Create a dummy char with just a dc:title and place its
        // description in the breakdown DS.
        var name = chars[i].getAttribute("name").replace(/\s*\(.*\)/, '');
        var tmpchar = rdfsvc.GetAnonymousResource();
        this.bdds.Assert(tmpchar, typearc, casttype, true);
        this.bdds.Assert(tmpchar, titlearc, rdfsvc.GetLiteral(name), true);
        charuri = tmpchar.Value;
        this.resolveMap[name.toUpperCase()] = charuri;
        members.push(tmpchar);
        seen[charuri] = 1;
      }
      if (! seen[charuri]) {
        members.push(rdfsvc.GetResource(charuri));
        seen[charuri] = 1;
      }
    }
  },


  // No-op
  endBreakdownDS: function endBreakdownDS () {
    return;
  },


  resolveMap: {},
  resolveExplicitMap: {},


  populateCharacterMap: function populateCharacterMap () {
    this.resolveMap = {};
    var rdfsvc      = getRDFService();
    var IRes        = Components.interfaces.nsIRDFResource;
    var ILit        = Components.interfaces.nsIRDFLiteral;
    var titleProp   = rdfsvc.GetResource(Cx.NS_DC  + "title");
    var typeProp    = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var classRes    = rdfsvc.GetResource(Cx.NS_CX + "Cast");
    var ds          = this.project.ds;

    var characters = ds.GetSources(typeProp, classRes, true);
    while (characters.hasMoreElements()) {
      try {
        var ref = characters.getNext().QueryInterface(IRes);
        var title = ds.GetTarget(ref, titleProp, true).QueryInterface(ILit);
        this.resolveMap[title.Value.toUpperCase()] = ref.Value;
      }
      catch (ex) {
        dump("*** populateCharacterMap: " + ex + "\n");
      }
    }
  },


  resolveCharacterName: function resolveCharacterName (str) {
    str = str.replace(/\s*\(.*\)/, '');   // Trim parenthesized chunks
    if (str == '') return;
    str = str.toUpperCase();

    // New method: If it isn't explicit, it isn't there.
    return this.resolveMap[str];

    // Old method (not reached)

    // Check the map
    if (this.resolveMap[str]) return this.resolveMap[str];

    var rdfsvc = getRDFService();
    var IRes = Components.interfaces.nsIRDFResource;
    var ILit = Components.interfaces.nsIRDFLiteral;

    // Not in the map, grovel through the model
    var nameProp  = rdfsvc.GetResource(Cx.NS_CX  + "scriptName");
    var titleProp = rdfsvc.GetResource(Cx.NS_DC  + "title");
    var typeProp  = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var classRes  = rdfsvc.GetResource(Cx.NS_CX + "Cast");
    var name      = rdfsvc.GetLiteral(str);

    var ds = this.project.ds;
    var res = ds.GetSource(nameProp, name, true);
    if (! res) {
      // Try to locate via title
      var matches = ds.GetSources(titleProp, name, true);
      while (matches.hasMoreElements()) {
        var match = matches.getNext().QueryInterface(IRes);
        if (ds.HasAssertion(match, typeProp, classRes, true)) {
          res = match;
          ds.Assert(res, nameProp, name, true);
          break;
        }
      }
    }

    if (! res && this.resolveExplicitMap[str]) {
      res = rdfsvc.GetResource(this.resolveExplicitMap[str]);
      ds.Assert(res, nameProp, name, true);
    }

    if (! res) {
      var casts = ds.GetSources(typeProp, classRes, true);
      while (casts.hasMoreElements()) {
        var cast = casts.getNext().QueryInterface(IRes);
        var title = ds.GetTarget(cast, titleProp, true);
        if (title) {
          title = title.QueryInterface(ILit);
          if (str == title.Value.toUpperCase()) {
            res = cast;
            ds.Assert(res, nameProp, name, true);
            break;
          }
        }
        else {
          dump("*** Empty cx:Cast element\n");
        }
      }
    }

    if (! res) {
      // Add them in ourselves
      res = rdfsvc.GetResource(Cx.DOCUMENTS_URL + generateID());
      ds.Assert(res, typeProp, classRes, true);
      ds.Assert(res, titleProp, name, true);
      ds.Assert(res, nameProp, name, true);
    }

    this.resolveMap[str] = res.Value;
    return res.Value;
  }
};


function report_onEnterPrintPreview () {
  gReportController.inPrintPreview = true;
  document.getElementById("reportheader").collapsed = true;
  document.getElementById("columnbox").collapsed = true;
  try {
    var printPreviewTB = document.createElementNS(XUL_NS, "toolbar");
    printPreviewTB.setAttribute("printpreview", true);
    printPreviewTB.setAttribute("id", "print-preview-toolbar");
    getBrowser().parentNode.insertBefore(printPreviewTB, getBrowser());
  }
  catch (ex) {
    dump("*** report_onEnterPrintPreview: " + ex + "\n");
  }
  gController.updateCommands();
  getBrowser().contentWindow.focus();
}


function report_onExitPrintPreview () {
  try {
    var printPreviewTB = document.getElementById("print-preview-toolbar");
    if (printPreviewTB)
      printPreviewTB.parentNode.removeChild(printPreviewTB);
  }
  catch (ex) {
    dump("*** report_onExitPrintPreview: " + ex  + "\n");
  }
  gReportController.inPrintPreview = false;
  document.getElementById("reportheader").collapsed = false;
  document.getElementById("columnbox").collapsed = false;
  gController.updateCommands();
}
