/* ***** BEGIN LICENCE BLOCK *****
 * Version: CePL 1.1
 * 
 * The contents of this file are subject to the Celtx Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.celtx.com/CePL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Celtx Script Manager.
 * 
 * The Initial Developer of the Original Code is Chad House and 4067479
 * Canada Inc. t/a CELTX.
 * 
 * Portions created by Chad House are Copyright (C) 2000-2004 Chad House,
 * parts created by Celtx are Copyright (C) 4067479 Canada Inc. All Rights
 * Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENCE BLOCK ***** */

var gPanel = {
  fileCopier: null,
  deptList: null,
  itemList: null,
  currentItem: null,
  suppressEvents: false,


  QueryInterface: function QueryInterface (iid) {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsIRDFObserver))
      return this;
    throw Components.results.NS_ERROR_NOINTERFACE;
  },


  onAssert: function (ds, src, prop, tgt) {},
  onChange: function (ds, src, prop, oldtgt, newtgt) {},
  onMove: function (ds, oldsrc, newsrc, prop, tgt) {},
  onUnassert: function (ds, src, prop, tgt) {},


  onBeginUpdateBatch: function (ds) {
    if (this.deptList.selectedItem)
      this._cachedDept = this.deptList.selectedItem.id;
    if (this.itemList.selectedItem)
      this._cachedItem = this.itemList.selectedItem.id;

    this.suppressEvents = true;
    this.currentItem = null;
  },


  onEndUpdateBatch: function (ds) {
    setTimeout("gPanel.restoreSelection()", 0);
    this.suppressEvents = false;
  },


  restoreSelection: function restoreSelection () {
    if (this._cachedDept) {
      this.selectDept(this._cachedDept);
      if (this._cachedItem)
        this.selectItem(this._cachedItem);
    }
    this._cachedDept = null;
    this._cachedItem = null;
    if (gController.activeController == gScriptController)
      gScriptController.editor.contentWindow.focus();
  },


  init: function (controller) {
    this.controller = controller;

    this.itemList.database.AddDataSource(controller.project.ds);
    this.itemList.builder.rebuild();

    this.mediaView.database.AddDataSource(controller.project.ds);
    this.updateCommands();

    controller.project.ds.AddObserver(this);
  },


  shutdown: function () {
    this.controller.project.ds.RemoveObserver(this);
    this.itemList.database.RemoveDataSource(this.controller.project.ds);
    this.mediaView.database.AddDataSource(this.controller.project.ds);
  },


  updateCommands: function () {
    goUpdateCommand("cmd-add-media");
    goUpdateCommand("cmd-remove-media");
    goUpdateCommand("cmd-tag-item");
    goUpdateCommand("cmd-delete-item");
    goUpdateCommand("cmd-gis-search");
  },


  cmdDeptSelected: function cmdDeptSelected () {
    if (this.suppressEvents)
      return;
    this.clearItem();
    this.itemList.ref = this.deptList.selectedItem.id;
    this.updateCommands();
  },


  cmdItemSelected: function cmdItemSelected () {
    if (this.suppressEvents)
      return;

    var focused = document.commandDispatcher.focusedElement;

    if (this.currentItem) {
      this.itemName.inputField.blur();
      this.itemDesc.inputField.blur();
    }

    var item = this.itemList.selectedItem;

    if (! item) {
      this.itemName.disabled = true;
      this.itemDesc.disabled = true;
      this.currentItem = null;
    }
    else {
      this.itemName.disabled = false;
      this.itemDesc.disabled = false;
    }
    if (! item || item.id == "new-item") {
      this.itemName.value = item ?
        gScriptController.editor.selection.toString() : "";
      this.itemDesc.value = "";
      this.currentItem = null;
      this.mediaView.ref = "";
    }
    else {
      var rdfsvc = getRDFService();
      var ILit = Components.interfaces.nsIRDFLiteral;
      var titleArc = rdfsvc.GetResource(Cx.NS_DC + "title");
      var descArc = rdfsvc.GetResource(Cx.NS_DC + "description");
      var itemRes = rdfsvc.GetResource(item.id);
      this.currentItem = itemRes;
      this.itemName.value = getRDFString(this.project.ds, itemRes, titleArc);
      this.itemDesc.value = getRDFString(this.project.ds, itemRes, descArc);
      this.mediaView.ref = itemRes.Value;
    }

    this.updateCommands();

    if (focused)
      focused.focus();
  },


  clearItem: function clearItem () {
    // Kludge - blur not always received in time otherwise
    if (this.currentItem) {
      this.itemName.inputField.blur();
      this.itemDesc.inputField.blur();
    }

    this.currentItem = null;
    this.itemList.clearSelection();
  },


  markupSelected: function markupSelected (ref) {
    if (gScriptController.sidebar.collapsed)
      return;

    try {
      // Perhaps should just fire an event here?
      var rdfsvc = getRDFService();
      var refres = rdfsvc.GetResource(ref);
      var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
      var ds = gController.project.ds;
      var type = ds.GetTarget(refres, typearc, true);
      if (! type)
        return;
      type = type.QueryInterface(Components.interfaces.nsIRDFResource);
      if (this.selectDept(type.Value))
        this.selectItem(ref);
      _content.focus();
    }
    catch (ex) {
      dump("*** gPanel.markupSelected: " + ex + "\n");
    }
  },


  selectItem: function selectItem (id) {
    return this.selectListItem(this.itemList, id);
  },


  selectDept: function selectDept (id) {
    var items = this.deptPopup.childNodes;
    for (var i = 0; i < items.length; ++i) {
      if (items[i].id == id) {
        this.deptList.selectedItem = items[i];
        this.cmdDeptSelected();
        return true;
      }
    }
    return false;
    // return this.selectListItem(this.deptList, id);
  },


  selectListItem: function selectListItem (list, id) {
    try {
      var item, el;
      var cnt = list.getRowCount();
      for (var i = 0; i < cnt; i++) {
        el = list.getItemAtIndex(i);
        if (el.id == id) {
          item = el;
          break;
        }
      }
      if (! item) throw "not found " + id;
      list.ensureElementIsVisible(item);
      list.selectItem(item);
      return true;
    }
    catch (ex) {
      dump("*** gPanel.selectListItem: " + ex + "\n");
      return false;
    }
  },


  nameChanged: function namedChanged () {
    if (this.suppressEvents)
      return;

    var itemres = this.currentItem;
    if (! itemres)
      return;
    var rdfsvc = getRDFService();
    var ds = gController.project.ds;
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var title = this.itemName.value;
    setRDFString(ds, itemres, titlearc, title);
    // Find the matching document(s) and rename it/them too
    var typearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var doctype = rdfsvc.GetResource(Cx.NS_CX + "Document");
    var sourcearc = rdfsvc.GetResource(Cx.NS_DC + "source");
    var sources = ds.GetSources(sourcearc, itemres, true);
    while (sources.hasMoreElements()) {
      var source = sources.getNext().QueryInterface(
        Components.interfaces.nsIRDFResource);
      if (ds.HasAssertion(source, typearc, doctype, true))
        setRDFString(ds, source, titlearc, title);
    }
    gController.project.isModified = true;
    this.selectItem(itemres.Value);
  },


  descChanged: function descChanged () {
    if (this.suppressEvents)
      return;

    var itemres = this.currentItem;
    if (! itemres)
      return;
    var rdfsvc = getRDFService();
    var ds = gController.project.ds;
    var descarc = rdfsvc.GetResource(Cx.NS_DC + "description");
    setRDFString(ds, itemres, descarc, this.itemDesc.value);
    this.selectItem(itemres.Value);
  },


  cmdDeleteItem: function cmdDeleteItem () {
    if (! this.currentItem)
      return;
    var rdfsvc = getRDFService();
    var IRes = Components.interfaces.nsIRDFResource;
    var ILit = Components.interfaces.nsIRDFLiteral;
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var label = this.project.ds.GetTarget(this.currentItem, titlearc, true);
    if (label)
      label = label.QueryInterface(ILit).Value;
    else
      label = "";
    var ps = getPromptService();
    var title = gApp.getText("DeleteItem");
    var msg = gApp.getText("DeleteItemPrompt", [ label ]);
    if (! ps.confirm(window, title, msg))
      return;
    var itemRes = this.currentItem;
    var arcs = this.project.ds.ArcLabelsOut(itemRes);
    while (arcs.hasMoreElements()) {
      var arc = arcs.getNext().QueryInterface(IRes);
      var targets = this.project.ds.GetTargets(itemRes, arc, true);
      while (targets.hasMoreElements()) {
        var target = targets.getNext();
        this.project.ds.Unassert(itemRes, arc, target);
      }
    }

    var markupProp = rdfsvc.GetResource(Cx.NS_CX + 'markup');
    var markupSrc  = this.project.ds.GetSource(markupProp, itemRes, true);
    if (markupSrc) {
      this.project.ds.Unassert(markupSrc, markupProp, itemRes, true);
    }

    // Remove all references from the script?
    this.updateCommands();
  },


  cmdAddMedia: function cmdAddMedia () {
    if (! this.currentItem) return;

    try {
      // Get the file
      var fp = getFilePicker();
      fp.init(window, gApp.getText("AddMedia"), fp.modeOpen);
      if (isMac())
        fp.appendFilters(fp.filterAll);
      else
        fp.appendFilters(fp.filterImages | fp.filterAll);
      fp.displayDirectory = getMediaDir();
      if (fp.show() != fp.returnOK) return;

      var file = fp.file;
      setMediaDir(fp.file.parent);
      var mimeType = getMIMEService().getTypeFromFile(file);
      var type = mimeType.split('/').shift();
      var ios = getIOService();
      var url = ios.newFileURI(file);
      url = url.QueryInterface(Components.interfaces.nsIURL);

      // Ensure it's a supported media type
      switch (type) {
        case "image":
        case "audio":
        case "video":
          this.addMediaFile(this.currentItem, url);
          break;
        default:
          window.alert(gApp.getText("UnsupportedMediaMsg"));
          return;
      }
    }
    catch (ex) {
      dump("*** gPanel.addMedia: " + ex + "\n");
    }
  },


  addMediaFile: function addMediaFile (itemres, file) {
    try {
      if (! this.fileCopier)
        this.fileCopier = new FileCopier(gController.project);

      if (file instanceof Components.interfaces.nsIFile) {
        var IFileURL = Components.interfaces.nsIFileURL;
        var ios = getIOService();
        file = ios.newFileURI(file).QueryInterface(IFileURL);
      }
      if (! (file instanceof Components.interfaces.nsIURL))
        throw "addMediaFile requires an nsIURL argument";

      var rv = this.fileCopier.copyToProject(file);

      var m = new RDFModel(gController.project.ds);
      var mediaseq = m.target(RES(itemres.Value), PROP('cx:media'));
      if (mediaseq) {
        mediaseq = m.container(mediaseq);
      }
      else {
        var mediaseqres = RES();
        mediaseq = m.makeSeq(mediaseqres);
        m.add(RES(itemres.Value), PROP('cx:media'), mediaseqres);
      }
      m.add(rv.res, PROP('dc:title'), LIT(file.fileName));
      mediaseq.append(rv.res);
    }
    catch (ex) {
      dump("*** gPanel.addMediaFile: " + ex + "\n");
    }
  },
  
  
  cmdRemoveMedia: function cmdRemoveMedia () {
    var selected = this.mediaView.selected;
    if (! selected || ! this.currentItem)
      return;

    var m = new RDFModel(gController.project.ds);
    var itemres = RES(this.currentItem.Value);
    var mediaseq = m.target(itemres, PROP('cx:media'));
    if (mediaseq) {
      mediaseq = m.container(mediaseq);
      mediaseq.remove(RES(selected.id));
    }
    this.updateCommands();
  },


  cmdGisSearch: function cmdGisSearch () {
    if (! this.currentItem)
      return;

    var rdfsvc = getRDFService();
    var ds = gController.project.ds;
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var title = ds.GetTarget(this.currentItem, titlearc, true);
    if (title)
      title = title.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
    else
      title = "";

    var prefix = 'http://images.google.com/images?q=';
    var suffix = '&btnG=Google+Search';
    var term = encodeURIComponent(title);

    gApp.openBrowser(prefix + term + suffix);
  },


  mediaDoubleClick: function mediaDoubleClick (event) {
    if (! event.target.id || ! this.mediaView.selected) return;

    var itemres = getRDFService().GetResource(event.target.id);
    var file = gController.project.localFileFor(itemres);
    openExternalFile(file);
    /*
    var fileURL = fileToFileURL(file);
    window.openDialog(Cx.CONTENT_PATH + "editors/mediaviewer.xul",
                      "_blank",
                      "chrome,centerscreen,dialog=no",
                      fileURL);
    */
  },
  
  
  mediaItemChanged: function mediaItemChanged (elem) {
    var ds = gController.project.ds;
    var rdfsvc = getRDFService();
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
    var itemres = rdfsvc.GetResource(elem.id);
    var oldtitle = ds.GetTarget(itemres, titlearc, true);
    var newtitle = rdfsvc.GetLiteral(elem.title);
    if (oldtitle)
      ds.Change(itemres, titlearc, oldtitle, newtitle);
    else
      ds.Assert(itemres, titlearc, newtitle, true);
    // this.project.updateResource({ uri: elem.id, title: elem.title });
  },
  
  
  mediaDragOver: function mediaDragOver (event) {
    nsDragAndDrop.dragOver(event, this.mediaDragObserver);
  },
  
  
  mediaDragDrop: function mediaDragDrop (event) {
    if (! this.currentItem)
      return;
    nsDragAndDrop.drop(event, this.mediaDragObserver);
  },
  
  
  mediaDragObserver: {

    flavours: [
      "text/x-moz-url",
      "text/x-moz-url-data",
      "text/unicode",
      "application/x-moz-file",
    ],

    canHandleMultipleItems: true,

    getSupportedFlavours: function () {
      var flavours = new FlavourSet();
      for (var i = 0; i < this.flavours.length; i++)
        flavours.appendFlavour(this.flavours[i]);
      return flavours;
    },

    onDragOver: function (event, flavour, session) { },

    onDrop: function (event, data, session) {
      var urls = [];

      for (var i = 0; i < data.dataList.length; i++) {
        var dataitem = data.dataList[i];
        var flavours = {};
        for (var j = 0; j < dataitem.dataList.length; j++) {
          var flavourData = dataitem.dataList[j];
          if (flavourData.flavour.contentType == "application/x-moz-file") {
            // For some reason, this gets assigned nsISupportsString instead
            // of nsIFile, causing flavourData.data to fail.
            var nsIFile = Components.interfaces.nsIFile;
            var file = flavourData.supports.QueryInterface(nsIFile);
            flavours["application/x-moz-file"] = file;
          }
          else
            flavours[flavourData.flavour.contentType] = flavourData.data;
        }
        if (flavours["text/x-moz-url"]) {
          urls.push(flavours["text/x-moz-url"].split("\n")[0]);
        }
        else if (flavours["text/x-moz-url-data"]) {
          urls.push(flavours["text/x-moz-url-data"]);
        }
        else if (flavours["text/unicode"]) {
          urls.push(flavours["text/unicode"]);
        }
        else if (flavours["application/x-moz-file"]) {
          urls.push(fileToFileURL(flavours["application/x-moz-file"]));
        }
      }

      for (var i = 0 ; i < urls.length; i++) {
        var imgURL = urls[i];
        if (imgURL.match(/^http/)) {
          // Check for GIS searches
          var gis;
          if (gis = imgURL.match(/imgurl=([^&]+)/)) {
            imgURL = unescape(gis[1]);
          }
        }

        if (! imgURL) {
          dump("Couldn't match " + imgURL + " to anything meaningful.\n");
          return;
        }

        try {
          var ios = getIOService();
          var url = ios.newURI(imgURL, null, null);
          gPanel.addMediaFile(gPanel.currentItem, url);
        }
        catch (ex) {
          dump("panel.mediaDragObserver.onDrop error: " + ex + "\n");
        }
      }
    }
  }
};

