function getController () {
  return gController;
}

function loaded () {
}

function setOutlineView (outline) {
  gController.outline = outline;
}

var gController = {
  startSeq: 1,
  startRow: 1,
  modified: false,


  get selection () {
    return this._selection;
  },


  set selection (val) {
    this._selection = val;
    if (val) {
      var node = this.shotForCell(val);
      var row = this.treeview.nodeToRow(node);
      if (row >= 0) {
        this.treeview.selection.select(row);
      }
      else {
        this.treeview.selection.clearSelection();
      }
    }
    else {
      this.treeview.selection.clearSelection();
    }
    return val;
  },


  supportsCommand: function (cmd) {
    switch (cmd) {
      case "cmd-page-setup":
      case "cmd-print":
        return true;
      default:
        return false;
    }
  },
  isCommandEnabled: function (cmd) {
    return this.supportsCommand(cmd);
  },
  doCommand: function (cmd) {
    switch (cmd) {
      case "cmd-page-setup":
        PrintUtils.showPageSetup();
        break;
      case "cmd-print":
        this.print();
        break;
    }
  },
  open: function open (project, docres) {
    this.project = project;
    this.docres = docres;
    var file = this.project.fileForResource(this.docres);
    if (isReadableFile(file)) {
      var req = new XMLHttpRequest();
      req.open("GET", fileToFileURL(file), false);
      req.send(null);
      this.storydom = req.responseXML;
    }
    else {
      var impl = document.implementation;
      this.storydom = impl.createDocument("", "storyboard", null);
      var seq = this.storydom.createElement("sequence");
      seq.setAttribute("title", gApp.getText("Untitled"));
      this.storydom.documentElement.appendChild(seq);
      this.modified = true;
    }

    this.checkMediaResources();
    this.resolveImageAttributes();

    // Add IDs to each sequence if there are none
    var seqs = this.storydom.getElementsByTagName("sequence");
    for (var i = 0; i < seqs.length; ++i) {
      if (! seqs[i].id)
        seqs[i].id = generateID();
    }

    this.treeview = new StoryboardTree(this);
    var navtree = this.outline.document.getElementById("navtree");
    navtree.view = this.treeview;

    try {
      this.refresh();
    }
    catch (ex) {
      dump("*** storyboard.open: " + ex + "\n");
    }
    var title = project.model.target(RES(docres.Value), PROP('dc:title'));
    this.storydom.documentElement.setAttribute("title", title.value);

    var boardbox = document.getElementById("boardbox");
    boardbox.addEventListener("editlabel-change", this, false);
  },
  close: function close () {},
  save: function save () {
    var file = this.project.fileForResource(this.docres);
    if (! file) {
      file = this.project.projectFolder;
      file.append("storyboard.xml");
      file.createUnique(0, 0644 & file.parent.permissions);
      this.project.addFileToDocument(file, this.docres);
    }
    serializeDOMtoFile(this.storydom, file);
    this.modified = false;
  },
  focus: function focus () {
    document.getElementById("printframe")
      .setAttribute("type", "content-primary");
  },
  blur: function blur () {
    document.getElementById("printframe")
      .setAttribute("type", "content");
  },


  print: function print () {
    var config = {
      accepted: false,
      orientation: "portrait"
    };
    openDialog(Cx.CONTENT_PATH + "editors/storyboardprint.xul", "_blank",
      Cx.MODAL_DIALOG_FLAGS, config);
    if (! config.accepted)
      return;

    var xsl = document.implementation.createDocument("", "", null);
    xsl.async = false;
    xsl.load(Cx.TRANSFORM_PATH + "storyboard-print.xml");

    var proc = new XSLTProcessor();
    proc.importStylesheet(xsl);
    // proc.setParameter(null, "fileprefix",
    //   fileToFileURL(this.project.projectFolder));
    proc.setParameter(null, "orientation", config.orientation);

    var doc = proc.transformToDocument(this.storydom);
    var tmpfile = tempFile('html');
    serializeDOMtoFile(doc, tmpfile);
    document.getElementById("printframe").setAttribute("src",
      fileToFileURL(tmpfile));
    setTimeout("gController.tryToPrint()", 100);
  },


  tryToPrint: function tryToPrint () {
    var frame = document.getElementById("printframe");
    if (frame.docShell.busyFlags) {
      setTimeout("gController.tryToPrint()", 100);
      return;
    }

    var settings = PrintUtils.getPrintSettings();
    var flags = settings.kInitSaveNativeData | settings.kInitSaveOrientation |
      settings.kInitSaveHeaderRight;
    var oldorient = settings.orientation;
    var oldheader = settings.headerStrRight;
    if (frame.contentDocument.body.getAttribute("orientation")
        == "landscape")
      settings.orientation = settings.kLandscapeOrientation;
    else
      settings.orientation = settings.kPortraitOrientation;
    settings.headerStrRight = "&P";
    var pssvc = getPrintSettingsService();
    pssvc.savePrintSettingsToPrefs(settings, true, flags);
    PrintUtils.print();
    settings.orientation = oldorient;
    settings.headerStrRight = oldheader;
    pssvc.savePrintSettingsToPrefs(settings, true, flags);
  },


  handleEvent: function handleEvent (event) {
    if (event.type != "editlabel-change")
      return;
    if (event.target.parentNode.getAttribute("class") == "seqheader") {
      var header = event.target;
      var str = "/storyboard/sequence[position() = "
        + header.getAttribute("sequence") + "]";
      var xpath = new XPathEvaluator();
      var xset = xpath.evaluate(str, this.storydom, null,
        XPathResult.FIRST_ORDERED_NODE_TYPE, null);
      if (xset.singleNodeValue) {
        xset.singleNodeValue.setAttribute("title", header.value);
        this.treeview.handleEvent(
          { type: "DOMAttrModified", target: xset.singleNodeValue }
        );
      }
      else {
        dump("*** handleEvent: Couldn't find sequence for edit label\n");
        return;
      }
    }
    else {
      var cell = event.target;
      while (cell && cell.localName != "shotcell")
        cell = cell.parentNode;
      if (! cell) {
        dump("** Couldn't locate cell for editlabel-change event.\n");
        return;
      }
      var shot = this.shotForCell(cell);
      shot.setAttribute("title", event.target.value);
      this.treeview.handleEvent(
        { type: "DOMAttrModified", target: shot }
      );
    }
    this.modified = true;
  },


  createImageResForFile: function createImageResForFile (file) {
    var rdfsvc = getRDFService();
    var rdftypearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var localfilearc = rdfsvc.GetResource(Cx.NS_CX + "localFile");
    var imagesarc = rdfsvc.GetResource(Cx.NS_CX + "media");
    var imagetype = rdfsvc.GetResource(Cx.NS_CX + "Image");
    var ds = this.project.ds;

    var imageres = rdfsvc.GetResource(this.project.mintURI());
    ds.Assert(imageres, rdftypearc, imagetype, true);
    setRDFString(ds, imageres, localfilearc, file.leafName);
    if (this.images.indexOf(imageres) < 0)
      this.images.push(imageres);

    return imageres;
  },


  checkMediaResources: function checkMediaResources () {
    var rdfsvc = getRDFService();
    var rdftypearc = rdfsvc.GetResource(Cx.NS_RDF + "type");
    var localfilearc = rdfsvc.GetResource(Cx.NS_CX + "localFile");
    var imagesarc = rdfsvc.GetResource(Cx.NS_CX + "media");
    var imagetype = rdfsvc.GetResource(Cx.NS_CX + "Image");
    var ds = this.project.ds;

    // Ensure the cx:images sequence off the docres
    var imageseq = ds.GetTarget(this.docres, imagesarc, true);
    if (! imageseq) {
      imageseq = rdfsvc.GetAnonymousResource();
      ds.Assert(this.docres, imagesarc, imageseq, true);
    }
    this.images = new RDFSeq(ds, imageseq);

    // Ensure an imageres attribute wherever it's lacking, and make sure
    // the imageres is the correct one for its position (this will be
    // incorrect after a copy)
    var shots = this.storydom.documentElement.getElementsByTagName("shot");
    for (var i = 0; i < shots.length; ++i) {
      if (shots[i].hasAttribute("imageres") &&
          shots[i].hasAttribute("imageno")) {
        try {
          var imageres = this.images.get(
            Number(shots[i].getAttribute("imageno"))).QueryInterface(
            Components.interfaces.nsIRDFResource);
          if (shots[i].getAttribute("imageres") != imageres.Value)
            shots[i].setAttribute("imageres", imageres.Value);
        }
        catch (ex) {
          dump("*** checkMediaResources: " + ex + "\n");
        }
        continue;
      }
      var imageres = rdfsvc.GetResource(this.project.mintURI());
      ds.Assert(imageres, rdftypearc, imagetype, true);
      var leafname = shots[i].getAttribute("image").replace(/.*\//, "");
      setRDFString(ds, imageres, localfilearc, leafname);
      shots[i].setAttribute("imageres", imageres.Value);
      if (this.images.indexOf(imageres) < 0)
        this.images.push(imageres);
      shots[i].setAttribute("imageno", this.images.indexOf(imageres));
    }
  },


  resolveImageAttributes: function resolveImageAttributes () {
    var rdfsvc = getRDFService();
    var shots = this.storydom.documentElement.getElementsByTagName("shot");
    for (var i = 0; i < shots.length; ++i) {
      if (! shots[i].hasAttribute("imageres")) {
        dump("*** resolveImageAttributes: missing imageres on shot\n");
        continue;
      }
      var imageres = shots[i].getAttribute("imageres");
      shots[i].setAttribute("image", this.resolveImageToURL(imageres));
    }
  },


  resolveImageToURL: function resolveImageToURL (imageresuri) {
    var rdfsvc = getRDFService();
    var localFileArc = rdfsvc.GetResource(Cx.NS_CX + "localFile");
    var imageres = rdfsvc.GetResource(imageresuri);
    var image = getRDFString(this.project.ds, imageres, localFileArc);
    if (! image) {
      dump("*** no localFile for " + imageresuri + "\n");
      return null;
    }
    try {
      var imagefile = this.project.projectFolder;
      imagefile.append(image);
      return fileToFileURL(imagefile);
    }
    catch (ex) {
      dump("*** resolveImageToURL: " + ex + "\n    image: " + image + "\n");
      return null;
    }
  },


  refresh: function refresh () {
    this.selection = null;
    var xslt = document.implementation.createDocument("", "", null);
    xslt.async = false;
    xslt.load(Cx.TRANSFORM_PATH + "storyboard.xml");
    var proc = new XSLTProcessor();
    proc.importStylesheet(xslt);
    proc.setParameter(null, "seqoffset", this.startSeq);
    proc.setParameter(null, "seqrow", this.startRow);
    // var prefix = fileToFileURL(this.project.projectFolder);
    // proc.setParameter(null, "fileprefix", prefix);
    var frag = proc.transformToFragment(this.storydom, document);
    var boardbox = document.getElementById("boardbox");
    while (boardbox.hasChildNodes())
      boardbox.removeChild(boardbox.lastChild);
    boardbox.appendChild(frag);
  },


  scrollUp: function scrollUp (rows) {
    this.startRow = this.startRow - rows;
    if (this.startRow < 1) {
      if (this.startSeq > 1) {
        this.startSeq -= 1;
        // TODO: Calculate actual underflow
        var seqs = this.storydom.documentElement
          .getElementsByTagName("sequence");
        var seq = seqs[this.startSeq - 1];
        var rows = Math.ceil(seq.childNodes.length / 3);
        if (rows == 0)
          this.startRow = 1;
        else
          this.startRow = rows;
      }
      else {
        this.startRow = 1;
      }
    }
    this.refresh();
  },


  scrollDown: function scrollDown (rows) {
    var seqs = this.storydom.documentElement.getElementsByTagName("sequence");
    var seq = seqs[this.startSeq - 1];
    var seqrows = Math.ceil(seq.childNodes.length / 3);
    if (seqrows == 0)
      seqrows == 1;
    this.startRow += rows;
    if (this.startRow > seqrows) {
      if (this.startSeq < this.storydom.documentElement.childNodes.length) {
        this.startSeq += 1;
        // Don't stop at the first row if scrolling should go further
        var overflow = this.startRow - seqrows;
        var seq = this.storydom.documentElement.childNodes[this.startSeq - 1];
        seqrows = Math.ceil(seq.childNodes.length / 3);
        this.startRow = (overflow > seqrows) ? 1 : overflow;
      }
      else {
        this.startRow = seqrows;
      }
    }
    this.refresh();
  },


  scrollUpSequence: function scrollUpSequence () {
    if (this.startRow > 1) {
      this.startRow = 1;
    }
    else if (this.startSeq > 1) {
      this.startSeq -= 1;
    }
    else
      return;
    this.refresh();
  },


  scrollDownSequence: function scrollDownSequence () {
    var seqs = this.storydom.documentElement.getElementsByTagName("sequence");
    if (this.startSeq < seqs.length) {
      this.startSeq += 1;
      this.startRow = 1;
    }
    else {
      var seq = seqs[this.startSeq - 1];
      var seqrows = Math.ceil(seq.childNodes.length / 3);
      if (this.startRow < seqrows)
        this.startRow = seqrows;
      else
        return;
    }
    this.refresh();
  },


  // Maps view to model parent sequence
  sequenceForCell: function sequenceForCell (cell) {
    var seqnum = cell.getAttribute("sequence");
    var str = "/storyboard/sequence[position() = " + seqnum + "]";
    var xpath = new XPathEvaluator();
    var xset = xpath.evaluate(str, this.storydom, null,
      XPathResult.FIRST_ORDERED_NODE_TYPE, null);
    return xset.singleNodeValue;
  },


  // Maps view to model
  shotForCell: function shotForCell (cell) {
    var seqnum = cell.getAttribute("sequence");
    var shotnum = cell.getAttribute("position");
    var str = "/storyboard/sequence[position() = " + seqnum
      + "]/shot[position() = " + shotnum + "]";
    var xpath = new XPathEvaluator();
    var xset = xpath.evaluate(str, this.storydom, null,
      XPathResult.FIRST_ORDERED_NODE_TYPE, null);
    return xset.singleNodeValue;
  },


  deleteCell: function deleteCell (cell) {
    var shot = this.shotForCell(cell);
    if (! shot)
      return;
    this.treeview.handleEvent(
      { type: "DOMNodeRemoved", target: shot }
    );
    shot.parentNode.removeChild(shot);
    this.treeview.handleEvent(
      { type: "DOMNodeWasRemoved", target: shot }
    );
    this.modified = true;
    this.refresh();
  },


  addSequence: function addSequence () {
    var seq = this.storydom.createElement("sequence");
    seq.id = generateID();
    seq.setAttribute("title", gApp.getText("Untitled"));
    this.storydom.documentElement.appendChild(seq);
    var seqs = this.storydom.getElementsByTagName("sequence");
    if (seqs.length == 1) {
      this.startSeq = 1;
      this.startRow = 1;
    }
    else {
      var prevseq = seqs[seqs.length - 2];
      var shots = prevseq.getElementsByTagName("shot");
      var rows = Math.ceil(shots.length / 3);
      this.startSeq = seqs.length - 1;
      this.startRow = rows;
    }
    this.treeview.handleEvent(
      { type: "DOMNodeInserted", target: seq }
    );
    this.modified = true;
    this.refresh();
  },


  deleteSequence: function deleteSequence (seqnum) {
    var str = "/storyboard/sequence[position() = " + seqnum + "]";
    var xpath = new XPathEvaluator();
    var xset = xpath.evaluate(str, this.storydom, null,
      XPathResult.FIRST_ORDERED_NODE_TYPE, null);
    var seq = xset.singleNodeValue;
    var ps = getPromptService();
    if (! ps.confirm(window, gApp.getText("DeleteSequence"),
      gApp.getText("DeleteSequencePrompt", [ seq.getAttribute("title") ])))
      return;
    this.treeview.handleEvent(
      { type: "DOMNodeRemoved", target: seq }
    );
    seq.parentNode.removeChild(seq);
    this.treeview.handleEvent(
      { type: "DOMNodeWasRemoved", target: seq }
    );
    this.modified = true;
    this.refresh();
  },


  slideshow: function slideshow () {
    openDialog(Cx.CONTENT_PATH + "editors/storyboardplay.xul", "_blank",
      Cx.NEW_WINDOW_FLAGS, this.storydom, this.project);
  },


  keypress: function keypress (event) {
    if (! this.selection)
      return;
    if (event.keyCode != event.DOM_VK_BACK_SPACE &&
        event.keyCode != event.DOM_VK_DELETE)
      return;

    var shot = this.shotForCell(this.selection);
    if (! shot)
      return;
    this.treeview.handleEvent(
      { type: "DOMNodeRemoved", target: shot }
    );
    shot.parentNode.removeChild(shot);
    this.treeview.handleEvent(
      { type: "DOMNodeWasRemoved", target: shot }
    );
    this.modified = true;
    this.refresh();
  },


  click: function click (event) {
    if (event.target.localName == "shotcell" &&
        event.originalTarget.localName == "label") {
      this.selection = event.target;
      event.target.editlabel.setEditable(true);
      return;
    }
    if (event.target.localName != "shotcell" ||
        this.selection == event.target)
      return;
    if (this.selection)
      this.selection.removeAttribute("selected");
    this.selection = event.target;
    this.selection.setAttribute("selected", true);
    if (event.originalTarget.localName != "description")
      event.target.focus();
  },


  doubleClick: function doubleClick (event) {
    if (event.target.localName == "shotholder") {
      var fp = getFilePicker();
      var mediadir = getMediaDir();
      fp.init(window, null, fp.modeOpenMultiple);
      fp.appendFilters(fp.filterImages);
      fp.displayDirectory = getMediaDir();

      if (fp.show() != fp.returnOK)
        return;

      var files = fp.files;
      while (files.hasMoreElements()) {
        var file = files.getNext().QueryInterface(
          Components.interfaces.nsIFile);

        setMediaDir(file.parent);

        // var copier = new FileCopier(this.project);
        // var IFileURL = Components.interfaces.nsIFileURL;
        // var url = getIOService().newFileURI(file).QueryInterface(IFileURL);
        // var result = copier.copyToProject(url);
        var result = {
          file: copyToUnique(file, this.project.projectFolder, file.leafName)
        };

        var imageres = this.createImageResForFile(result.file);

        var shot = this.storydom.createElement("shot");
        shot.setAttribute("ratio", "4x3");
        shot.setAttribute("title", file.leafName);
        shot.setAttribute("image", fileToFileURL(result.file));
        shot.setAttribute("imageres", imageres.Value);
        shot.setAttribute("imageno", this.images.indexOf(imageres));
        var seq = this.sequenceForCell(event.target);
        seq.appendChild(shot);
        this.modified = true;

        this.treeview.handleEvent(
          { type: "DOMNodeInserted", target: shot }
        );
      }

      // Check if we need to scroll down to keep the placeholder visible
      var boardbox = document.getElementById("boardbox");
      try {
        var rows = boardbox.lastChild.lastChild;
        if (rows.childNodes.length >= 3 &&
            event.target == rows.lastChild.lastChild) {
          this.scrollDown(1);
        }
        else {
          this.refresh();
        }
      }
      catch (ex) {
        dump("*** doubleClick: " + ex + "\n");
        this.refresh();
      }

      var cells = boardbox.getElementsByTagName("shotcell");
      for (var i = cells.length - 1; i >= 0; --i) {
        if (this.shotForCell(cells[i]) == shot) {
          cells[i].editlabel.setEditable(true);
          break;
        }
      }
    }
    else if (event.target.localName == "shotcell" &&
             (event.originalTarget.localName == "image" ||
              event.originalTarget.getAttribute("class") == "shotoverlay")) {
      openExternalFile(fileURLToFile(event.target.getAttribute("image")));
    }
  }
};


var gDragController = {
  // Modified version of nsDragAndDrop.startDrag with additional |dragrect|
  // field for specifying the drag rect in the |action| parameter
  startDrag: function (aEvent) {
    const kDSIID = Components.interfaces.nsIDragService;
    var dragAction = {
      action: kDSIID.DRAGDROP_ACTION_COPY |
              kDSIID.DRAGDROP_ACTION_MOVE |
              kDSIID.DRAGDROP_ACTION_LINK,
      dragrect: null
    };
    var transferData = { data: null };
    try {
      this.onDragStart(aEvent, transferData, dragAction);
    }
    catch (ex) { return; }
    if (! transferData.data) return;
    transferData = transferData.data;

    var transArray = createSupportsArray();
    var count = 0;
    do {
      var trans = nsTransferable.set(transferData._XferID == "TransferData"
        ? transferData : transferData.dataList[count++]);
      transArray.AppendElement(trans.QueryInterface(
        Components.interfaces.nsISupports));
    }
    while (transferData.XferID == "TransferDataSet" &&
           count < transferData.dataList.length);
    try {
      nsDragAndDrop.mDragService.invokeDragSession(aEvent.target, transArray,
        dragAction.dragrect, dragAction.action);
    }
    catch (ex) {}
    aEvent.stopPropagation();
  },


  onDragStart: function (event, xferData, action) {
    var target = event.target;
    if (target.localName != "shotcell")
      return;

    xferData.data = new TransferData();
    xferData.data.addDataForFlavour("text/unicode",
      target.getAttribute("image"));

    var region = Components.classes["@mozilla.org/gfx/region;1"]
      .createInstance(Components.interfaces.nsIScriptableRegion);
    region.init();
    var bo = target.boxObject;
    region.setToRect(bo.x, bo.y, bo.width, bo.height);
    action.dragrect = region;
  },


  onDragOver: function (event, flavour, session) {
    if (event.target.localName != "shotcell" &&
        event.target.localName != "shotholder")
      return;

    try {
      var seq = gController.sequenceForCell(event.target);
      var shot = gController.shotForCell(session.sourceNode);
      var before = null;
      if (event.target.localName == "shotcell") {
        var before = gController.shotForCell(event.target);
        var box = event.target.boxObject;
        if (event.screenX > (box.screenX + box.width / 2))
          before = nextElement(before);
      }
      if (before && before == gController.shotForCell(event.target)) {
        event.target.setAttribute("dropbefore", "true");
        event.target.removeAttribute("dropafter", "true");
      }
      else {
        event.target.removeAttribute("dropbefore", "true");
        event.target.setAttribute("dropafter", "true");
      }
    }
    catch (ex) {
      dump("*** onDrop: " + ex + "\n");
    }
  },


  onDragExit: function (event, session) {
    if (event.target.hasAttribute("dropbefore"))
      event.target.removeAttribute("dropbefore");
  },


  onDrop: function (event, xferData, session) {
    // Only allow local drops
    if (! (session.sourceNode && session.sourceNode.localName == "shotcell")) {
      dump("*** no sourceNode (or non-shotcell sourceNode)\n");
      return;
    }

    if (event.target.localName != "shotcell" &&
        event.target.localName != "shotholder")
      return;

    try {
      var seq = gController.sequenceForCell(event.target);
      var shot = gController.shotForCell(session.sourceNode);
      var before = null;
      if (event.target.localName == "shotcell") {
        var before = gController.shotForCell(event.target);
        var box = event.target.boxObject;
        if (event.screenX > (box.screenX + box.width / 2))
          before = nextElement(before);
      }
      gController.treeview.handleEvent(
        { type: "DOMNodeMoving", target: shot }
      );
      if (before)
        seq.insertBefore(shot, before);
      else
        seq.appendChild(shot);
      gController.treeview.handleEvent(
        { type: "DOMNodeMoved", target: shot }
      );
      gController.modified = true;
      gController.refresh();
      gController.treeview.treebox.invalidate();
    }
    catch (ex) {
      dump("*** onDrop: " + ex + "\n");
    }
  },


  getSupportedFlavours: function () {
    if (! this._flavours) {
      this._flavours = new FlavourSet();
      // this._flavours.appendFlavour("x-celtx/x-image");
      this._flavours.appendFlavour("text/unicode");
    }
    return this._flavours;
  }
};

function onDeleteCell (event) {
  var cell = event.target;
  while (cell && cell.localName != "shotcell")
    cell = cell.parentNode;
  if (! cell) {
    dump("*** onDeleteCell: no shotcell in hierarchy\n");
    return;
  }
  gController.deleteCell(cell);
}


function StoryboardTree (controller) {
  this.controller = controller;
  this.dragsvc = Components.classes["@mozilla.org/widget/dragservice;1"]
    .getService(Components.interfaces.nsIDragService);
}


StoryboardTree.prototype = {
  QueryInterface: function (iid) {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsIDOMEventListener) ||
        iid.equals(Components.interfaces.nsITreeView))
      return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },


  toggleMap: {},


  DROP_BEFORE: Components.interfaces.nsITreeView.DROP_BEFORE,
  DROP_ON: Components.interfaces.nsITreeView.DROP_ON,
  DROP_AFTER: Components.interfaces.nsITreeView.DROP_AFTER,
  PROGRESS_NORMAL: Components.interfaces.nsITreeView.PROGRESS_NORMAL,
  PROGRESS_UNDETERMINED:
    Components.interfaces.nsITreeView.PROGRESS_UNDETERMINED,
  PROGRESS_NONE: Components.interfaces.nsITreeView.PROGRESS_NONE,


  handleEvent: function (event) {
    switch (event.type) {
      case "DOMAttrModified":
        var row = this.nodeToRow(event.target);
        if (row >= 0) {
          // this.treebox.invalidateRow(row);
          this.treebox.invalidate();
        }
        break;
      case "DOMNodeInserted":
        var row = this.nodeToRow(event.target);
        if (row >= 0)
          this.treebox.rowCountChanged(row, 1);
        this.treebox.invalidate();
        break;
      case "DOMNodeRemoved":
        var row = this.nodeToRow(event.target);
        if (row >= 0) {
          var count = 1;
          if (event.target.localName == "sequence" &&
              this.toggleMap[event.target.id])
            count += event.target.getElementsByTagName("shot");
          this._deleteRow = row;
          this._deleteCount = -count;
        }
        else {
          this._deleteRow = -1;
        }
        break;
      case "DOMNodeWasRemoved":
        if (this._deleteRow >= 0) {
          this.treebox.rowCountChanged(this._deleteRow, this._deleteCount);
          this.treebox.invalidate();
        }
        break;
      case "DOMNodeMoving":
        this._movingParent = event.target.parentNode;
        this._movingRow = this.nodeToRow(event.target);
        break;
      case "DOMNodeMoved":
        if (this._movingParent != event.target.parentNode) {
          // Change the row count if it moved from an open sequence
          // to a closed one or vice versa.
          if (this.toggleMap[this._movingParent.id] !=
              this.toggleMap[event.target.parentNode.id]) {
            if (this.toggleMap[this._movingParent.id])
              this.treebox.rowCountChanged(this._movingRow, -1);
            else
              this.treebox.rowCountChanged(this.nodeToRow(event.target), 1);
          }
        }
        this.treebox.invalidate();
        break;
      case "dblclick":
        if (event.button != 0)
          return;
        if (event.target.localName != "treechildren")
          return;
        if (this.selection.count == 0)
          return;
        var node = this.rowToNode(this.selection.currentIndex);
        if (! node) {
          dump("*** couldn't locate node for row "
            + this.selection.currentIndex + "\n");
          return;
        }
        var seq = node.localName == "sequence" ? node : node.parentNode;
        var seqs = this.controller.storydom.getElementsByTagName("sequence");
        for (var i = 0; i < seqs.length; ++i) {
          if (seq == seqs[i]) {
            this.controller.startSeq = i + 1;
            break;
          }
        }
        if (node.localName == "shot") {
          var shots = seq.getElementsByTagName("shot");
          for (var i = 0; i < shots.length; ++i) {
            if (node == shots[i]) {
              this.controller.startRow = Math.floor(i / 3) + 1;
              this.controller.refresh();
              break;
            }
          }
        }
        else {
          this.controller.startRow = 1;
          this.controller.refresh();
        }
        break;
      case "draggesture":
        if (event.originalTarget.localName == "treechildren")
          nsDragAndDrop.startDrag(event, this);
        break;
    }
  },


  rowToNode: function (row) {
    var seqs = this.controller.storydom.getElementsByTagName("sequence");
    var remaining = row;
    for (var i = 0; i < seqs.length; ++i) {
      if (remaining < 0)
        return null;

      if (remaining == 0)
        return seqs[i];

      --remaining;

      if (this.toggleMap[seqs[i].id]) {
        var shots = seqs[i].getElementsByTagName("shot");
        if (remaining < shots.length)
          return shots[remaining];
        remaining -= shots.length;
      }
    }
    return null;
  },


  nodeToRow: function (node) {
    var seq = node;
    if (node.localName == "shot")
      seq = node.parentNode;
    var seqs = this.controller.storydom.getElementsByTagName("sequence");
    var row = 0;
    for (var i = 0; i < seqs.length; ++i) {
      if (seq == seqs[i])
        break;
      row += 1;
      if (this.toggleMap[seqs[i].id])
        row += seqs[i].getElementsByTagName("shot").length;
    }
    if (seq == node)
      return row;
    if (! this.toggleMap[seq.id])
      return -1;
    var shots = seq.getElementsByTagName("shot");
    for (var i = 0; i < shots.length; ++i) {
      row += 1;
      if (node == shots[i])
        return row;
    }
    return -1;
  },


  get rowCount () {
    var dom = this.controller.storydom;
    var seqs = dom.getElementsByTagName("sequence");
    var count = 0;
    for (var i = 0; i < seqs.length; ++i) {
      ++count;
      if (this.toggleMap[seqs[i].id])
        count += seqs[i].getElementsByTagName("shot").length;
    }
    return count;
  },


  getCellText: function (row,column) {
    var node = this.rowToNode(row);
    if (! node)
      return "";
    var title = node.getAttribute("title");
    var cursor = node;
    var count = 1;
    while (cursor = previousElement(cursor))
      ++count;
    title = count + ". " + title;
    if (node.localName == "shot") {
      count = 1;
      cursor = node.parentNode;
      while (cursor = previousElement(cursor))
        ++count;
      title = count + "." + title;
    }
    return title;
  },


  setTree: function (treebox) {
    this.treebox = treebox;
    treebox.treeBody.addEventListener("dblclick", this, false);
    treebox.treeBody.addEventListener("draggesture", this, false);
  },


  isContainer: function (row) {
    var node = this.rowToNode(row);
    if (node && node.localName == "sequence")
      return true;
    return false;
  },


  isContainerOpen: function (row) {
    var node = this.rowToNode(row);
    if (node && node.localName == "sequence") {
      return this.toggleMap[node.id] == true;
    }
    return false;
  },


  isContainerEmpty: function (row) {
    var node = this.rowToNode(row);
    if (node && node.localName == "sequence") {
      return node.getElementsByTagName("shot").length == 0;
    }
    return true;
  },


  getLevel: function (row) {
    return this.isContainer(row) ? 0 : 1;
  },


  getParentIndex: function (row) {
    var node = this.rowToNode(row);
    if (! node || node.localName == "sequence")
      return -1;
    var index = row - 1;
    while (node = previousElement(node))
      --index;
    return index;
  },


  hasNextSibling: function (row, afterIndex) {
    var node = this.rowToNode(row);
    if (! node)
      return false;
    var sibling = this.rowToNode(afterIndex + 1);
    if (! sibling)
      return false;
    return node.parentNode == sibling.parentNode;
  },


  toggleOpenState: function (row) {
    var node = this.rowToNode(row);
    if (! node || node.localName != "sequence") {
      dump("*** toggleOpenState: invalid node\n");
      return;
    }
    var rows = node.getElementsByTagName("shot").length;
    if (this.toggleMap[node.id]) {
      this.toggleMap[node.id] = false;
      this.treebox.rowCountChanged(row + 1, -rows);
    }
    else {
      this.toggleMap[node.id] = true;
      this.treebox.rowCountChanged(row + 1, rows);
    }
    this.treebox.invalidate();
  },


  isSeparator: function (row) { return false; },
  isSorted: function () { return false; },
  getImageSrc: function (row,col) { return null; },
  getRowProperties: function (row,props) {},
  getCellProperties: function (row,col,props) {},
  getColumnProperties: function (colid,col,props) {},


  onDragStart: function onDragStart (aEvent, aXferData, aDragAction) {
    if (this.selection.count != 1)
      return;
    var data = new TransferData();
    data.addDataForFlavour("x-celtx/x-treerow", this.selection.currentIndex);
    aXferData.data = data;
  },


  canDrop: function (row, orientation) {
    var dragSession = this.dragsvc.getCurrentSession();
    if (! dragSession)
      return false;

    var dstnode = this.rowToNode(row);
    if (! dstnode)
      return false;

    if (dragSession.numDropItems != 1 ||
        ! dragSession.isDataFlavorSupported("x-celtx/x-treerow"))
      return false;

    var trans = Components.classes["@mozilla.org/widget/transferable;1"]
      .createInstance(Components.interfaces.nsITransferable);
    trans.addDataFlavor("x-celtx/x-treerow");
    dragSession.getData(trans, 0);
    var data = {};
    var len = {};
    trans.getTransferData("x-celtx/x-treerow", data, len);
    data = data.value.QueryInterface(Components.interfaces.nsISupportsString);
    if (! data)
      return false;
    data = data.data.substring(0, len.value);

    var srcnode = this.rowToNode(data);
    if (! srcnode)
      return false;

    if (srcnode == dstnode)
      return false;
    // Moving a whole sequence?
    if (srcnode.localName == "sequence") {
      return dstnode.localName == "sequence" && orientation != this.DROP_ON;
    }
    // Moving just a shot
    else {
      if (dstnode.localName == "sequence") {
        return dstnode != srcnode.parentNode && orientation == this.DROP_ON;
      }
      else
        return orientation != this.DROP_ON;
    }
  },


  drop: function (row, orientation) {
    var dragSession = this.dragsvc.getCurrentSession();
    if (! dragSession)
      return;

    var dstnode = this.rowToNode(row);
    if (! dstnode)
      return;

    if (dragSession.numDropItems != 1 ||
        ! dragSession.isDataFlavorSupported("x-celtx/x-treerow"))
      return;

    var trans = Components.classes["@mozilla.org/widget/transferable;1"]
      .createInstance(Components.interfaces.nsITransferable);
    trans.addDataFlavor("x-celtx/x-treerow");
    dragSession.getData(trans, 0);
    var data = {};
    var len = {};
    trans.getTransferData("x-celtx/x-treerow", data, len);
    data = data.value.QueryInterface(Components.interfaces.nsISupportsString);
    if (! data)
      return;
    data = data.data.substring(0, len.value);

    var srcnode = this.rowToNode(data);
    if (! srcnode)
      return;

    if (srcnode == dstnode)
      return;
    // Moving a whole sequence?
    if (srcnode.localName == "sequence") {
      if (dstnode.localName == "sequence" && orientation != this.DROP_ON) {
        if (orientation == this.DROP_BEFORE)
          dstnode.parentNode.insertBefore(srcnode, dstnode);
        else {
          var next = nextElement(dstnode);
          if (next)
            dstnode.parentNode.insertBefore(srcnode, next);
          else
            dstnode.parentNode.appendChild(srcnode);
        }
      }
    }
    // Moving just a shot
    else {
      if (dstnode.localName == "sequence") {
        if (dstnode != srcnode.parentNode && orientation == this.DROP_ON) {
          dstnode.appendChild(srcnode);
        }
      }
      else if (orientation != this.DROP_ON) {
        if (orientation == this.DROP_BEFORE) {
          dstnode.parentNode.insertBefore(srcnode, dstnode);
        }
        else {
          var next = nextElement(dstnode);
          if (next)
            dstnode.parentNode.insertBefore(srcnode, next);
          else
            dstnode.parentNode.appendChild(srcnode);
        }
      }
    }

    this.treebox.invalidate();
    this.controller.refresh();
  }
};
